@@ -25,6 +25,7 @@
 #include <Foundation/Foundation.h>
 #include <AppKit/AppKit.h>
 #include <math.h>
+#include <unistd.h>
 #include "FSNBrowser.h"
 #include "FSNBrowserColumn.h"
 #include "FSNBrowserMatrix.h"
@@ -163,7 +164,7 @@
   ASSIGN (baseNode, [FSNode nodeWithPath: [node path]]);
   [self readNodeInfo];
   [self loadColumnZero];
-  [self notifySelectionChange: [NSArray arrayWithObject: [node path]]];
+  [self notifySelectionChange: [NSArray arrayWithObject: node]];
 }
 
 - (void)setUsesCellsIcons:(BOOL)cicns
@@ -298,7 +299,7 @@
   if (selection && [selection count]) {
     FSNode *node = [selection objectAtIndex: 0];
     FSNBrowserColumn *bc;
-    NSArray *selPaths;
+    NSArray *selNodes;
     
     updateViewsLock++;
     
@@ -342,11 +343,11 @@
     updateViewsLock--;
     [self tile];
 
-    selPaths = [bc selectedPaths];
-    if (selPaths == nil) {
-      selPaths = [NSArray arrayWithObject: [[bc shownNode] path]];
+    selNodes = [bc selectedNodes];
+    if (selNodes == nil) {
+      selNodes = [NSArray arrayWithObject: [bc shownNode]];
     }
-    [self notifySelectionChange: selPaths];
+    [self notifySelectionChange: selNodes];
   }
 }
 
@@ -355,7 +356,7 @@
   if (selpaths && [selpaths count]) {
     FSNode *node = [FSNode nodeWithPath: [selpaths objectAtIndex: 0]];      
     FSNBrowserColumn *bc;
-    NSArray *selPaths;
+    NSArray *selNodes;
 
     updateViewsLock++;
 
@@ -397,11 +398,11 @@
     updateViewsLock--;
     [self tile];
 
-    selPaths = [bc selectedPaths];
-    if (selPaths == nil) {
-      selPaths = [NSArray arrayWithObject: [[bc shownNode] path]];
+    selNodes = [bc selectedNodes];
+    if (selNodes == nil) {
+      selNodes = [NSArray arrayWithObject: [bc shownNode]];
     }
-    [self notifySelectionChange: selPaths];
+    [self notifySelectionChange: selNodes];
   }
 }
 
@@ -547,6 +548,7 @@
 
 - (void)reloadFromColumn:(FSNBrowserColumn *)col
 {
+  CREATE_AUTORELEASE_POOL(arp);
 	int index = [col index];
   int i = 0;
 
@@ -600,7 +602,7 @@
   col = [self lastLoadedColumn];
 
   if (col) {
-    NSArray *selection = [col selectedPaths];
+    NSArray *selection = [col selectedNodes];
     int index = [col index];
 
     if (index < firstVisibleColumn) {
@@ -612,7 +614,7 @@
     if (selection) {
       if (selColumn && (index == lastColumnLoaded)) {
         if ([selection count] == 1) {
-          FSNode *node = [FSNode nodeWithPath: [selection objectAtIndex: 0]];
+          FSNode *node = [selection objectAtIndex: 0];
         
           if (([node isDirectory] == NO) || [node isPackage]) {
             [self addFillingColumn];
@@ -627,12 +629,13 @@
     
     } else {
       FSNode *node = [col shownNode];
-      [self notifySelectionChange: [NSArray arrayWithObject: [node path]]];
+      [self notifySelectionChange: [NSArray arrayWithObject: node]];
     }
   }
 
   updateViewsLock--;
   [self tile];
+  RELEASE (arp);
 }
 
 - (void)reloadFromColumnWithNode:(FSNode *)anode
@@ -666,7 +669,7 @@
 
   if (updateViewsLock == 0) {
     NSWindow *window = [self window];
-    NSRect r = [self frame];
+    NSRect r = [self bounds];
     float frameWidth = r.size.width - visibleColumns;
     int count = [columns count];
     NSRect colrect;
@@ -715,6 +718,7 @@
 - (void)scrollViaScroller:(NSScroller *)sender
 {
   NSScrollerPart hit = [sender hitPart];
+  BOOL needsDisplay = NO;
   
   updateViewsLock++;
   skipUpdateScroller = YES;
@@ -734,6 +738,7 @@
 		case NSScrollerIncrementLine:
 		case NSScrollerIncrementPage:
       [self scrollColumnsRightBy: 1];
+      needsDisplay = YES;
 			break;
       
 		// The knob or knob slot
@@ -749,6 +754,8 @@
           [self setLastColumn: (lastColumnLoaded - currentshift)];			
 		      currentshift = 0;
         }
+        
+        needsDisplay = YES;
 			}
       break;
       
@@ -760,6 +767,7 @@
   
   updateViewsLock--;
   [self tile];
+  [self setNeedsDisplay: needsDisplay];
 } 
 
 - (void)updateScroller
@@ -881,7 +889,7 @@
 	} else {
     NSMatrix *matrix = [selCol cmatrix];
     
-    if (matrix) {
+    if ([[matrix cells] count]) {
       int index = [selCol index];
       
       [matrix sendAction];
@@ -889,7 +897,8 @@
       if (index < ([columns count] - 1)) {
         selCol = [columns objectAtIndex: index + 1];
         matrix = [selCol cmatrix];
-        if (matrix) {
+        
+        if ([[matrix cells] count]) {
           if ([selCol selectFirstCell]) {
             [matrix sendAction];  
             [[self window] makeFirstResponder: matrix];
@@ -930,7 +939,12 @@
 - (NSString *)pathToLastColumn
 {
   FSNode *node = [self nodeOfLastColumn];
-  return node ? [node path] : nil;
+
+  if (node) {
+    return [node path];
+  }
+  
+  return nil;
 }
 
 - (NSArray *)selectionInColumnBeforeColumn:(FSNBrowserColumn *)col
@@ -1120,7 +1134,7 @@
   NSArray *selection = [col selectedNodes];
   
   if ((selection == nil) || ([selection count] == 0)) {
-    [self notifySelectionChange: [NSArray arrayWithObject: [[col shownNode] path]]];
+    [self notifySelectionChange: [NSArray arrayWithObject: [col shownNode]]];
     return;
   }
 
@@ -1176,7 +1190,7 @@
   updateViewsLock--;
   [self tile];
   
-  [self notifySelectionChange: [col selectedPaths]];		  
+  [self notifySelectionChange: [col selectedNodes]];		  
 }
 
 - (void)doubleClickInMatrixOfColumn:(FSNBrowserColumn *)col
@@ -1334,7 +1348,7 @@
     matrix = [selCol cmatrix];
 	}
 	
-  if (matrix) {
+  if ([[matrix cells] count]) {
     [[self window] makeFirstResponder: matrix];
 	}
 	
@@ -1632,9 +1646,25 @@
       [self reloadFromColumnWithPath: s]; 
     }
     
-  } else if ([event isEqual: @"GWFileDeletedInWatchedDirectory"]) {
-    [self removeCellsWithNames: [info objectForKey: @"files"]
-              inColumnWithPath: path];
+  } else if ([event isEqual: @"GWFileDeletedInWatchedDirectory"]) {    
+    if ([self isShowingPath: path]) {
+      FSNBrowserColumn *col;
+    
+      [self reloadFromColumnWithPath: path]; 
+       
+      col = [self lastLoadedColumn];
+
+      if (col) {
+        NSArray *selection = [col selectedNodes];
+        
+        if (selection == nil) {
+          selection = [NSArray arrayWithObject: [col shownNode]];
+        }
+        
+        [viewer selectionChanged: selection];
+        [self synchronizeViewer];
+      }  
+    }      
                  
   } else if ([event isEqual: @"GWFileCreatedInWatchedDirectory"]) {   
     [self addCellsWithNames: [info objectForKey: @"files"]
@@ -1812,7 +1842,7 @@
   
   if (bc) {
     [[bc cmatrix] deselectAllCells];
-    [self notifySelectionChange: [NSArray arrayWithObject: [[bc shownNode] path]]];
+    [self notifySelectionChange: [NSArray arrayWithObject: [bc shownNode]]];
   }
 }
 
@@ -1977,7 +2007,7 @@
 - (void)restoreLastSelection
 {
   if (lastSelection) {
-    [self selectRepsOfPaths: lastSelection];
+    [self selectRepsOfSubnodes: lastSelection];
   }
 }
 
@@ -1993,12 +2023,12 @@
     if (prev) {
       if ([prev selectCellOfNode: anode sendAction: YES] == nil) {
         [self setLastColumn: [prev index]];
-        [self notifySelectionChange: [NSArray arrayWithObject: [[prev shownNode] path]]];
+        [self notifySelectionChange: [NSArray arrayWithObject: [prev shownNode]]];
       }
     } else {
       [self setLastColumn: 0];
       [bc unselectAllCells];
-      [self notifySelectionChange: [NSArray arrayWithObject: [baseNode path]]];
+      [self notifySelectionChange: [NSArray arrayWithObject: baseNode]];
     }
     
     updateViewsLock--;
@@ -2132,7 +2162,6 @@
     [nameEditor setNode: nil stringValue: @"" index: -1];
     [nameEditor removeFromSuperview];
     [self setNeedsDisplayInRect: [nameEditor frame]];
-    [[NSCursor arrowCursor] set];
   }
 }
 
@@ -2200,9 +2229,9 @@
       NSArray *dirContents = [ednode subNodeNamesOfParent];
       NSMutableDictionary *opinfo = [NSMutableDictionary dictionary];
 
-      if (range.length > 0) {
+      if (([newname length] == 0) || (range.length > 0)) {
         NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
-                  NSLocalizedString(@"Invalid char in name", @""), 
+                  NSLocalizedString(@"Invalid name", @""), 
                             NSLocalizedString(@"Continue", @""), nil, nil);   
         CLEAREDITING;
       }	
