@@ -272,7 +272,7 @@ static NSImage *branchImage;
 
 - (void)tile
 {
-  NSRect frameRect = [self frame];
+  NSRect frameRect = [self bounds];
   NSSize sz = [icon size];
   int lblmargin = [fsnodeRep labelMargin];
   BOOL hasinfo = ([[infolabel stringValue] length] > 0);
@@ -1301,6 +1301,26 @@ static NSImage *branchImage;
     prePath = [prePath stringByDeletingLastPathComponent];
   }
 
+  if ([node isDirectory] && [node isParentOfPath: fromPath]) {
+    NSArray *subNodes = [node subNodes];
+    
+    for (i = 0; i < [subNodes count]; i++) {
+      FSNode *nd = [subNodes objectAtIndex: i];
+      
+      if ([nd isDirectory]) {
+        int j;
+        
+        for (j = 0; j < count; j++) {
+          NSString *fname = [[sourcePaths objectAtIndex: j] lastPathComponent];
+          
+          if ([[nd name] isEqual: fname]) {
+            return NSDragOperationNone;
+          }
+        }
+      }
+    }
+  }	
+
   if ([node isApplication]) {
     if (([container respondsToSelector: @selector(baseNode)] == NO)
                         || ([node isEqual: [container baseNode]] == NO)) { 
@@ -1576,6 +1596,7 @@ static NSImage *branchImage;
       [self setAlignment: NSLeftTextAlignment];  
       [self setSelectable: YES];  
       [self setEditable: YES];  
+      [[self window] makeFirstResponder: self]; 
     }
     
   } else {
