@@ -1667,10 +1667,10 @@ pp.y = NSMaxY(br) + 1; \
 - (void)selectionDidChange
 {
 	if (!(selectionMask & FSNCreatingSelectionMask)) {
-    NSArray *selection = [self selectedPaths];
+    NSArray *selection = [self selectedNodes];
 		
     if ([selection count] == 0) {
-      selection = [NSArray arrayWithObject: [node path]];
+      selection = [NSArray arrayWithObject: node];
     }
 
     if ((lastSelection == nil) || ([selection isEqual: lastSelection] == NO)) {
@@ -1709,7 +1709,7 @@ pp.y = NSMaxY(br) + 1; \
 - (void)restoreLastSelection
 {
   if (lastSelection) {
-    [self selectRepsOfPaths: lastSelection];
+    [self selectRepsOfSubnodes: lastSelection];
   }
 }
 
@@ -1869,6 +1869,27 @@ pp.y = NSMaxY(br) + 1; \
     prePath = [prePath stringByDeletingLastPathComponent];
   }
 
+  if ([node isDirectory] && [node isParentOfPath: basePath]) {
+    NSArray *subNodes = [node subNodes];
+    int i;
+    
+    for (i = 0; i < [subNodes count]; i++) {
+      FSNode *nd = [subNodes objectAtIndex: i];
+      
+      if ([nd isDirectory]) {
+        int j;
+        
+        for (j = 0; j < count; j++) {
+          NSString *fname = [[sourcePaths objectAtIndex: j] lastPathComponent];
+          
+          if ([[nd name] isEqual: fname]) {
+            return NSDragOperationNone;
+          }
+        }
+      }
+    }
+  }	
+
   isDragTarget = YES;	
   forceCopy = NO;
     
@@ -2056,7 +2077,7 @@ pp.y = NSMaxY(br) + 1; \
   [self stopRepNameEditing];
   
   if (lastSelection && ([lastSelection count] == 1)) {
-    editIcon = [self repOfSubnodePath: [lastSelection objectAtIndex: 0]]; 
+    editIcon = [self repOfSubnode: [lastSelection objectAtIndex: 0]]; 
   } 
   
   if (editIcon) {
@@ -2141,7 +2162,6 @@ pp.y = NSMaxY(br) + 1; \
     [nameEditor setNode: nil stringValue: @"" index: -1];
     [nameEditor removeFromSuperview];
     [self setNeedsDisplayInRect: edrect];
-    [[NSCursor arrowCursor] set];
   }
 
   for (i = 0; i < [icons count]; i++) {
@@ -2255,9 +2275,9 @@ pp.y = NSMaxY(br) + 1; \
       NSArray *dirContents = [ednode subNodeNamesOfParent];
       NSMutableDictionary *opinfo = [NSMutableDictionary dictionary];
 
-      if (range.length > 0) {
+      if (([newname length] == 0) || (range.length > 0)) {
         NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
-                  NSLocalizedString(@"Invalid char in name", @""), 
+                  NSLocalizedString(@"Invalid name", @""), 
                             NSLocalizedString(@"Continue", @""), nil, nil);   
         CLEAREDITING;
       }	
