@@ -25,6 +25,7 @@
 #include <Foundation/Foundation.h>
 #include <AppKit/AppKit.h>
 #include <math.h>
+#include <unistd.h>
 #include "FSNListView.h"
 #include "FSNTextCell.h"
 #include "FSNFunctions.h"
@@ -115,9 +116,8 @@ static NSString *defaultColumns = @"{ \
 
     nameEditor = [FSNListViewNameEditor new];
     [nameEditor setDelegate: self];  
-    [nameEditor setEditable: YES];
-    [nameEditor setSelectable: YES];	   
-//	  [nameEditor setFont: [cellPrototype font]];
+    [nameEditor setEditable: NO];
+    [nameEditor setSelectable: NO];	   
 		[nameEditor setBezeled: NO];
 		[nameEditor setAlignment: NSLeftTextAlignment];
     
@@ -1319,10 +1319,10 @@ static NSString *defaultColumns = @"{ \
 
 - (void)selectionDidChange
 {
-  NSArray *selection = [self selectedPaths];
+  NSArray *selection = [self selectedNodes];
 
   if ([selection count] == 0) {
-    selection = [NSArray arrayWithObject: [node path]];
+    selection = [NSArray arrayWithObject: node];
   }
 
   if ((lastSelection == nil) || ([selection isEqual: lastSelection] == NO)) {
@@ -1398,14 +1398,13 @@ static NSString *defaultColumns = @"{ \
 
 - (void)stopRepNameEditing
 {
-  if (nameEditor && [[listView subviews] containsObject: nameEditor]) {
+  if ([[listView subviews] containsObject: nameEditor]) {
     [nameEditor abortEditing];
     [nameEditor setEditable: NO];
     [nameEditor setSelectable: NO];
     [nameEditor setNode: nil stringValue: @"" index: -1];
     [nameEditor removeFromSuperview];
     [listView setNeedsDisplayInRect: [nameEditor frame]];
-    [[NSCursor arrowCursor] set];
   }
 }
 
@@ -1416,13 +1415,13 @@ static NSString *defaultColumns = @"{ \
 
 - (void)setEditorAtRow:(int)row
 {
+  [self stopRepNameEditing];
+
   if ([[listView selectedRowIndexes] count] == 1) {
     FSNListViewNodeRep *rep = [nodeReps objectAtIndex: row];  
     FSNode *nd = [rep node];
     BOOL canedit = (([rep isLocked] == NO) && ([nd isMountPoint] == NO));
     
-    [self stopRepNameEditing];
-  
     if (canedit) {   
       NSNumber *num = [NSNumber numberWithInt: FSNInfoNameType];
       unsigned col = [listView columnWithIdentifier: num];
@@ -1441,8 +1440,6 @@ static NSString *defaultColumns = @"{ \
 
       [nameEditor setNode: nd stringValue: [nd name] index: 0];
 
-      [nameEditor setEditable: YES];
-      [nameEditor setSelectable: YES];	
       [listView addSubview: nameEditor];
     }
   }
@@ -1507,9 +1504,9 @@ static NSString *defaultColumns = @"{ \
       NSArray *dirContents = [ednode subNodeNamesOfParent];
       NSMutableDictionary *opinfo = [NSMutableDictionary dictionary];
 
-      if (range.length > 0) {
+      if (([newname length] == 0) || (range.length > 0)) {
         NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
-                  NSLocalizedString(@"Invalid char in name", @""), 
+                  NSLocalizedString(@"Invalid name", @""), 
                             NSLocalizedString(@"Continue", @""), nil, nil);   
         CLEAREDITING;
       }	
@@ -1683,6 +1680,27 @@ static NSString *defaultColumns = @"{ \
       prePath = [prePath stringByDeletingLastPathComponent];
     }
 
+    if ([node isDirectory] && [node isParentOfPath: basePath]) {
+      NSArray *subNodes = [node subNodes];
+      int i;
+
+      for (i = 0; i < [subNodes count]; i++) {
+        FSNode *nd = [subNodes objectAtIndex: i];
+
+        if ([nd isDirectory]) {
+          int j;
+
+          for (j = 0; j < count; j++) {
+            NSString *fname = [[sourcePaths objectAtIndex: j] lastPathComponent];
+
+            if ([[nd name] isEqual: fname]) {
+              return NSDragOperationNone;
+            }
+          }
+        }
+      }
+    }	
+
     isDragTarget = YES;	
     forceCopy = NO;
 
@@ -2260,6 +2278,27 @@ static NSString *defaultColumns = @"{ \
     prePath = [prePath stringByDeletingLastPathComponent];
   }
 
+  if ([node isDirectory] && [node isParentOfPath: fromPath]) {
+    NSArray *subNodes = [node subNodes];
+    int i;
+    
+    for (i = 0; i < [subNodes count]; i++) {
+      FSNode *nd = [subNodes objectAtIndex: i];
+      
+      if ([nd isDirectory]) {
+        int j;
+        
+        for (j = 0; j < count; j++) {
+          NSString *fname = [[sourcePaths objectAtIndex: j] lastPathComponent];
+          
+          if ([[nd name] isEqual: fname]) {
+            return NSDragOperationNone;
+          }
+        }
+      }
+    }
+  }	
+
   isDragTarget = YES;
   forceCopy = NO;
 
@@ -2349,6 +2388,14 @@ static NSString *defaultColumns = @"{ \
 
 @implementation FSNListViewNameEditor
 
+int sortSubviews(id view1, id view2, void *context)
+{
+  if ([view1 isMemberOfClass: [FSNListViewNameEditor class]]) {
+    return NSOrderedAscending;
+  }  
+  return NSOrderedDescending;
+}
+
 - (void)dealloc
 {
   TEST_RELEASE (node);
@@ -2377,12 +2424,20 @@ static NSString *defaultColumns = @"{ \
   return index;
 }
 
-- (void)mouseDown:(NSEvent*)theEvent
+- (void)mouseDown:(NSEvent *)theEvent
 {
-  if ([self isEditable]) {
+  NSView *view = [self superview];
+  
+  if ([self isEditable] == NO) {
+    [self setSelectable: YES];  
+    [self setEditable: YES];       
     [[self window] makeFirstResponder: self];
+  } else {  
+    [super mouseDown: theEvent];
   }
-  [super mouseDown: theEvent];
+
+  [view sortSubviewsUsingFunction: (int (*)(id, id, void *))sortSubviews context: nil];  
+  [view setNeedsDisplayInRect: [self frame]];
 }
 
 @end
@@ -2419,9 +2474,6 @@ static NSString *defaultColumns = @"{ \
     [self setTarget: dsource];
     [self setDoubleAction: @selector(doubleClickOnListView:)];
   
-    editstamp = 0.0;
-    editindex = -1;
-
 		lastKeyPressed = 0.;
     charBuffer = nil;
   
@@ -2438,12 +2490,12 @@ static NSString *defaultColumns = @"{ \
 - (void)checkSize
 {
   id sview = [self superview];
-  if (sview && ([self frame].size.width < [sview frame].size.width)) {
+  if (sview && ([self bounds].size.width < [sview bounds].size.width)) {
     [self sizeLastColumnToFit];  
   }
 }
 
-- (void)mouseDown:(NSEvent*)theEvent
+- (void)mouseDown:(NSEvent *)theEvent
 {
   NSPoint location;
   int clickCount;
@@ -2458,12 +2510,10 @@ static NSString *defaultColumns = @"{ \
   clickCount = [theEvent clickCount];
   
   if (clickCount >= 2) {
-    editindex = -1;
     return;
   }
 
   if ([theEvent modifierFlags] & NSShiftKeyMask) {
-    editindex = -1;
     return;
   }
   
@@ -2471,20 +2521,8 @@ static NSString *defaultColumns = @"{ \
   location = [self convertPoint: location fromView: nil];
   row = [self rowAtPoint: location];
   
-  if (row != -1) {
-    if (editindex != row) {
-      editindex = row;
-            
-    } else {
-      NSTimeInterval interval = ([theEvent timestamp] - editstamp);
-          
-      if ((interval > DOUBLE_CLICK_LIMIT)
-                                && (interval < EDIT_CLICK_LIMIT)) {
-        [dsource setEditorAtRow: row];
-      }
-    }
-    
-    editstamp = [theEvent timestamp];   
+  if (row != -1) {  
+    [dsource setEditorAtRow: row];
   }
 }
 
@@ -2502,6 +2540,7 @@ static NSString *defaultColumns = @"{ \
 
   switch (character) {
     case NSPageUpFunctionKey:
+      [dsource stopRepNameEditing];
 		  vRect = [self visibleRect];
 		  p = vRect.origin;
 		  x = p.x;
@@ -2513,6 +2552,7 @@ static NSString *defaultColumns = @"{ \
 	    return;
 
     case NSPageDownFunctionKey:
+      [dsource stopRepNameEditing];
 		  vRect = [self visibleRect];
 		  p = vRect.origin;    
 		  x = p.x;
@@ -2524,10 +2564,12 @@ static NSString *defaultColumns = @"{ \
 	    return;
 
     case NSUpArrowFunctionKey:
+      [dsource stopRepNameEditing];
 	    [dsource selectRepInPrevRow];
       return;
 
     case NSDownArrowFunctionKey:
+      [dsource stopRepNameEditing];
 	    [dsource selectRepInNextRow];
       return;
   
@@ -2861,3 +2903,5 @@ static NSString *defaultColumns = @"{ \
 }
 
 @end
+
+
