@@ -116,12 +116,11 @@
     
     fileType = nil;    
     typeDescription = nil;
+    
+    application = nil;
                                       
     attributes = [fm fileAttributesAtPath: path traverseLink: NO];
-                                              
-    if (attributes) {
-      RETAIN (attributes);
-    }
+    TEST_RETAIN (attributes);
   }
     
   return self;
@@ -182,9 +181,13 @@
   CREATE_AUTORELEASE_POOL(arp);
   NSMutableArray *nodes = [NSMutableArray array];
   NSArray *fnames = [fsnodeRep directoryContentsAtPath: [self parentPath]];
-  FSNode *pnd = (parent != nil) ? [parent parent] : nil;
+  FSNode *pnd = nil;
   int i;
   
+  if (parent != nil) {
+    pnd = [parent parent];
+  }
+  
   for (i = 0; i < [fnames count]; i++) {
     NSString *fname = [fnames objectAtIndex: i];
     FSNode *node = [[FSNode alloc] initWithRelativePath: fname parent: pnd];
@@ -257,7 +260,7 @@
     RETAIN (components);
     RELEASE (arp);
     
-    return [components makeImmutableCopyOnFail: NO];
+    return [[components autorelease] makeImmutableCopyOnFail: NO];
     
   } else if ([secondNode isEqual: firstNode]) {
     return [NSArray arrayWithObject: firstNode];
@@ -438,7 +441,15 @@
   if (attributes && (fileType == nil)) {
     ASSIGN (fileType, [attributes fileType]);
   }
-  return (fileType ? fileType : [NSString string]);
+  return (fileType ? fileType : (NSString *)[NSString string]);
+}
+
+- (NSString *)application
+{
+  if ([self isApplication] == NO) {
+    return application;
+  }
+  return nil;
 }
 
 - (void)setTypeFlags
@@ -466,7 +477,11 @@
 	    NSString *defApp = nil, *type = nil;
 
 	    [ws getInfoForFile: path application: &defApp type: &type]; 
-
+      
+      if (defApp) {
+        ASSIGN (application, defApp);
+      }
+      
       flags.directory = 1;
 
 	    if (type == NSApplicationFileType) {
@@ -511,7 +526,11 @@
 	  NSString *defApp = nil, *type = nil;
 
 	  [ws getInfoForFile: path application: &defApp type: &type]; 
-
+      
+    if (defApp) {
+      ASSIGN (application, defApp);
+    }
+    
     flags.directory = 1;
 
 	  if (type == NSApplicationFileType) {
@@ -577,7 +596,7 @@
   if (attributes && (crDate == nil)) {
     ASSIGN (crDate, [attributes fileCreationDate]);
   }
-  return (crDate ? crDate : [NSDate date]);
+  return (crDate ? crDate : (NSDate *)[NSDate date]);
 }
 
 - (NSString *)crDateDescription
@@ -603,7 +622,7 @@
   if (attributes && (modDate == nil)) {
     ASSIGN (modDate, [attributes fileModificationDate]);
   }
-  return (modDate ? modDate : [NSDate date]);
+  return (modDate ? modDate : (NSDate *)[NSDate date]);
 }
 
 - (NSString *)modDateDescription
@@ -670,7 +689,7 @@
   if (attributes && (owner == nil)) {
     ASSIGN (owner, [attributes fileOwnerAccountName]);
   }
-  return (owner ? owner : [NSString string]);
+  return (owner ? owner : (NSString *)[NSString string]);
 }
 
 - (NSNumber *)ownerId
@@ -686,7 +705,7 @@
   if (attributes && (group == nil)) {
     ASSIGN (group, [attributes fileGroupOwnerAccountName]);
   }
-  return (group ? group : [NSString string]);
+  return (group ? group : (NSString *)[NSString string]);
 }
 
 - (NSNumber *)groupId
@@ -831,12 +850,16 @@
 
 - (BOOL)isValid
 {
-  BOOL valid = [fm fileExistsAtPath: path];
+  BOOL valid = (attributes != nil);
 
-  if ((valid == NO) && flags.link) {
-    valid = ([fm fileAttributesAtPath: path traverseLink: NO] != nil);
+  if (valid) {
+    valid = [fm fileExistsAtPath: path];
+
+    if ((valid == NO) && flags.link) {
+      valid = ([fm fileAttributesAtPath: path traverseLink: NO] != nil);
+    }
   }
-
+  
   return valid;
 }
 
@@ -978,6 +1001,11 @@
 
 @implementation FSNode (Comparing)
 
+- (NSComparisonResult)compareAccordingToPath:(FSNode *)aNode
+{
+  return [path compare: [aNode path]];
+}
+
 - (NSComparisonResult)compareAccordingToName:(FSNode *)aNode
 {
   NSString *n1 = [self name];
