@@ -625,22 +625,35 @@ static FSNodeRep *shared = nil;
   if ([created count]) {
     NSString *dictName = @"thumbnails.plist";
     NSString *dictPath = [thumbnailDir stringByAppendingPathComponent: dictName];
-    NSDictionary *tdict = [NSDictionary dictionaryWithContentsOfFile: dictPath];
-  
-    for (i = 0; i < [created count]; i++) {
-      NSString *key = [created objectAtIndex: i];
-      NSString *tumbname = [tdict objectForKey: key];
-      NSString *tumbpath = [thumbnailDir stringByAppendingPathComponent: tumbname]; 
+    
+    if ([fm fileExistsAtPath: dictPath]) {
+      NSDictionary *tdict = [NSDictionary dictionaryWithContentsOfFile: dictPath];
 
-      if ([fm fileExistsAtPath: tumbpath]) {
-        NSImage *tumb = [[NSImage alloc] initWithContentsOfFile: tumbpath];
+      for (i = 0; i < [created count]; i++) {
+        NSString *key = [created objectAtIndex: i];
+        NSString *tumbname = [tdict objectForKey: key];
+        NSString *tumbpath = [thumbnailDir stringByAppendingPathComponent: tumbname]; 
+
+        if ([fm fileExistsAtPath: tumbpath]) {
+          NSImage *tumb = nil;
         
-        if (tumb) {
-          [tumbsCache setObject: tumb forKey: key];
-          RELEASE (tumb);
+          NS_DURING
+            {
+          tumb = [[NSImage alloc] initWithContentsOfFile: tumbpath];
+          
+          if (tumb) {
+            [tumbsCache setObject: tumb forKey: key];
+            RELEASE (tumb);
+          }
+            }
+          NS_HANDLER
+            {
+          NSLog(@"BAD IMAGE '%@'", tumbpath);
+            }
+          NS_ENDHANDLER
         }
-      }
-    }
+      }    
+    }  
   }
 }
 
@@ -817,19 +830,19 @@ static FSNodeRep *shared = nil;
 
 - (NSArray *)removableMediaPaths
 {
-  NSUserDefaults *defaults;
-  NSMutableDictionary *domain;
+  NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
+  NSDictionary *domain;
   NSArray *removables;
-  
-  defaults = [NSUserDefaults standardUserDefaults];
-  [defaults synchronize];
-  domain = [[defaults persistentDomainForName: NSGlobalDomain] mutableCopy];
+    
+  domain = [defaults persistentDomainForName: NSGlobalDomain];
   removables = [domain objectForKey: @"GSRemovableMediaPaths"];
 
   if (removables == nil) {
+    CREATE_AUTORELEASE_POOL(arp);
+    NSMutableDictionary *mdomain = [domain mutableCopy];  
     unsigned int systype = [[NSProcessInfo processInfo] operatingSystem];
   
-    switch(systype) {
+    switch (systype) {
       case NSGNULinuxOperatingSystem:
         removables = [NSArray arrayWithObjects: @"/mnt/floppy", @"/mnt/cdrom", nil];
         break;
@@ -841,23 +854,33 @@ static FSNodeRep *shared = nil;
       default:
         break;
     }
+    
+    if (removables) {
+      [mdomain setObject: removables forKey: @"GSRemovableMediaPaths"];
+      [defaults setPersistentDomain: mdomain forName: NSGlobalDomain];
+      [defaults synchronize];
+    }
+    
+    RELEASE (mdomain);
+    RELEASE (arp);
   }
-
+    
   return removables;
 }
 
 - (NSArray *)reservedMountNames
 {
-  NSUserDefaults *defaults;
-  NSMutableDictionary *domain;
+  NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
+  NSDictionary *domain;
   NSArray *reserved;
 
-  defaults = [NSUserDefaults standardUserDefaults];
   [defaults synchronize];
-  domain = [[defaults persistentDomainForName: NSGlobalDomain] mutableCopy];
+  domain = [defaults persistentDomainForName: NSGlobalDomain];
   reserved = [domain objectForKey: @"GSReservedMountNames"];
   
   if (reserved == nil) {
+    CREATE_AUTORELEASE_POOL(arp);
+    NSMutableDictionary *mdomain = [domain mutableCopy];  
     unsigned int systype = [[NSProcessInfo processInfo] operatingSystem];
   
     switch(systype) {
@@ -879,6 +902,15 @@ static FSNodeRep *shared = nil;
       default:
         break;
     }
+    
+    if (reserved) {
+      [mdomain setObject: reserved forKey: @"GSReservedMountNames"];
+      [defaults setPersistentDomain: mdomain forName: NSGlobalDomain];
+      [defaults synchronize];
+    }
+    
+    RELEASE (mdomain);
+    RELEASE (arp);
   }
 
   return reserved;
