@@ -61,16 +61,16 @@ enum {
   NUM_BOOL
 };
 
-typedef enum _GMDSOperatorType
+typedef enum _MDKOperatorType
 {
-  GMDSLessThanOperatorType = 0,
-  GMDSLessThanOrEqualToOperatorType,
-  GMDSGreaterThanOperatorType,
-  GMDSGreaterThanOrEqualToOperatorType,
-  GMDSEqualToOperatorType,
-  GMDSNotEqualToOperatorType,
-  GMDSInRangeOperatorType
-} GMDSOperatorType;
+  MDKLessThanOperatorType,
+  MDKLessThanOrEqualToOperatorType,
+  MDKGreaterThanOperatorType,
+  MDKGreaterThanOrEqualToOperatorType,
+  MDKEqualToOperatorType,
+  MDKNotEqualToOperatorType,
+  MDKInRangeOperatorType
+} MDKOperatorType;
 
 
 static void path_exists(sqlite3_context *context, int argc, sqlite3_value **argv)
@@ -86,6 +86,64 @@ static void path_exists(sqlite3_context *context, int argc, sqlite3_value **argv
   sqlite3_result_int(context, exists);
 }
 
+static void path_moved(sqlite3_context *context, int argc, sqlite3_value **argv)
+{
+  const unsigned char *oldbase = sqlite3_value_text(argv[0]);
+  int oldblen = strlen((const char *)oldbase);
+  const unsigned char *newbase = sqlite3_value_text(argv[1]);
+  int newblen = strlen((const char *)newbase);
+  const unsigned char *oldpath = sqlite3_value_text(argv[2]);
+  int oldplen = strlen((const char *)oldpath);
+  char newpath[PATH_MAX] = "";
+  int i = newblen;
+  int j;
+  
+  strncpy(newpath, (const char *)newbase, newblen);  
+  
+  for (j = oldblen; j < oldplen; j++) {
+    newpath[i] = oldpath[j];
+    i++;
+  }
+  
+  newpath[i] = '\0';
+  
+  sqlite3_result_text(context, newpath, strlen(newpath), SQLITE_TRANSIENT);
+}
+
+static void time_stamp(sqlite3_context *context, int argc, sqlite3_value **argv)
+{
+  NSTimeInterval interval = [[NSDate date] timeIntervalSinceReferenceDate];
+
+  sqlite3_result_double(context, interval);
+}
+
+static void contains_substr(sqlite3_context *context, int argc, sqlite3_value **argv)
+{
+  const char *buff = (const char *)sqlite3_value_text(argv[0]);
+  const char *substr = (const char *)sqlite3_value_text(argv[1]);
+  int contains = (strstr(buff, substr) != NULL);
+  
+  sqlite3_result_int(context, contains);
+}
+
+static void append_string(sqlite3_context *context, int argc, sqlite3_value **argv)
+{
+  const char *buff = (const char *)sqlite3_value_text(argv[0]);
+  const char *str = (const char *)sqlite3_value_text(argv[1]);
+
+  if (strstr(buff, str) == NULL) {
+    char newbuff[2048] = "";
+  
+    sprintf(newbuff, "%s %s", buff, str);
+    newbuff[strlen(newbuff)] = '\0';
+    sqlite3_result_text(context, newbuff, strlen(newbuff), SQLITE_TRANSIENT);
+    
+    return;
+  } 
+  
+  sqlite3_result_text(context, buff, strlen(buff), SQLITE_TRANSIENT);  
+}
+
 static void word_score(sqlite3_context *context, int argc, sqlite3_value **argv)
 {
   int searchlen = strlen((const char *)sqlite3_value_text(argv[0]));
@@ -95,7 +153,6 @@ static void word_score(sqlite3_context *context, int argc, sqlite3_value **argv)
   float score = (1.0 * posting_wcount / path_wcount);
 
   if (searchlen != foundlen) {
-    /* TODO a better correction algorithm for score */
     score *= (1.0 * searchlen / foundlen);    
   } 
 
@@ -104,37 +161,7 @@ static void word_score(sqlite3_context *context, int argc, sqlite3_value **argv)
 
 static void attribute_score(sqlite3_context *context, int argc, sqlite3_value **argv)
 {
-  const unsigned char *search_val = sqlite3_value_text(argv[0]);
-  const unsigned char *found_val = sqlite3_value_text(argv[1]);
-  int attribute_type = sqlite3_value_int(argv[2]);
-  int operator_type = sqlite3_value_int(argv[3]);
-  float score = 1.0;
-
-  if ((attribute_type == STRING) 
-              || (attribute_type == ARRAY) 
-                          || (attribute_type == DATA)) {
-    if (operator_type == GMDSEqualToOperatorType) {                          
-      int searchlen = strlen((const char *)search_val);
-      int foundlen = strlen((const char *)found_val);
-    
-      score *= (searchlen / foundlen); 
-    }
-  }
-
-
-
-  /*
-    const char *searchValue IL VALORE CERCATO
-    
-    attributes.attribute IL VALORE TROVATO
-    
-    int attributeType [STRING, ARRAY, NUMBER, DATE, DATA]
-    
-    GMDSOperatorType operatorType
-  */
-  
-
-  sqlite3_result_double(context, score);
+  sqlite3_result_double(context, 0.0);
 }
 
 
@@ -199,7 +226,7 @@ static void attribute_score(sqlite3_context *context, int argc, sqlite3_value **
       }
     }
 
-    dbdir = [dbdir stringByAppendingPathComponent: @"v3"];
+    dbdir = [dbdir stringByAppendingPathComponent: db_version];
 
     if (([fm fileExistsAtPath: dbdir isDirectory: &isdir] &isdir) == NO) {
       if ([fm createDirectoryAtPath: dbdir attributes: nil] == NO) { 
@@ -429,15 +456,14 @@ static void attribute_score(sqlite3_context *context, int argc, sqlite3_value **
   [self performSubquery: @"COMMIT"];
 }
 
-- (oneway void)performQuery:(NSData *)queryInfo
+- (oneway void)performQuery:(NSDictionary *)queryInfo
 {
   CREATE_AUTORELEASE_POOL(pool); 
-  NSDictionary *dict = [NSUnarchiver unarchiveObjectWithData: queryInfo];  
-  NSArray *prequeries = [dict objectForKey: @"pre_queries"];
+  NSArray *prequeries = [queryInfo objectForKey: @"pre"];
   BOOL prepared = YES;
-  NSString *query = [dict objectForKey: @"query"];
-  NSArray *postqueries = [dict objectForKey: @"post_queries"];  
-  NSNumber *queryNumber = [dict objectForKey: @"query_number"];  
+  NSString *query = [queryInfo objectForKey: @"join"];
+  NSArray *postqueries = [queryInfo objectForKey: @"post"];  
+  NSNumber *queryNumber = [queryInfo objectForKey: @"qnumber"];  
   const char *qbuff = [query UTF8String];
   NSMutableArray *reslines = [NSMutableArray array];
   struct sqlite3_stmt *stmt;
@@ -497,12 +523,10 @@ static void attribute_score(sqlite3_context *context, int argc, sqlite3_value **
         if (err == SQLITE_DONE) {
           GWDebugLog(@"SENDING (last)");
 
-          if ([reslines count]) {
-            if ([self sendResults: reslines forQueryWithNumber: queryNumber]) {
-              GWDebugLog(@"SENT");
-            } else {
-              GWDebugLog(@"INVALID!");
-            }
+          if ([self sendResults: reslines forQueryWithNumber: queryNumber]) {
+            GWDebugLog(@"SENT");
+          } else {
+            GWDebugLog(@"INVALID!");
           }
 
           break;
@@ -537,7 +561,7 @@ static void attribute_score(sqlite3_context *context, int argc, sqlite3_value **
     [self performPostQueries: postqueries];
   }
   
-  [[clientInfo objectForKey: @"client"] endOfQuery];
+  [self endOfQueryWithNumber: queryNumber];
   
   RELEASE (pool);
 }
@@ -547,17 +571,22 @@ static void attribute_score(sqlite3_context *context, int argc, sqlite3_value **
 {
   CREATE_AUTORELEASE_POOL(arp); 
   id client = [clientInfo objectForKey: @"client"];
-  NSMutableDictionary *results = [NSMutableDictionary dictionary];
+  NSDictionary *results;
   BOOL accepted;
   
-  [results setObject: qnum forKey: @"query_number"];  
-  [results setObject: lines forKey: @"lines"];
+  results = [NSDictionary dictionaryWithObjectsAndKeys: qnum, @"qnumber",
+                                                        lines, @"lines", nil];  
   accepted = [client queryResults: [NSArchiver archivedDataWithRootObject: results]];    
   RELEASE (arp);
   
   return accepted;
 }
 
+- (void)endOfQueryWithNumber:(NSNumber *)qnum
+{
+  [[clientInfo objectForKey: @"client"] endOfQueryWithNumber: qnum];
+}
+
 - (BOOL)opendb
 {
   if (db == NULL) {
@@ -575,17 +604,25 @@ static void attribute_score(sqlite3_context *context, int argc, sqlite3_value **
         } else {
           GWDebugLog(@"contents database created");
         }
-      }    
+      }
     } else {
       NSLog(@"unable to open the database at %@", dbpath);
       return NO;
     }    
     
     sqlite3_create_function(db, "pathExists", 1, 
-                                SQLITE_UTF8, 0, path_exists, 0, 0);
+                                SQLITE_UTF8, 0, path_exists, 0, 0);                                
+    sqlite3_create_function(db, "pathMoved", 3, 
+                                SQLITE_UTF8, 0, path_moved, 0, 0);
+    sqlite3_create_function(db, "timeStamp", 0, 
+                                SQLITE_UTF8, 0, time_stamp, 0, 0);
+    sqlite3_create_function(db, "containsSubstr", 2, 
+                                SQLITE_UTF8, 0, contains_substr, 0, 0);                                
+    sqlite3_create_function(db, "appendString", 2, 
+                                SQLITE_UTF8, 0, append_string, 0, 0);
     sqlite3_create_function(db, "wordScore", 4, 
                                 SQLITE_UTF8, 0, word_score, 0, 0);
-    sqlite3_create_function(db, "attributeScore", 4, 
+    sqlite3_create_function(db, "attributeScore", 5, 
                                 SQLITE_UTF8, 0, attribute_score, 0, 0);
 
     performWriteQuery(db, @"PRAGMA cache_size = 20000");
