@@ -48,15 +48,6 @@ do { \
   } \
 } while (0)
 
-#define EXECUTE_OR_ROLLBACK(q, r) \
-do { \
-  if ([sqlite executeQuery: q] == NO) { \
-    [sqlite executeQuery: @"ROLLBACK"]; \
-    NSLog(@"error at: %@", q); \
-    return r; \
-  } \
-} while (0)
-
 #define STATEMENT_EXECUTE_QUERY(s, r) \
 do { \
   if ([sqlite executeQueryWithStatement: s] == NO) { \
@@ -65,15 +56,13 @@ do { \
   } \
 } while (0)
 
-#define STATEMENT_EXECUTE_OR_ROLLBACK(s, r) \
-do { \
-  if ([sqlite executeQueryWithStatement: s] == NO) { \
-    [sqlite executeQuery: @"ROLLBACK"]; \
-    NSLog(@"error at: %@", [s query]); \
-    return r; \
-  } \
-} while (0)
 
+static BOOL updating = NO;
+
+static void check_updating(sqlite3_context *context, int argc, sqlite3_value **argv)
+{
+  sqlite3_result_int(context, (int)updating);
+}
 
 static void path_exists(sqlite3_context *context, int argc, sqlite3_value **argv)
 {
@@ -133,6 +122,7 @@ static void time_stamp(sqlite3_context *context, int argc, sqlite3_value **argv)
   [nc removeObserver: self];
   
   RELEASE (indexablePaths);
+  freeTree(includePathsTree);
   freeTree(excludedPathsTree);
   RELEASE (excludedSuffixes);
   RELEASE (dbpath);
@@ -164,17 +154,30 @@ static void time_stamp(sqlite3_context *context, int argc, sqlite3_value **argv)
     [lostPathsTimer invalidate];
   }
   TEST_RELEASE (lostPathsTimer);
-  
+
+  //
+  // ddbd_update
+  //
+  DESTROY (ddbd);
+    
   //  
   // scheduled_update  
   //
   if (schedupdateTimer && [schedupdateTimer isValid]) {
     [schedupdateTimer invalidate];
   }
-  TEST_RELEASE (schedupdateTimer);
-  
+  TEST_RELEASE (schedupdateTimer);  
   RELEASE (directories);
   
+  //
+  // update_notifications
+  //
+  if (notificationsTimer && [notificationsTimer isValid]) {
+    [notificationsTimer invalidate];
+  }
+  TEST_RELEASE (notificationsTimer);
+  RELEASE (notifDate);
+    
   [super dealloc];
 }
 
@@ -185,7 +188,6 @@ static void time_stamp(sqlite3_context *context, int argc, sqlite3_value **argv)
   if (self) {
     NSUserDefaults *defaults;
     id entry;
-    NSString *dbdir;
     NSString *lockpath;
     NSString *errpath;
     unsigned i;
@@ -197,12 +199,12 @@ static void time_stamp(sqlite3_context *context, int argc, sqlite3_value **argv)
     dbdir = [dbdir stringByAppendingPathComponent: @".db"];
 
     ASSIGN (indexedStatusPath, [dbdir stringByAppendingPathComponent: @"status.plist"]);
-    lockpath = [dbdir stringByAppendingPathComponent: @"extractors.lock"];
-    indexedStatusLock = [[NSDistributedLock alloc] initWithPath: lockpath];
+    lockpath = [dbdir stringByAppendingPathComponent: @"extractors.lock"];    
 
     errpath = [dbdir stringByAppendingPathComponent: @"error.log"];
         
-    dbdir = [dbdir stringByAppendingPathComponent: @"v3"];
+    dbdir = [dbdir stringByAppendingPathComponent: db_version];
+    RETAIN (dbdir);
     ASSIGN (dbpath, [dbdir stringByAppendingPathComponent: @"contents.db"]);    
     
     sqlite = [SQLite new];
@@ -212,12 +214,20 @@ static void time_stamp(sqlite3_context *context, int argc, sqlite3_value **argv)
       return self;    
     }
 
+    indexedStatusLock = [[NSDistributedLock alloc] initWithPath: lockpath];
+
+    if (indexedStatusLock == nil) {
+      DESTROY (self);
+      return self;    
+    }
+
     if ([fm fileExistsAtPath: errpath] == NO) {
       [fm createFileAtPath: errpath contents: nil attributes: nil];
     }
     errHandle = [NSFileHandle fileHandleForWritingAtPath: errpath];
     RETAIN (errHandle);
 
+
     conn = [NSConnection defaultConnection];
     [conn setRootObject: self];
     [conn setDelegate: self];
@@ -251,6 +261,8 @@ static void time_stamp(sqlite3_context *context, int argc, sqlite3_value **argv)
     [defaults synchronize];
     
     indexablePaths = [NSMutableArray new];
+    
+    includePathsTree = newTreeWithIdentifier(@"included");
     excludedPathsTree = newTreeWithIdentifier(@"excluded");
     excludedSuffixes = [[NSMutableSet alloc] initWithCapacity: 1];
     
@@ -263,6 +275,8 @@ static void time_stamp(sqlite3_context *context, int argc, sqlite3_value **argv)
                                                                     ancestor: nil];
         [indexablePaths addObject: indpath];
         RELEASE (indpath);
+        
+        insertComponentsOfPath(path, includePathsTree);
       }
     }
 
@@ -313,6 +327,8 @@ static void time_stamp(sqlite3_context *context, int argc, sqlite3_value **argv)
   unsigned count;
   unsigned i;
 
+  emptyTreeWithBase(includePathsTree);
+
   for (i = 0; i < [indexable count]; i++) {
     NSString *path = [indexable objectAtIndex: i];
     GMDSIndexablePath *indpath = [self indexablePathWithPath: path];   
@@ -322,6 +338,8 @@ static void time_stamp(sqlite3_context *context, int argc, sqlite3_value **argv)
       [indexablePaths addObject: indpath];
       RELEASE (indpath);
     }
+    
+    insertComponentsOfPath(path, includePathsTree);
   }
   
   count = [indexablePaths count];
@@ -521,7 +539,11 @@ static void time_stamp(sqlite3_context *context, int argc, sqlite3_value **argv)
     [indexedStatusLock unlock];
   }
   
-  return ((status != nil) ? status : [NSArray array]);
+  if (status != nil) {
+    return status;
+  }
+  
+  return [NSArray array];
 }
 
 - (void)writePathsStatus:(id)sender
@@ -569,7 +591,7 @@ static void time_stamp(sqlite3_context *context, int argc, sqlite3_value **argv)
     [status writeToFile: indexedStatusPath atomically: YES];
     [indexedStatusLock unlock];
     
- //   GWDebugLog(@"paths status updated"); 
+    GWDebugLog(@"paths status updated"); 
     
     RELEASE (arp);
   }
@@ -767,6 +789,8 @@ static void time_stamp(sqlite3_context *context, int argc, sqlite3_value **argv)
   NSDictionary *attributes = [fm fileAttributesAtPath: path traverseLink: NO];
   
   if (attributes) {
+    NSString *app = nil;
+    NSString *type = nil;
     NSDirectoryEnumerator *enumerator;
     id extractor = nil;
     unsigned long fcount = 0;  
@@ -780,14 +804,20 @@ static void time_stamp(sqlite3_context *context, int argc, sqlite3_value **argv)
     
     EXECUTE_QUERY (@"BEGIN", NO);
     
-    path_id = [self insertOrUpdatePath: path withAttributes: attributes];
+    [ws getInfoForFile: path application: &app type: &type];  
+    
+    path_id = [self insertOrUpdatePath: path 
+                                ofType: type
+                        withAttributes: attributes];
     
     if (path_id == -1) {
       [sqlite executeQuery: @"ROLLBACK"];
       return NO;
     }
 
-    extractor = [self extractorForPath: path withAttributes: attributes];
+    extractor = [self extractorForPath: path 
+                                ofType: type
+                        withAttributes: attributes];
 
     if (extractor) {
       if ([extractor extractMetadataAtPath: path
@@ -829,16 +859,20 @@ static void time_stamp(sqlite3_context *context, int argc, sqlite3_value **argv)
           BOOL hasextractor = NO;
         
           if (skip == NO) {
-            #if 0 
-              [self logError: [NSString stringWithFormat: @"EXTRACT %@", subpath]];   // TOGLIERE !!!!
-            #endif
-            
+            NSString *app = nil;
+            NSString *type = nil;        
+                                
             [sqlite executeQuery: @"BEGIN"];
             
-            path_id = [self insertOrUpdatePath: subpath withAttributes: attributes];
+            [ws getInfoForFile: subpath application: &app type: &type];
+            
+            path_id = [self insertOrUpdatePath: subpath 
+                                        ofType: type
+                                withAttributes: attributes];
                     
             if (path_id != -1) {
               extractor = [self extractorForPath: subpath 
+                                          ofType: type
                                   withAttributes: attributes];
 
               if (extractor) {
@@ -920,6 +954,7 @@ static void time_stamp(sqlite3_context *context, int argc, sqlite3_value **argv)
 }
 
 - (int)insertOrUpdatePath:(NSString *)path
+                   ofType:(NSString *)type
            withAttributes:(NSDictionary *)attributes
 {
   NSTimeInterval interval = [[attributes fileModificationDate] timeIntervalSinceReferenceDate];
@@ -935,11 +970,11 @@ static void time_stamp(sqlite3_context *context, int argc, sqlite3_value **argv)
 
 #define KEY_AND_ATTRIBUTE(k, a) \
 do { \
-  NSMutableDictionary *dict = [NSMutableDictionary dictionary]; \
-  [dict setObject: k forKey: @"key"]; \
-  [dict setObject: a forKey: @"attribute"]; \
-\
-  [mdattributes addObject: dict]; \
+  if (a) { \
+    NSDictionary *dict = [NSDictionary dictionaryWithObjectsAndKeys: \
+                                        k, @"key", a, @"attribute", nil]; \
+    [mdattributes addObject: dict]; \
+  } \
 } while (0)
     
   query = @"SELECT id FROM paths WHERE path = :path";
@@ -1004,7 +1039,15 @@ do { \
 
   KEY_AND_ATTRIBUTE (@"GSMDItemFSName", qname);  
   KEY_AND_ATTRIBUTE (@"GSMDItemFSExtension", qext);  
-  KEY_AND_ATTRIBUTE (@"GSMDItemFSType", [attributes fileType]);  
+  KEY_AND_ATTRIBUTE (@"GSMDItemFSType", type);  
+  
+  if (ddbd) {
+    NSArray *usermdata = [ddbd userMetadataForPath: path];
+    
+    if (usermdata) {
+      [mdattributes addObjectsFromArray: usermdata];
+    }
+  }
   
   for (i = 0; i < [mdattributes count]; i++) {
     NSDictionary *dict = [mdattributes objectAtIndex: i];      
@@ -1046,7 +1089,7 @@ do { \
     query = @"UPDATE paths "
             @"SET words_count = :wcount "
             @"WHERE id = :pathid";
-
+  
     statement = [sqlite statementForQuery: query 
                            withIdentifier: @"set_metadata_1"
                                  bindings: SQLITE_INTEGER, @":wcount", wcount, 
@@ -1145,15 +1188,13 @@ do { \
 }
 
 - (id)extractorForPath:(NSString *)path
+                ofType:(NSString *)type
         withAttributes:(NSDictionary *)attributes
 {
   NSString *ext = [[path pathExtension] lowercaseString];
-  NSString *app = nil, *type = nil;
   NSData *data = nil;
   id extractor = nil;  
   
-  [ws getInfoForFile: path application: &app type: &type]; 
-  
   if ([attributes fileType] == NSFileTypeRegular) {
     NSFileHandle *handle = [NSFileHandle fileHandleForReadingAtPath: path];
 
@@ -1189,7 +1230,7 @@ do { \
                                    testData: data]) {
     return textExtractor;
   }
-  
+
   return nil;
 }
 
@@ -1252,7 +1293,7 @@ do { \
 {
   BOOL newdb;
 
-  if ([sqlite opendbAtPath: dbpath isNew: &newdb]) {
+  if ([sqlite opendbAtPath: dbpath isNew: &newdb]) {    
     if (newdb) {
       if ([sqlite executeSimpleQuery: db_schema] == NO) {
         NSLog(@"unable to create the database at %@", dbpath);
@@ -1266,6 +1307,10 @@ do { \
     return NO;
   }    
 
+  [sqlite createFunctionWithName: @"checkUpdating"
+                  argumentsCount: 0
+                    userFunction: check_updating];
+
   [sqlite createFunctionWithName: @"pathExists"
                   argumentsCount: 1
                     userFunction: path_exists];
@@ -1578,7 +1623,10 @@ int main(int argc, char** argv)
 
   {
     CREATE_AUTORELEASE_POOL (pool);
-	  GMDSExtractor *extractor = [[GMDSExtractor alloc] init];
+	  GMDSExtractor *extractor;
+    
+    [NSApplication sharedApplication];
+    extractor = [GMDSExtractor new];
     RELEASE (pool);
 
     if (extractor != nil) {
@@ -1592,6 +1640,11 @@ int main(int argc, char** argv)
 }
 
 
+void setUpdating(BOOL value)
+{
+  updating = value;
+}
+
 BOOL isDotFile(NSString *path)
 {
   int len = ([path length] - 1);
