@@ -60,9 +60,12 @@ do { \
   } \
 } while (0)
 
-#define STATEMENT_EXECUTE_OR_ROLLBACK(s, r) \
+#define STATEMENT_EXECUTE_OR_ROLLBACK(s, u, r) \
 do { \
   if ([sqlite executeQueryWithStatement: s] == NO) { \
+    if (u) { \
+      setUpdating(NO); \
+    } \
     [sqlite executeQuery: @"ROLLBACK"]; \
     NSLog(@"error at: %@", [s query]); \
     return r; \
@@ -74,6 +77,7 @@ do { \
 #define LOST_PATHS_EXPIRE (60.0)
 #define LOST_PATHS_CHECK (30.0)
 #define SCHED_TIME (1.0)
+#define NOTIF_TIME (60.0)
 
 
 @implementation GMDSExtractor (updater)
@@ -81,7 +85,9 @@ do { \
 - (void)setupUpdaters
 {
   [self setupFswatcherUpdater];
+  [self setupDDBdUpdater];
   [self setupScheduledUpdater];
+  [self setupUpdateNotifications];
 }
 
 - (BOOL)addPath:(NSString *)path
@@ -89,21 +95,26 @@ do { \
   NSDictionary *attributes = [fm fileAttributesAtPath: path traverseLink: NO];
       
   if (attributes) {
+    NSString *app = nil;
+    NSString *type = nil;
     id extractor = nil;
     BOOL failed = NO;
     BOOL hasextractor = NO;
     int path_id;
-
-    #if 0 
-      [self logError: [NSString stringWithFormat: @"UPDATE %@", path]];   
-    #endif
     
-    EXECUTE_QUERY (@"BEGIN", NO);
+    EXECUTE_QUERY (@"BEGIN", NO); 
+    setUpdating(YES);
+     
+    [ws getInfoForFile: path application: &app type: &type];  
     
-    path_id = [self insertOrUpdatePath: path withAttributes: attributes];
+    path_id = [self insertOrUpdatePath: path 
+                                ofType: type
+                        withAttributes: attributes];
     
     if (path_id != -1) {
-      extractor = [self extractorForPath: path withAttributes: attributes];    
+      extractor = [self extractorForPath: path 
+                                  ofType: type
+                          withAttributes: attributes];    
     
       if (extractor) {
         hasextractor = YES;
@@ -120,6 +131,7 @@ do { \
     }
     
     if (failed == NO) {
+      setUpdating(NO);
       [sqlite executeQuery: @"COMMIT"];
       
       if (hasextractor) {
@@ -129,6 +141,7 @@ do { \
       }
       
     } else {
+      setUpdating(NO);
       [sqlite executeQuery: @"ROLLBACK"];
       [self logError: [NSString stringWithFormat: @"UPDATE %@", path]];
       GWDebugLog(@"error updating at: %@", path);
@@ -157,21 +170,27 @@ do { \
         
           attributes = [fm fileAttributesAtPath: subpath traverseLink: NO];
         
-          if (attributes) {
+          if (attributes) {          
             failed = NO;
             hasextractor = NO;
             
             if (skip == NO) {
-              #if 0 
-                [self logError: [NSString stringWithFormat: @"UPDATE %@", subpath]]; 
-              #endif
+              NSString *app = nil;
+              NSString *type = nil;
               
               [sqlite executeQuery: @"BEGIN"];
+              setUpdating(YES);
               
-              path_id = [self insertOrUpdatePath: subpath withAttributes: attributes];
+              [ws getInfoForFile: subpath application: &app type: &type];
+              
+              path_id = [self insertOrUpdatePath: subpath 
+                                          ofType: type
+                                  withAttributes: attributes];
     
               if (path_id != -1) {
-                extractor = [self extractorForPath: subpath withAttributes: attributes];
+                extractor = [self extractorForPath: subpath 
+                                            ofType: type
+                                    withAttributes: attributes];
     
                 if (extractor) {
                   hasextractor = YES;
@@ -186,8 +205,9 @@ do { \
               } else {
                 failed = YES;
               }
-    
-              [sqlite executeQuery: (failed ? @"ROLLBACK" : @"COMMIT")];
+              
+              setUpdating(NO);
+              [sqlite executeQuery: @"COMMIT"];
             }
             
             if (skip) { 
@@ -227,29 +247,41 @@ do { \
   NSDictionary *attributes = [fm fileAttributesAtPath: path traverseLink: NO];
       
   if (attributes) {
+    NSString *app = nil;
+    NSString *type = nil;  
     id extractor;
     int path_id;
     
     EXECUTE_QUERY (@"BEGIN", NO);
+    setUpdating(YES);
     
-    path_id = [self insertOrUpdatePath: path withAttributes: attributes];
+    [ws getInfoForFile: path application: &app type: &type];  
+    
+    path_id = [self insertOrUpdatePath: path 
+                                ofType: type
+                        withAttributes: attributes];
     
     if (path_id == -1) {
-      [sqlite executeQuery: @"ROLLBACK"];
+      setUpdating(NO);
+      [sqlite executeQuery: @"COMMIT"];
       return NO;
     }
 
-    extractor = [self extractorForPath: path withAttributes: attributes];
+    extractor = [self extractorForPath: path 
+                                ofType: type
+                        withAttributes: attributes];
 
     if (extractor) {
       if ([extractor extractMetadataAtPath: path
                                     withID: path_id
                                 attributes: attributes] == NO) {
-        [sqlite executeQuery: @"ROLLBACK"];
+        setUpdating(NO);                         
+        [sqlite executeQuery: @"COMMIT"];
         return NO;
       }
     }
     
+    setUpdating(NO);
     [sqlite executeQuery: @"COMMIT"];
   }
   
@@ -266,18 +298,19 @@ do { \
   NSString *query;
         
   EXECUTE_QUERY (@"BEGIN", NO);
+  setUpdating(YES);
 
   statement = [sqlite statementForQuery: @"DELETE FROM renamed_paths" 
                          withIdentifier: @"update_renamed_1"
                                bindings: 0];
 
-  STATEMENT_EXECUTE_OR_ROLLBACK (statement, NO);
+  STATEMENT_EXECUTE_OR_ROLLBACK (statement, YES, NO);
 
   statement = [sqlite statementForQuery: @"DELETE FROM renamed_paths_base" 
                          withIdentifier: @"update_renamed_2"
                                bindings: 0];
 
-  STATEMENT_EXECUTE_OR_ROLLBACK (statement, NO);
+  STATEMENT_EXECUTE_OR_ROLLBACK (statement, YES, NO);
 
   query = @"INSERT INTO renamed_paths_base "
           @"(base, oldbase) "
@@ -288,7 +321,7 @@ do { \
                                bindings: SQLITE_TEXT, @":path", qpath, 
                                          SQLITE_TEXT, @":oldpath", qoldpath, 0];
 
-  STATEMENT_EXECUTE_OR_ROLLBACK (statement, NO);
+  STATEMENT_EXECUTE_OR_ROLLBACK (statement, YES, NO);
   
   if (isdir) {
     query = @"INSERT INTO renamed_paths "
@@ -317,8 +350,9 @@ do { \
                                  bindings: SQLITE_TEXT, @":oldpath", qoldpath, 0];
   }
   
-  STATEMENT_EXECUTE_OR_ROLLBACK (statement, NO);
+  STATEMENT_EXECUTE_OR_ROLLBACK (statement, YES, NO);
 
+  setUpdating(NO);
   EXECUTE_QUERY (@"COMMIT", NO);
   
   return YES;
@@ -331,12 +365,13 @@ do { \
   NSString *query;
       
   EXECUTE_QUERY (@"BEGIN", NO);
+  setUpdating(YES);
 
   statement = [sqlite statementForQuery: @"DELETE FROM removed_id" 
                          withIdentifier: @"remove_path_1"
                                bindings: 0];
                              
-  STATEMENT_EXECUTE_OR_ROLLBACK (statement, NO);
+  STATEMENT_EXECUTE_OR_ROLLBACK (statement, YES, NO);
     
   query = @"INSERT INTO removed_id (id) "
           @"SELECT id FROM paths "
@@ -349,7 +384,7 @@ do { \
                                          SQLITE_TEXT, @":minpath", 
           [NSString stringWithFormat: @"%@%@*", qpath, path_separator()], 0];
       
-  STATEMENT_EXECUTE_OR_ROLLBACK (statement, NO);
+  STATEMENT_EXECUTE_OR_ROLLBACK (statement, YES, NO);
 
   query = @"DELETE FROM attributes WHERE path_id IN (SELECT id FROM removed_id)";
 
@@ -357,7 +392,7 @@ do { \
                          withIdentifier: @"remove_path_3"
                                bindings: 0];
 
-  STATEMENT_EXECUTE_OR_ROLLBACK (statement, NO);
+  STATEMENT_EXECUTE_OR_ROLLBACK (statement, YES, NO);
 
   query = @"DELETE FROM postings WHERE path_id IN (SELECT id FROM removed_id)";
 
@@ -365,7 +400,7 @@ do { \
                          withIdentifier: @"remove_path_4"
                                bindings: 0];
 
-  STATEMENT_EXECUTE_OR_ROLLBACK (statement, NO);
+  STATEMENT_EXECUTE_OR_ROLLBACK (statement, YES, NO);
 
   query = @"DELETE FROM paths WHERE id IN (SELECT id FROM removed_id)";
 
@@ -373,8 +408,9 @@ do { \
                          withIdentifier: @"remove_path_5"
                                bindings: 0];
 
-  STATEMENT_EXECUTE_OR_ROLLBACK (statement, NO);
+  STATEMENT_EXECUTE_OR_ROLLBACK (statement, YES, NO);
 
+  setUpdating(NO);
   EXECUTE_QUERY (@"COMMIT", NO);
 
   return YES;
@@ -446,254 +482,254 @@ do { \
   RETAIN (lostPathsTimer);     
      
   fswatcher = nil;
-  [self connectFSWatcher];
+  [self connectFSWatcher: nil];
 }
 
 - (oneway void)globalWatchedPathDidChange:(NSDictionary *)info
 {
-  CREATE_AUTORELEASE_POOL(arp);
-  NSMutableDictionary *dict = [NSMutableDictionary dictionary];  
-  NSString *path = [info objectForKey: @"path"];
-  NSString *event = [info objectForKey: @"event"];
-  NSNumber *exists;
-    
-  if ([event isEqual: @"GWWatchedPathDeleted"]) {
-    exists = [NSNumber numberWithBool: NO];
-    
-  } else if ([event isEqual: @"GWWatchedFileModified"]) {
-    exists = [NSNumber numberWithBool: YES];
-  
-  } else if ([event isEqual: @"GWWatchedPathRenamed"]) {
-    NSString *oldpath = [info objectForKey: @"oldpath"];
- 
-    if (oldpath != nil) {    
-      [dict setObject: oldpath forKey: @"oldpath"];   
-    }
-    
-    exists = [NSNumber numberWithBool: YES];
-  } 
+  if (extracting == NO) {
+    CREATE_AUTORELEASE_POOL(arp);
+    NSMutableDictionary *dict = [NSMutableDictionary dictionary];  
+    NSString *path = [info objectForKey: @"path"];
+    NSString *event = [info objectForKey: @"event"];
+    NSNumber *exists;
 
-  [dict setObject: path forKey: @"path"];
-  [dict setObject: event forKey: @"event"];
-  [dict setObject: exists forKey: @"exists"];
+    if ([event isEqual: @"GWWatchedPathDeleted"]) {
+      exists = [NSNumber numberWithBool: NO];
 
-  if ([fswupdatePaths containsObject: dict] == NO) {
-    NSDictionary *skipInfo = [fswupdateSkipBuff objectForKey: path];
-    BOOL caninsert = YES;
-    
-    if (skipInfo != nil) {
-      NSNumber *didexists = [skipInfo objectForKey: @"exists"];
-      NSDate *stamp = [skipInfo objectForKey: @"stamp"];
-      NSDate *now = [NSDate date];
-            
-      if ([exists isEqual: didexists]
-             && ([now timeIntervalSinceDate: stamp] < SKIP_EXPIRE)) {
-        caninsert = NO;
+    } else if ([event isEqual: @"GWWatchedFileModified"]
+                  || [event isEqual: @"GWFileCreatedInWatchedDirectory"]) {
+      exists = [NSNumber numberWithBool: YES];
+
+    } else if ([event isEqual: @"GWWatchedPathRenamed"]) {
+      NSString *oldpath = [info objectForKey: @"oldpath"];
+
+      if (oldpath != nil) {    
+        [dict setObject: oldpath forKey: @"oldpath"];   
+      }
+
+      exists = [NSNumber numberWithBool: YES];
+    } 
+
+    [dict setObject: path forKey: @"path"];
+    [dict setObject: event forKey: @"event"];
+    [dict setObject: exists forKey: @"exists"];
+
+    if ([fswupdatePaths containsObject: dict] == NO) {
+      NSDictionary *skipInfo = [fswupdateSkipBuff objectForKey: path];
+      BOOL caninsert = YES;
+
+      if (skipInfo != nil) {
+        NSNumber *didexists = [skipInfo objectForKey: @"exists"];
+        NSDate *stamp = [skipInfo objectForKey: @"stamp"];
+        NSDate *now = [NSDate date];
+
+        if ([exists isEqual: didexists]
+               && ([now timeIntervalSinceDate: stamp] < SKIP_EXPIRE)) {
+          caninsert = NO;
+        } else {
+          skipInfo = [NSDictionary dictionaryWithObjectsAndKeys: event, @"event", 
+                                                    exists, @"exists",
+                                                    now, @"stamp", nil];     
+          [fswupdateSkipBuff setObject: skipInfo forKey: path];
+        }
       } else {
-        skipInfo = [NSDictionary dictionaryWithObjectsAndKeys: event, @"event", 
-                                                  exists, @"exists",
-                                                  now, @"stamp", nil];     
+        skipInfo = [NSDictionary dictionaryWithObjectsAndKeys: event, @"event",
+                                                   exists, @"exists",
+                                                   [NSDate date], @"stamp", nil];     
         [fswupdateSkipBuff setObject: skipInfo forKey: path];
       }
-    } else {
-      skipInfo = [NSDictionary dictionaryWithObjectsAndKeys: event, @"event",
-                                                 exists, @"exists",
-                                                 [NSDate date], @"stamp", nil];     
-      [fswupdateSkipBuff setObject: skipInfo forKey: path];
+
+      if (caninsert) {
+        [fswupdatePaths insertObject: dict atIndex: 0];
+        GWDebugLog(@"queueing: %@ - %@", path, event);
+      }
     }
-  
-    if (caninsert) {
-      [fswupdatePaths insertObject: dict atIndex: 0];
-      GWDebugLog(@"queueing: %@ - %@", path, event);
-    }
-  }
-     
-  RELEASE (arp);                       
+
+    RELEASE (arp);     
+  }                  
 }
 
 - (void)processPendingChanges:(id)sender
 {
-  CREATE_AUTORELEASE_POOL(arp);
-  
-  while ([fswupdatePaths count] > 0) {
-    NSDictionary *dict = [fswupdatePaths lastObject];
-    NSString *path = [dict objectForKey: @"path"];    
-    NSString *event = [dict objectForKey: @"event"];
-    NSDate *date = [NSDate dateWithTimeIntervalSinceNow: 0.001];
-        
-    [[NSRunLoop currentRunLoop] runUntilDate: date]; 
-    
-    if ([event isEqual: @"GWWatchedFileModified"]) {
-      if ([fm fileExistsAtPath: path]) {
-        GWDebugLog(@"db update: %@", path);
+  if (extracting == NO) {
+    CREATE_AUTORELEASE_POOL(arp);
 
-        if ([self updatePath: path] == NO) {      
-          NSLog(@"An error occurred while processing %@", path);
-        }        
-      } else {
-        NSMutableDictionary *d = [NSMutableDictionary dictionary];
-        
-        [d setObject: path forKey: @"path"];
-        [d setObject: [NSDate date] forKey: @"stamp"];
-        
-        GWDebugLog(@"add lost path: %@", path);
-        [lostPaths addObject: d];
-      }
-       
-    } else if ([event isEqual: @"GWWatchedPathDeleted"]) {
-      if ([fm fileExistsAtPath: path] == NO) {
-        GWDebugLog(@"db remove: %@", path);
-        [self removePath: path];
-      }
+    while ([fswupdatePaths count] > 0) {
+      NSDictionary *dict = [fswupdatePaths lastObject];
+      NSString *path = [dict objectForKey: @"path"];    
+      NSString *event = [dict objectForKey: @"event"];
+      NSDate *date = [NSDate dateWithTimeIntervalSinceNow: 0.001];
 
-    } else if ([event isEqual: @"GWWatchedPathRenamed"]) {
-      BOOL isdir;
-      
-      if ([fm fileExistsAtPath: path isDirectory: &isdir]) {
-        NSString *oldpath = [dict objectForKey: @"oldpath"];
-      
-        if (oldpath != nil) {
-          unsigned count = [lostPaths count];
-          unsigned i;
+      [[NSRunLoop currentRunLoop] runUntilDate: date]; 
 
-          for (i = 0; i < count; i++) {
-            NSMutableDictionary *d = [lostPaths objectAtIndex: i];
-            NSString *lost = [d objectForKey: @"path"];
+      if ([event isEqual: @"GWWatchedFileModified"]
+            || [event isEqual: @"GWFileCreatedInWatchedDirectory"]) {
+        if ([fm fileExistsAtPath: path]) {
+          GWDebugLog(@"db update: %@", path);
 
-            if (subPathOfPath(oldpath, lost)) {
-              unsigned pos = [lost rangeOfString: oldpath].length +1;
-              NSString *part = [lost substringFromIndex: pos];
-              NSString *newpath = [path stringByAppendingPathComponent: part];          
+          if ([event isEqual: @"GWFileCreatedInWatchedDirectory"]) {
+            /* 
+               "GWFileCreatedInWatchedDirectory" is reported only by 
+               fswatcher-inotify.
+               In this case, if "path" is a directory, we must add 
+               also its contents.
+            */
+            if ([self addPath: path] == NO) {
+              NSLog(@"An error occurred while processing %@", path);
+            }                  
+          } else {
+            if ([self updatePath: path] == NO) {      
+              NSLog(@"An error occurred while processing %@", path);
+            }        
+          }          
+          
+        } else {
+          NSMutableDictionary *d = [NSMutableDictionary dictionary];
 
-              GWDebugLog(@"found lost path: %@", lost);          
+          [d setObject: path forKey: @"path"];
+          [d setObject: [NSDate date] forKey: @"stamp"];
 
-              [self removePath: lost];
-              
-              if ([fm fileExistsAtPath: newpath]) {
-                [self updatePath: newpath]; 
-                [lostPaths removeObjectAtIndex: i];
-                count--;
-                i--;
-              } else {
-                [d setObject: newpath forKey: @"path"];
-                [d setObject: [NSDate date] forKey: @"stamp"];
-                GWDebugLog(@"changed lost path: %@ to: %@", lost, newpath);    
+          GWDebugLog(@"add lost path: %@", path);
+          [lostPaths addObject: d];
+        }
+
+      } else if ([event isEqual: @"GWWatchedPathDeleted"]) {
+        if ([fm fileExistsAtPath: path] == NO) {
+          GWDebugLog(@"db remove: %@", path);
+          [self removePath: path];
+        }
+
+      } else if ([event isEqual: @"GWWatchedPathRenamed"]) {
+        BOOL isdir;
+
+        if ([fm fileExistsAtPath: path isDirectory: &isdir]) {
+          NSString *oldpath = [dict objectForKey: @"oldpath"];
+
+          if (oldpath != nil) {
+            unsigned count = [lostPaths count];
+            unsigned i;
+
+            for (i = 0; i < count; i++) {
+              NSMutableDictionary *d = [lostPaths objectAtIndex: i];
+              NSString *lost = [d objectForKey: @"path"];
+
+              if (subPathOfPath(oldpath, lost)) {
+                unsigned pos = [lost rangeOfString: oldpath].length +1;
+                NSString *part = [lost substringFromIndex: pos];
+                NSString *newpath = [path stringByAppendingPathComponent: part];          
+
+                GWDebugLog(@"found lost path: %@", lost);          
+
+                [self removePath: lost];
+
+                if ([fm fileExistsAtPath: newpath]) {
+                  [self updatePath: newpath]; 
+                  [lostPaths removeObjectAtIndex: i];
+                  count--;
+                  i--;
+                } else {
+                  [d setObject: newpath forKey: @"path"];
+                  [d setObject: [NSDate date] forKey: @"stamp"];
+                  GWDebugLog(@"changed lost path: %@ to: %@", lost, newpath);    
+                }
               }
             }
-          }
-        
-          GWDebugLog(@"db rename: %@ -> %@", oldpath, path);
 
-          if ([self updateRenamedPath: path 
-                              oldPath: oldpath 
-                          isDirectory: isdir] == NO) { 
-            NSLog(@"An error occurred while processing %@", path);
+            GWDebugLog(@"db rename: %@ -> %@", oldpath, path);
+
+            if ([self updateRenamedPath: path 
+                                oldPath: oldpath 
+                            isDirectory: isdir] == NO) { 
+              NSLog(@"An error occurred while processing %@", path);
+            }
+
+          } else {
+            GWDebugLog(@"db update renamed: %@", path);
+            [self addPath: path];      
           }
 
         } else {
-          GWDebugLog(@"db update renamed: %@", path);
-          [self addPath: path];      
+          NSMutableDictionary *d = [NSMutableDictionary dictionary];
+
+          [d setObject: path forKey: @"path"];
+          [d setObject: [NSDate date] forKey: @"stamp"];
+          [lostPaths addObject: d];
+          GWDebugLog(@"add lost path: %@", path);
         }
-        
-      } else {
-        NSMutableDictionary *d = [NSMutableDictionary dictionary];
-        
-        [d setObject: path forKey: @"path"];
-        [d setObject: [NSDate date] forKey: @"stamp"];
-        [lostPaths addObject: d];
-        GWDebugLog(@"add lost path: %@", path);
       }
+
+      [fswupdatePaths removeLastObject];    
     }
-       
-    [fswupdatePaths removeLastObject];    
+
+    {
+      NSArray *skipPaths = [fswupdateSkipBuff allKeys];
+      NSDate *now = [NSDate date];
+      unsigned i;
+
+      RETAIN (skipPaths);
+
+      for (i = 0; i < [skipPaths count]; i++) {
+        NSString *path = [skipPaths objectAtIndex: i];
+        NSDictionary *skipInfo = [fswupdateSkipBuff objectForKey: path];
+        NSDate *stamp = [skipInfo objectForKey: @"stamp"];
+
+        if ([now timeIntervalSinceDate: stamp] > SKIP_EXPIRE) {
+          [fswupdateSkipBuff removeObjectForKey: path];
+        }
+      }
+
+      RELEASE (skipPaths);
+    }  
+
+    RELEASE (arp);  
   }
-  
-  {
-    NSArray *skipPaths = [fswupdateSkipBuff allKeys];
-    NSDate *now = [NSDate date];
-    unsigned i;
-    
-    RETAIN (skipPaths);
-    
-    for (i = 0; i < [skipPaths count]; i++) {
-      NSString *path = [skipPaths objectAtIndex: i];
-      NSDictionary *skipInfo = [fswupdateSkipBuff objectForKey: path];
-      NSDate *stamp = [skipInfo objectForKey: @"stamp"];
-      
-      if ([now timeIntervalSinceDate: stamp] > SKIP_EXPIRE) {
-        [fswupdateSkipBuff removeObjectForKey: path];
-   //     GWDebugLog(@"expired skip-info %@", path);
-      }
-    }
-    
-    RELEASE (skipPaths);
-  }  
-    
-  RELEASE (arp);  
 }
 
-- (void)connectFSWatcher
+- (void)connectFSWatcher:(id)sender
 {
   if (fswatcher == nil) {
-    id fsw = [NSConnection rootProxyForConnectionWithRegisteredName: @"fswatcher" 
-                                                               host: @""];
+    fswatcher = [NSConnection rootProxyForConnectionWithRegisteredName: @"fswatcher" 
+                                                                  host: @""];
 
-    if (fsw) {
-      NSConnection *c = [fsw connectionForProxy];
+    if (fswatcher == nil) {
+	    NSString *cmd;
+      int i;
+    
+      cmd = [[NSSearchPathForDirectoriesInDomains(
+                GSToolsDirectory, NSSystemDomainMask, YES) objectAtIndex: 0]
+                                      stringByAppendingPathComponent: @"fswatcher"];    
+                
+     [NSTask launchedTaskWithLaunchPath: cmd arguments: nil];
+   
+      for (i = 0; i < 40; i++) {
+	      [[NSRunLoop currentRunLoop] runUntilDate:
+		                     [NSDate dateWithTimeIntervalSinceNow: 0.1]];
 
-	    [nc addObserver: self
-	           selector: @selector(fswatcherConnectionDidDie:)
-		             name: NSConnectionDidDieNotification
-		           object: c];
-      
-      fswatcher = fsw;
-	    [fswatcher setProtocolForProxy: @protocol(FSWatcherProtocol)];
+        fswatcher = [NSConnection rootProxyForConnectionWithRegisteredName: @"fswatcher" 
+                                                                      host: @""];                  
+        if (fswatcher) {
+          break;
+        }
+      }
+    }
+    
+    if (fswatcher) {
       RETAIN (fswatcher);
-                                   
+      [fswatcher setProtocolForProxy: @protocol(FSWatcherProtocol)];
+    
+	    [[NSNotificationCenter defaultCenter] addObserver: self
+	                   selector: @selector(fswatcherConnectionDidDie:)
+		                     name: NSConnectionDidDieNotification
+		                   object: [fswatcher connectionForProxy]];
+                       
 	    [fswatcher registerClient: (id <FSWClientProtocol>)self 
                 isGlobalWatcher: YES];
-      
-      NSLog(@"fswatcher connected!");
-      
-	  } else {
-	    static BOOL recursion = NO;
-	    static NSString	*cmd = nil;
 
-	    if (recursion == NO) {
-        if (cmd == nil) {
-          cmd = RETAIN ([[NSSearchPathForDirectoriesInDomains(
-                      GSToolsDirectory, NSSystemDomainMask, YES) objectAtIndex: 0]
-                            stringByAppendingPathComponent: @"fswatcher"]);
-		    }
-      }
-	  
-      if (recursion == NO && cmd != nil) {
-        int i;
-        
-	      [NSTask launchedTaskWithLaunchPath: cmd arguments: nil];
-        DESTROY (cmd);
-        
-        for (i = 1; i <= 40; i++) {
-	        [[NSRunLoop currentRunLoop] runUntilDate:
-		                       [NSDate dateWithTimeIntervalSinceNow: 0.1]];
-                           
-          fsw = [NSConnection rootProxyForConnectionWithRegisteredName: @"fswatcher" 
-                                                                  host: @""];                  
-          if (fsw) {
-            break;
-          }
-        }
-        
-	      recursion = YES;
-	      [self connectFSWatcher];
-	      recursion = NO;
-        
-	    } else { 
-        DESTROY (cmd);
-	      recursion = NO;
-        NSLog(@"unable to contact fswatcher!");  
-      }
-	  }
+      NSLog(@"fswatcher connected!");                
+    } else {
+      NSLog(@"unable to contact fswatcher!");  
+    }
   }
 }
 
@@ -712,7 +748,105 @@ do { \
 
   NSLog(@"The fswatcher connection died!");
 
-  [self connectFSWatcher];                
+  [NSTimer scheduledTimerWithTimeInterval: 5.0
+						                       target: self 
+                                 selector: @selector(connectFSWatcher:) 
+															   userInfo: nil 
+                                  repeats: NO];
+}
+
+@end
+
+
+@implementation GMDSExtractor (ddbd_update)
+
+- (void)setupDDBdUpdater
+{
+  ddbd = nil;
+  [self connectDDBd];
+
+  [dnc addObserver: self
+          selector: @selector(userAttributeModified:)
+	            name: @"GSMetadataUserAttributeModifiedNotification"
+	          object: nil];
+}
+
+- (void)connectDDBd
+{
+  if (ddbd == nil) {
+    ddbd = [NSConnection rootProxyForConnectionWithRegisteredName: @"ddbd" 
+                                                             host: @""];
+
+    if (ddbd == nil) {
+	    NSString *cmd;
+      int i;
+    
+      cmd = [[NSSearchPathForDirectoriesInDomains(
+                GSToolsDirectory, NSSystemDomainMask, YES) objectAtIndex: 0]
+                                      stringByAppendingPathComponent: @"ddbd"];    
+                
+     [NSTask launchedTaskWithLaunchPath: cmd arguments: nil];
+   
+      for (i = 0; i < 40; i++) {
+	      [[NSRunLoop currentRunLoop] runUntilDate:
+		                     [NSDate dateWithTimeIntervalSinceNow: 0.1]];
+
+        ddbd = [NSConnection rootProxyForConnectionWithRegisteredName: @"ddbd" 
+                                                                 host: @""];                  
+        if (ddbd) {
+          break;
+        }
+      }
+    }
+    
+    if (ddbd) {
+      RETAIN (ddbd);
+      [ddbd setProtocolForProxy: @protocol(DDBdProtocol)];
+    
+	    [[NSNotificationCenter defaultCenter] addObserver: self
+	                   selector: @selector(ddbdConnectionDidDie:)
+		                     name: NSConnectionDidDieNotification
+		                   object: [ddbd connectionForProxy]];
+    
+      NSLog(@"ddbd connected!");    
+    } else {
+      NSLog(@"unable to contact ddbd!");  
+    }
+  }
+}
+
+- (void)ddbdConnectionDidDie:(NSNotification *)notif
+{
+  id connection = [notif object];
+
+  [nc removeObserver: self
+	              name: NSConnectionDidDieNotification
+	            object: connection];
+
+  NSAssert(connection == [ddbd connectionForProxy],
+		                                  NSInternalInconsistencyException);
+  RELEASE (ddbd);
+  ddbd = nil;
+
+  NSLog(@"The ddbd connection died!");
+
+  [self connectDDBd];                
+}
+
+- (void)userAttributeModified:(NSNotification *)notif
+{
+  if (extracting == NO) {
+    NSString *path = [notif object];
+    NSString *ext = [[path pathExtension] lowercaseString];
+
+    if (([excludedSuffixes containsObject: ext] == NO)
+              && (isDotFile(path) == NO)
+              && inTreeFirstPartOfPath(path, includePathsTree)
+              && (inTreeFirstPartOfPath(path, excludedPathsTree) == NO)) {
+      GWDebugLog(@"ddbd_update: %@", path);        
+      [self updatePath: path];
+    }
+  }
 }
 
 @end
@@ -747,19 +881,22 @@ do { \
 
 - (void)checkNextDir:(id)sender
 {
-  unsigned count = [directories count];
-
-  if (extracting) {
-    return;
-  }
-    
-  if (dirpos < count) {
+  if (extracting == NO) {
     CREATE_AUTORELEASE_POOL(arp);
-    NSString *dir = [directories objectAtIndex: dirpos];
-    NSDictionary *attributes = [fm fileAttributesAtPath: dir traverseLink: NO];
-    BOOL dirok = (attributes && ([attributes fileType] == NSFileTypeDirectory));
+    unsigned count = [directories count];
+    NSString *dir;
+    NSDictionary *attributes;
+    BOOL dirok;
     unsigned i;
-    
+  
+    if ((dirpos >= count) || (dirpos < 0)) {
+      dirpos = 0;
+    }
+  
+    dir = [directories objectAtIndex: dirpos];
+    attributes = [fm fileAttributesAtPath: dir traverseLink: NO];
+    dirok = (attributes && ([attributes fileType] == NSFileTypeDirectory));
+  
     if (dirok) {
       NSArray *contents = [self filteredDirectoryContentsAtPath: dir escapeEntries: YES];
       NSMutableDictionary *dbcontents = [NSMutableDictionary dictionary];
@@ -804,7 +941,7 @@ do { \
         if (dbdate == nil) {
           GWDebugLog(@"schedule-add %@", path);
           [self addPath: path];
-        
+          
         } else {
           NSDictionary *attrs = [fm fileAttributesAtPath: path traverseLink: NO];
           NSTimeInterval date = [[attrs fileModificationDate] timeIntervalSinceReferenceDate];
@@ -869,6 +1006,76 @@ do { \
 @end
 
 
+@implementation GMDSExtractor (update_notifications)
+
+- (void)setupUpdateNotifications
+{
+  ASSIGN (notifDate, [NSDate date]);
+  
+  notificationsTimer = [NSTimer scheduledTimerWithTimeInterval: NOTIF_TIME
+						                                 target: self 
+                                           selector: @selector(notifyUpdates:) 
+																           userInfo: nil 
+                                            repeats: YES];
+  RETAIN (notificationsTimer);     
+}
+
+- (void)notifyUpdates:(id)sender
+{
+  if (extracting == NO) {
+    CREATE_AUTORELEASE_POOL(arp);
+    NSMutableArray *removed = [NSMutableArray array];
+    NSTimeInterval lastStamp;
+    NSString *query;
+    NSArray *results;
+    NSDictionary *info;
+    unsigned i;
+
+    [sqlite executeQuery: @"BEGIN"];
+
+    query = @"SELECT path FROM removed_paths;";
+    results = [sqlite resultsOfQuery: query];
+
+    query = @"DELETE FROM removed_paths;";
+    [sqlite executeQuery: query];
+
+    lastStamp = [notifDate timeIntervalSinceReferenceDate];
+    query = [NSString stringWithFormat: @"DELETE FROM updated_paths "
+                                        @"WHERE timestamp < %f;", lastStamp];
+    [sqlite executeQuery: query];
+
+    [sqlite executeQuery: @"COMMIT"];
+
+    ASSIGN (notifDate, [NSDate date]);
+
+    for (i = 0; i < [results count]; i++) {
+      [removed addObject: [[results objectAtIndex: i] objectForKey: @"path"]];
+    }
+
+    info = [NSDictionary dictionaryWithObject: [removed makeImmutableCopyOnFail: NO]
+                                       forKey: @"removed"];
+
+    [dnc postNotificationName: @"GWMetadataDidUpdateNotification"
+                       object: nil 
+                     userInfo: info];
+
+    RELEASE (arp);
+  }
+}
+
+@end
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 
