@@ -53,6 +53,7 @@ static GWDesktopManager *desktopManager = nil;
   TEST_RELEASE (dskNode);
   TEST_RELEASE (win);
   TEST_RELEASE (dock);
+  RELEASE (mpointWatcher);
     
 	[super dealloc];
 }
@@ -66,13 +67,13 @@ static GWDesktopManager *desktopManager = nil;
     id defentry;
     NSString *path;
     id window = nil;
-    GWDesktopView *desktopView;
 
     fm = [NSFileManager defaultManager];
     nc = [NSNotificationCenter defaultCenter];
     ws = [NSWorkspace sharedWorkspace];
     gworkspace = [GWorkspace gworkspace];
     fsnodeRep = [FSNodeRep sharedInstance];
+    mpointWatcher = [[MPointWatcher alloc] initForManager: self];
     
     [self checkDesktopDirs];
 
@@ -147,37 +148,24 @@ static GWDesktopManager *desktopManager = nil;
 
 - (void)activateDesktop
 {
-  NSSet *volumes = [fsnodeRep volumes];
-  NSEnumerator *enumerator = [volumes objectEnumerator];
-  NSString *vpath;
-  
   [win activate];
-  [[win desktopView] showMountedVolumes];
-  [[win desktopView] showContentsOfNode: dskNode];
+  [desktopView showMountedVolumes];
+  [desktopView showContentsOfNode: dskNode];
   [self addWatcherForPath: [dskNode path]];
     
   if ((hidedock == NO) && ([dock superview] == nil)) {
-    [[win desktopView] addSubview: dock];
+    [desktopView addSubview: dock];
     [dock tile];
   }
   
-  while ((vpath = [enumerator nextObject])) {
-    [self addWatcherForPath: vpath];
-  }
+  [mpointWatcher startWatching];  
 }
 
 - (void)deactivateDesktop
 {
-  NSSet *volumes = [fsnodeRep volumes];
-  NSEnumerator *enumerator = [volumes objectEnumerator];
-  NSString *vpath;
-
   [win deactivate];
-  [self removeWatcherForPath: [dskNode path]];
-
-  while ((vpath = [enumerator nextObject])) {
-    [self removeWatcherForPath: vpath];
-  }
+  [self removeWatcherForPath: [dskNode path]];  
+  [mpointWatcher stopWatching];
 }
 
 - (BOOL)isActive
@@ -221,7 +209,6 @@ static GWDesktopManager *desktopManager = nil;
   usexbundle = value;
   
   if ([self isActive]) { 
-    GWDesktopView *desktopView = [win desktopView];
     id window = nil;  
     BOOL changed = NO;
     
@@ -286,7 +273,7 @@ static GWDesktopManager *desktopManager = nil;
 
 - (id)desktopView
 {
-  return [win desktopView];
+  return desktopView;
 }
 
 - (Dock *)dock
@@ -304,7 +291,7 @@ static GWDesktopManager *desktopManager = nil;
   dockPosition = pos;
   [dock setPosition: pos];
   [self setReservedFrames];
-  [[win desktopView] dockPositionDidChange];
+  [desktopView dockPositionDidChange];
 }
 
 - (void)setDockActive:(BOOL)value
@@ -313,12 +300,12 @@ static GWDesktopManager *desktopManager = nil;
   
   if (hidedock && [dock superview]) {
     [dock removeFromSuperview];
-    [[win desktopView] setNeedsDisplayInRect: dockReservedFrame];
+    [desktopView setNeedsDisplayInRect: dockReservedFrame];
     
   } else if ([dock superview] == nil) {
-    [[win desktopView] addSubview: dock];
+    [desktopView addSubview: dock];
     [dock tile];
-    [[win desktopView] setNeedsDisplayInRect: dockReservedFrame];
+    [desktopView setNeedsDisplayInRect: dockReservedFrame];
   }
 }
 
@@ -377,7 +364,7 @@ static GWDesktopManager *desktopManager = nil;
 
 - (NSImage *)tabbedShelfBackground
 {
-  return [[win desktopView] tshelfBackground];
+  return [desktopView tshelfBackground];
 }
 
 - (void)mouseEnteredTShelfActivateFrame
@@ -392,11 +379,9 @@ static GWDesktopManager *desktopManager = nil;
 
 - (void)deselectAllIcons
 {
-  GWDesktopView *view = [win desktopView];
-  
-  [view unselectOtherReps: nil];
-  [view selectionDidChange];
-  [view stopRepNameEditing];
+  [desktopView unselectOtherReps: nil];
+  [desktopView selectionDidChange];
+  [desktopView stopRepNameEditing];
 }
 
 - (void)deselectInSpatialViewers
@@ -490,10 +475,7 @@ static GWDesktopManager *desktopManager = nil;
   NSString *path = [info objectForKey: @"path"];
   NSString *event = [info objectForKey: @"event"];
   
-  if ([[fsnodeRep volumes] containsObject: path]) {  
-    [[self desktopView] showMountedVolumes];
-  
-  } else if ([path isEqual: [dskNode path]]) {
+  if ([path isEqual: [dskNode path]]) {
     if ([event isEqual: @"GWWatchedPathDeleted"]) {
       NSRunAlertPanel(nil, 
                       NSLocalizedString(@"The Desktop directory has been deleted! Quiting now!", @""), 
@@ -518,6 +500,7 @@ static GWDesktopManager *desktopManager = nil;
 - (void)removableMediaPathsDidChange
 {
   [[self desktopView] showMountedVolumes];
+  [mpointWatcher startWatching];
 }
 
 - (void)hideDotsFileDidChange:(BOOL)hide
@@ -562,6 +545,11 @@ static GWDesktopManager *desktopManager = nil;
   }
 }
 
+- (void)mountedVolumesDidChange
+{
+  [[self desktopView] showMountedVolumes];
+}
+
 - (void)updateDefaults
 {
   NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
@@ -573,7 +561,7 @@ static GWDesktopManager *desktopManager = nil;
   [defaults setBool: hidedock forKey: @"hidedock"];
   
   [dock updateDefaults];
-  [[win desktopView] updateDefaults];
+  [desktopView updateDefaults];
 }
 
 - (void)setContextHelp
@@ -604,21 +592,20 @@ static GWDesktopManager *desktopManager = nil;
 - (BOOL)validateItem:(id)menuItem
 {
   if ([self isActive]) {
-    NSString *itemTitle = [menuItem title];
-    GWDesktopView *desktopView = [win desktopView];
+    SEL action = [menuItem action];
 
-    if ([itemTitle isEqual: NSLocalizedString(@"Duplicate", @"")]
-        || [itemTitle isEqual: NSLocalizedString(@"Move to Recycler", @"")]
-        || [itemTitle isEqual: NSLocalizedString(@"Destroy", @"")]) {
+    if (sel_eq(action, @selector(duplicateFiles:))
+                || sel_eq(action, @selector(recycleFiles:))
+                      || sel_eq(action, @selector(deleteFiles:))) {
       return ([[desktopView selectedNodes] count] > 0);
 
-    } else if ([itemTitle isEqual: NSLocalizedString(@"Open", @"")]) {
+    } else if (sel_eq(action, @selector(openSelection:))) {
       NSArray *selection = [desktopView selectedNodes];
      
       return (selection && [selection count] 
             && ([selection isEqual: [NSArray arrayWithObject: dskNode]] == NO));
     
-    } else if ([itemTitle isEqual: NSLocalizedString(@"Open With...", @"")]) {
+    } else if (sel_eq(action, @selector(openWith:))) {
       NSArray *selection = [desktopView selectedNodes];
       BOOL canopen = YES;
       int i;
@@ -640,7 +627,7 @@ static GWDesktopManager *desktopManager = nil;
 
       return canopen;
       
-    } else if ([itemTitle isEqual: NSLocalizedString(@"Open as Folder", @"")]) {
+    } else if (sel_eq(action, @selector(openSelectionAsFolder:))) {
       NSArray *selection = [desktopView selectedNodes];
     
       if (selection && ([selection count] == 1)) {  
@@ -658,7 +645,7 @@ static GWDesktopManager *desktopManager = nil;
 
 - (void)openSelectionInNewViewer:(BOOL)newv
 {
-  NSArray *selreps = [[win desktopView] selectedReps];
+  NSArray *selreps = [desktopView selectedReps];
   int i;
     
   for (i = 0; i < [selreps count]; i++) {
@@ -705,7 +692,7 @@ static GWDesktopManager *desktopManager = nil;
 
 - (void)openSelectionAsFolder
 {
-  NSArray *selnodes = [[win desktopView] selectedNodes];
+  NSArray *selnodes = [desktopView selectedNodes];
   int i;
     
   for (i = 0; i < [selnodes count]; i++) {
@@ -736,14 +723,14 @@ static GWDesktopManager *desktopManager = nil;
 
 - (void)duplicateFiles
 {
-  if ([[[win desktopView] selectedNodes] count]) {
+  if ([[desktopView selectedNodes] count]) {
     [gworkspace duplicateFiles];
   }
 }
 
 - (void)recycleFiles
 {
-  if ([[[win desktopView] selectedNodes] count]) {
+  if ([[desktopView selectedNodes] count]) {
     [gworkspace moveToTrash];
   }
 }
@@ -755,7 +742,7 @@ static GWDesktopManager *desktopManager = nil;
 
 - (void)deleteFiles
 {
-  if ([[[win desktopView] selectedNodes] count]) {
+  if ([[desktopView selectedNodes] count]) {
     [gworkspace deleteFiles];
   }
 }
@@ -779,17 +766,17 @@ static GWDesktopManager *desktopManager = nil;
     type = FSNInfoNameType;
   } 
 
-  [(id <FSNodeRepContainer>)[win desktopView] setShowType: type];  
+  [desktopView setShowType: type];  
 }
 
 - (void)setExtendedShownType:(id)sender
 {
-  [(id <FSNodeRepContainer>)[win desktopView] setExtendedShowType: [sender title]]; 
+  [desktopView setExtendedShowType: [sender title]]; 
 }
 
 - (void)setIconsSize:(id)sender
 {
-  [(id <FSNodeRepContainer>)[win desktopView] setIconSize: [[sender title] intValue]];
+  [desktopView setIconSize: [[sender title] intValue]];
 }
 
 - (void)setIconsPosition:(id)sender
@@ -797,20 +784,20 @@ static GWDesktopManager *desktopManager = nil;
   NSString *title = [sender title];
 
   if ([title isEqual: NSLocalizedString(@"Left", @"")]) {
-    [(id <FSNodeRepContainer>)[win desktopView] setIconPosition: NSImageLeft];
+    [desktopView setIconPosition: NSImageLeft];
   } else {
-    [(id <FSNodeRepContainer>)[win desktopView] setIconPosition: NSImageAbove];
+    [desktopView setIconPosition: NSImageAbove];
   }
 }
 
 - (void)setLabelSize:(id)sender
 {
-  [[win desktopView] setLabelTextSize: [[sender title] intValue]];
+  [desktopView setLabelTextSize: [[sender title] intValue]];
 }
 
 - (void)selectAllInViewer
 {
-	[[win desktopView] selectAll];
+	[desktopView selectAll];
 }
 
 - (void)showTerminal
@@ -821,6 +808,106 @@ static GWDesktopManager *desktopManager = nil;
 @end
 
 
+@implementation MPointWatcher
+
+- (void)dealloc
+{
+	if (timer && [timer isValid]) {
+		[timer invalidate];
+	}  
+  RELEASE (volinfo);  
+	[super dealloc];
+}
+
+- (id)initForManager:(GWDesktopManager *)mngr
+{
+  self = [super init];
+  
+  if (self) {
+    manager = mngr;    
+    volinfo = [NSMutableArray new];
+    active = NO;
+    fm = [NSFileManager defaultManager];
+    
+    timer = [NSTimer scheduledTimerWithTimeInterval: 1.0 
+												                     target: self 
+                                           selector: @selector(watchMountPoints:) 
+										                       userInfo: nil 
+                                            repeats: YES];
+  }
+  
+  return self;
+}
+
+- (void)startWatching
+{
+  NSSet *volumes = [[FSNodeRep sharedInstance] volumes];
+  NSEnumerator *enumerator = [volumes objectEnumerator];
+  NSString *path;
+  
+  [volinfo removeAllObjects];
+
+  while ((path = [enumerator nextObject])) {
+    NSDictionary *attributes = [fm fileAttributesAtPath: path traverseLink: NO];
+    
+    if (attributes) {
+      NSDate *moddate = [attributes fileModificationDate];
+      NSMutableDictionary *dict = [NSMutableDictionary dictionary];
+      
+      [dict setObject: path forKey: @"path"];
+      [dict setObject: moddate forKey: @"moddate"];
+      
+      [volinfo addObject: dict];  
+    }
+  }
+
+  active = YES;
+}
+
+- (void)stopWatching
+{
+  active = NO;
+  [volinfo removeAllObjects];
+}
+
+- (void)watchMountPoints:(id)sender
+{
+  if (active) {
+    int count = [volinfo count];
+    BOOL changed = NO;
+    int i;
+    
+    for (i = 0; i < count; i++) {
+      NSMutableDictionary *dict = [volinfo objectAtIndex: i];
+      NSString *path = [dict objectForKey: @"path"];
+      NSDate *moddate = [dict objectForKey: @"moddate"];
+      NSDictionary *attributes = [fm fileAttributesAtPath: path traverseLink: NO];      
+      
+      if (attributes) {
+        NSDate *lastmod = [attributes fileModificationDate];
+      
+        if ([moddate isEqualToDate: lastmod] == NO) {
+          [dict setObject: lastmod forKey: @"moddate"];
+          changed = YES;
+        }
+        
+      } else {
+        [volinfo removeObjectAtIndex: i]; 
+        count--;
+        i--;
+        changed = YES;       
+      }      
+    }
+    
+    if (changed) {
+      [manager mountedVolumesDidChange]; 
+    }
+  }
+}
+
+@end
+
+
 @implementation GWMounter
 
 + (void)mountRemovableMedia
