@@ -292,7 +292,11 @@
 		}
   }
   
-  return [icns count] ? icns : nil;
+  if ([icns count]) {
+    return icns;
+  }
+  
+  return nil;
 }
 
 - (NSArray *)iconsWithGridOriginY:(float)y
@@ -308,8 +312,12 @@
       [icns addObject: icon];
 		}
   }
+
+  if ([icns count]) {
+    return icns;
+  }
   
-  return [icns count] ? icns : nil;
+  return nil;
 }
 
 - (int)indexOfGridRectContainingPoint:(NSPoint)p
@@ -726,7 +734,10 @@
 
 - (void)mouseDown:(NSEvent *)theEvent
 {
-  [[self window] makeKeyWindow];
+  NSWindow *win = [self window];
+  
+  [win makeMainWindow];
+  [win makeKeyWindow];
   
   if ([theEvent modifierFlags] != NSShiftKeyMask) {
     selectionMask = NSSingleSelectionMask;
@@ -1310,10 +1321,10 @@
 - (void)selectionDidChange
 {
 	if (!(selectionMask & FSNCreatingSelectionMask)) {
-    NSArray *selection = [self selectedPaths];
+    NSArray *selection = [self selectedNodes];
 		
     if ([selection count] == 0) {
-      selection = [NSArray arrayWithObject: [node path]];
+      selection = [NSArray arrayWithObject: node];
     } else {
       [manager deselectInSpatialViewers];
     }
@@ -1487,6 +1498,27 @@
     prePath = [prePath stringByDeletingLastPathComponent];
   }
 
+  if ([node isDirectory] && [node isParentOfPath: basePath]) {
+    NSArray *subNodes = [node subNodes];
+    int i;
+    
+    for (i = 0; i < [subNodes count]; i++) {
+      FSNode *nd = [subNodes objectAtIndex: i];
+      
+      if ([nd isDirectory]) {
+        int j;
+        
+        for (j = 0; j < count; j++) {
+          NSString *fname = [[sourcePaths objectAtIndex: j] lastPathComponent];
+          
+          if ([[nd name] isEqual: fname]) {
+            return NSDragOperationNone;
+          }
+        }
+      }
+    }
+  }	
+
   isDragTarget = YES;	
   forceCopy = NO;
   dragPoint = NSZeroPoint;
