@@ -57,7 +57,7 @@ static NSString *nibName = @"RunExternal";
       ASSIGN (pathsArr, [paths componentsSeparatedByString: @":"]);
   
 		  cfield = [[CompletionField alloc] initForController: self];
-      [cfield setFrame: [[fieldBox contentView] frame]];
+      [cfield setFrame: [[fieldBox contentView] bounds]];
       [cfield setNextKeyView: okButt]; 
       [fieldBox setContentView: cfield];      
       RELEASE (cfield);
