@@ -98,7 +98,8 @@
   win = [[NSWindow alloc] initWithContentRect: NSMakeRect(0, 0, 64, 64)
 					                  styleMask: NSBorderlessWindowMask  
                               backing: NSBackingStoreBuffered defer: NO];
-  [self setFrame: [[win contentView] frame]];  
+  [win setExcludedFromWindowsMenu: YES];
+  [self setFrame: [[win contentView] bounds]];  
   [win setContentView: self];		
   [self setPosX: px posY: py relativeToPoint: p];
   
@@ -433,6 +434,26 @@
 		    buff = [buff stringByDeletingLastPathComponent];
 	    }
 
+      if ([node isDirectory] && [node isParentOfPath: fromPath]) {
+        NSArray *subNodes = [node subNodes];
+
+        for (i = 0; i < [subNodes count]; i++) {
+          FSNode *nd = [subNodes objectAtIndex: i];
+
+          if ([nd isDirectory]) {
+            int j;
+
+            for (j = 0; j < count; j++) {
+              NSString *fname = [[sourcePaths objectAtIndex: j] lastPathComponent];
+
+              if ([[nd name] isEqual: fname]) {
+                return NSDragOperationNone;
+              }
+            }
+          }
+        }
+      }	
+
       isDragTarget = YES;
       forceCopy = NO;
 			
