@@ -41,6 +41,9 @@
 #define DEFAULT_INCR 150
 #define MIN_W_HEIGHT 180
 
+#define OPEN_MAX 10
+
+
 @implementation GWSpatialViewer
 
 - (void)dealloc
@@ -48,11 +51,13 @@
   [nc removeObserver: self];
   
   RELEASE (baseNode);
+  RELEASE (baseNodeArray);
   TEST_RELEASE (lastSelection);
   TEST_RELEASE (rootViewerKey);
   RELEASE (watchedNodes);
   RELEASE (vwrwin);
   RELEASE (viewType);
+  RELEASE (viewerPrefs);
   
 	[super dealloc];
 }
@@ -66,11 +71,11 @@
   
   if (self) {
     NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];	
-    NSDictionary *viewerPrefs = nil;
     NSString *prefsname;
     id defEntry;
 
     ASSIGN (baseNode, [FSNode nodeWithPath: [node path]]);
+    ASSIGN (baseNodeArray, [NSArray arrayWithObject: baseNode]);
     fsnodeRep = [FSNodeRep sharedInstance];
     lastSelection = nil;
     watchedNodes = [NSMutableArray new];
@@ -174,7 +179,7 @@
       nodeView = [[GWViewerIconsView alloc] initForViewer: self];
 
     } else if ([viewType isEqual: @"List"]) { 
-      NSRect r = [[scroll contentView] frame];
+      NSRect r = [[scroll contentView] bounds];
       
       nodeView = [[GWViewerListView alloc] initWithFrame: r forViewer: self];
 
@@ -225,13 +230,14 @@
     [self updeateInfoLabels];
 
     [self scrollToBeginning];
-    
-    RELEASE (viewerPrefs);
-    
+        
     [nc addObserver: self 
            selector: @selector(columnsWidthChanged:) 
                name: @"GWBrowserColumnWidthChangedNotification"
              object: nil];
+    
+    invalidated = NO;
+    closing = NO;    
   }
   
   return self;
@@ -239,7 +245,7 @@
 
 - (void)createSubviews
 {
-  NSRect r = [[vwrwin contentView] frame];
+  NSRect r = [[vwrwin contentView] bounds];
   float w = r.size.width;
 	float h = r.size.height; 
   int boxh = 32;  
@@ -421,6 +427,10 @@
   return invalidated;
 }
 
+- (BOOL)isClosing
+{
+  return closing;
+}
 
 - (void)setOpened:(BOOL)opened 
         repOfNode:(FSNode *)anode
@@ -447,6 +457,10 @@
 {
   FSNode *node;
   NSArray *components;
+
+  if (closing) {
+    return;
+  }
   
   [manager selectionChanged: newsel];
 
@@ -457,7 +471,7 @@
   ASSIGN (lastSelection, newsel);
   [self updeateInfoLabels]; 
     
-  node = [FSNode nodeWithPath: [newsel objectAtIndex: 0]];   
+  node = [newsel objectAtIndex: 0];   
     
   if ([nodeView isSingleNode]) {    
     if ([node isEqual: baseNode] == NO) {
@@ -566,16 +580,18 @@
   if ([path isEqual: [baseNode path]] == NO) {
     FSNode *node = [FSNode nodeWithPath: path];
     BOOL close = [sender closeViewer];
-  
+    id vwr = nil;
+    
     if (close) {
       [pathsPopUp setTarget: nil];
+      vwr = self;
     }
   
     [manager newViewerOfType: SPATIAL
                     showType: nil
                      forNode: node 
                showSelection: NO
-              closeOldViewer: (close ? self : nil)
+              closeOldViewer: vwr
                     forceNew: NO];
   } else {
     [nodeView showContentsOfNode: baseNode]; 
@@ -666,6 +682,16 @@
 
     [updatedprefs setObject: viewType forKey: @"viewtype"];
 
+    defEntry = [viewerPrefs objectForKey: @"shelfheight"];
+    if (defEntry) {
+      [updatedprefs setObject: defEntry forKey: @"shelfheight"];
+    }
+
+    defEntry = [viewerPrefs objectForKey: @"shelfdicts"];
+    if (defEntry) {
+      [updatedprefs setObject: defEntry forKey: @"shelfdicts"];
+    }
+
     defEntry = [nodeView selectedPaths];
     if (defEntry) {
       [updatedprefs setObject: defEntry forKey: @"lastselection"];
@@ -694,6 +720,8 @@
     
       [defaults setObject: updatedprefs forKey: prefsname];
     }
+    
+    ASSIGN (viewerPrefs, [updatedprefs makeImmutableCopyOnFail: NO]);
   }
 }
 
@@ -707,7 +735,7 @@
 
 - (void)windowDidBecomeKey:(NSNotification *)aNotification
 {
-  NSArray *selection = [nodeView selectedPaths];  
+  NSArray *selection = [nodeView selectedNodes];  
   int count = [selection count];
   
   [vwrwin makeFirstResponder: nodeView]; 
@@ -715,12 +743,12 @@
   [manager selectedSpatialViewerChanged: self];
 
   if (count == 0) {
-    selection = [NSArray arrayWithObject: [[nodeView shownNode] path]];
+    selection = [NSArray arrayWithObject: [nodeView shownNode]];
     [manager synchronizeSelectionInParentOfViewer: self];
   
   } else if (count == 1) {
     if (([nodeView isSingleNode] == NO)
-             || ([[selection objectAtIndex: 0] isEqual: [baseNode path]])) {
+             || ([[selection objectAtIndex: 0] isEqual: baseNode])) {
       [manager synchronizeSelectionInParentOfViewer: self];
     }
     
@@ -738,12 +766,14 @@
 
 - (BOOL)windowShouldClose:(id)sender
 {
+  [manager updateDesktop];
 	return YES;
 }
 
 - (void)windowWillClose:(NSNotification *)aNotification
 {
   if (invalidated == NO) {
+    closing = YES;
     [self updateDefaults];
     [manager viewerWillClose: self]; 
   }
@@ -843,7 +873,7 @@
     if (selection && [selection count]) {
       if ([nodeView isSingleNode]) {
         [gworkspace duplicateFiles];
-      } else if ([selection isEqual: [NSArray arrayWithObject: baseNode]] == NO) {
+      } else if ([selection isEqual: baseNodeArray] == NO) {
         [gworkspace duplicateFiles];
       }
     }
@@ -864,7 +894,7 @@
     if (selection && [selection count]) {
       if ([nodeView isSingleNode]) {
         [gworkspace moveToTrash];
-      } else if ([selection isEqual: [NSArray arrayWithObject: baseNode]] == NO) {
+      } else if ([selection isEqual: baseNodeArray] == NO) {
         [gworkspace moveToTrash];
       }
     }
@@ -883,7 +913,7 @@
   if (selection && [selection count]) {
     if ([nodeView isSingleNode]) {
       [gworkspace deleteFiles];
-    } else if ([selection isEqual: [NSArray arrayWithObject: baseNode]] == NO) {
+    } else if ([selection isEqual: baseNodeArray] == NO) {
       [gworkspace deleteFiles];
     }
   }
@@ -909,14 +939,27 @@
   NSString *title = [sender title];
   
 	if ([title isEqual: NSLocalizedString(viewType, @"")] == NO) {
-    NSArray *selection = [nodeView selectedPaths];
+    NSArray *selection = [nodeView selectedNodes];
+    NSArray *reps = [nodeView reps];
+    NSMutableArray *opennodes = [NSMutableArray array];
     int i;
     
     [nodeView updateNodeInfo: YES];
     if ([nodeView isSingleNode] && ([selection count] == 0)) {
-      selection = [NSArray arrayWithObject: [[nodeView shownNode] path]];
+      selection = [NSArray arrayWithObject: [nodeView shownNode]];
     }    
     RETAIN (selection);
+    
+    for (i = 0; i < [reps count]; i++) {
+      id rep = [reps objectAtIndex: i];
+
+      if ([rep isOpened]) {
+        [opennodes addObject: [rep node]];
+      }
+    }
+
+    RETAIN (opennodes);
+    
     [scroll setDocumentView: nil];	
     
     if ([title isEqual: NSLocalizedString(@"Browser", @"")]) {
@@ -937,7 +980,7 @@
       ASSIGN (viewType, @"Icon");
       
     } else if ([title isEqual: NSLocalizedString(@"List", @"")]) {
-      NSRect r = [[scroll contentView] frame];
+      NSRect r = [[scroll contentView] bounds];
 
       nodeView = [[GWViewerListView alloc] initWithFrame: r forViewer: self];
       [scroll setHasVerticalScroller: YES];
@@ -947,9 +990,19 @@
     [scroll setDocumentView: nodeView];	
     RELEASE (nodeView);                 
     [nodeView showContentsOfNode: baseNode]; 
+        
+    for (i = 0; i < [opennodes count]; i++) {
+      id rep = [nodeView repOfSubnode: [opennodes objectAtIndex: i]];
+
+      if (rep) {
+        [rep setOpened: YES];
+      }
+    }
+    
+    RELEASE (opennodes);
     
     if ([selection count]) {
-      [nodeView selectRepsOfPaths: selection];
+      [nodeView selectRepsOfSubnodes: selection];
     }
     
     DESTROY (selection);
@@ -964,10 +1017,10 @@
     [watchedNodes removeAllObjects];
     
     DESTROY (lastSelection);
-    selection = [nodeView selectedPaths];
+    selection = [nodeView selectedNodes];
     
     if ([selection count] == 0) {
-      selection = [NSArray arrayWithObject: [[nodeView shownNode] path]];
+      selection = [NSArray arrayWithObject: [nodeView shownNode]];
     }
     
     [self selectionChanged: selection];
@@ -1092,94 +1145,81 @@
 
 - (BOOL)validateItem:(id)menuItem
 {
-  NSString *itemTitle = [menuItem title];
-  NSString *menuTitle = [[menuItem menu] title];
+  if ([NSApp keyWindow] == vwrwin) {
+    SEL action = [menuItem action];
+    NSString *itemTitle = [menuItem title];
+    NSString *menuTitle = [[menuItem menu] title];
 
-  if ([menuTitle isEqual: NSLocalizedString(@"Icon Size", @"")]) {
-    return [nodeView respondsToSelector: @selector(setIconSize:)];
-  } else if ([menuTitle isEqual: NSLocalizedString(@"Icon Position", @"")]) {
-    return [nodeView respondsToSelector: @selector(setIconPosition:)];
-  } else if ([menuTitle isEqual: NSLocalizedString(@"Label Size", @"")]) {
-    return [nodeView respondsToSelector: @selector(setLabelTextSize:)];
-  } else if ([itemTitle isEqual: NSLocalizedString(@"Label Color...", @"")]) {
-    return [nodeView respondsToSelector: @selector(setTextColor:)];
-  } else if ([itemTitle isEqual: NSLocalizedString(@"Background Color...", @"")]) {
-    return [nodeView respondsToSelector: @selector(setBackgroundColor:)];
-  
-  } else if ([itemTitle isEqual: NSLocalizedString(@"Duplicate", @"")]
-       || [itemTitle isEqual: NSLocalizedString(@"Move to Recycler", @"")]
-       || [itemTitle isEqual: NSLocalizedString(@"Destroy", @"")]) {
-    NSArray *selection = [nodeView selectedNodes];
+    if ([menuTitle isEqual: NSLocalizedString(@"Icon Size", @"")]) {
+      return [nodeView respondsToSelector: @selector(setIconSize:)];
+    } else if ([menuTitle isEqual: NSLocalizedString(@"Icon Position", @"")]) {
+      return [nodeView respondsToSelector: @selector(setIconPosition:)];
+    } else if ([menuTitle isEqual: NSLocalizedString(@"Label Size", @"")]) {
+      return [nodeView respondsToSelector: @selector(setLabelTextSize:)];
+    } else if ([itemTitle isEqual: NSLocalizedString(@"Label Color...", @"")]) {
+      return [nodeView respondsToSelector: @selector(setTextColor:)];
+    } else if ([itemTitle isEqual: NSLocalizedString(@"Background Color...", @"")]) {
+      return [nodeView respondsToSelector: @selector(setBackgroundColor:)];
 
-    if (selection && [selection count]) {
-      if ([nodeView isSingleNode]) {
-        if ([itemTitle isEqual: NSLocalizedString(@"Move to Recycler", @"")]) {
-          return ([[baseNode path] isEqual: [gworkspace trashPath]] == NO);
-        } else {
-          return YES;
-        }
-        
-      } else if ([selection isEqual: [NSArray arrayWithObject: baseNode]] == NO) {
-        if ([itemTitle isEqual: NSLocalizedString(@"Move to Recycler", @"")]) {
-          return ([[baseNode path] isEqual: [gworkspace trashPath]] == NO);
-        } else {
-          return YES;
-        }
+    } else if (sel_eq(action, @selector(duplicateFiles:))
+                    || sel_eq(action, @selector(recycleFiles:))
+                        || sel_eq(action, @selector(deleteFiles:))) {
+      if (lastSelection && [lastSelection count]
+              && ([lastSelection isEqual: baseNodeArray] == NO)) {
+        return ([[baseNode path] isEqual: [gworkspace trashPath]] == NO);
       }
-    }
-    
-    return NO;
 
-  } else if ([itemTitle isEqual: NSLocalizedString(@"Open", @"")]) {
-    NSArray *selection = [nodeView selectedNodes];
+      return NO;
 
-    return (selection && [selection count] 
-            && ([selection isEqual: [NSArray arrayWithObject: baseNode]] == NO));
-    
-  } else if ([itemTitle isEqual: NSLocalizedString(@"Open as Folder", @"")]) {
-    NSArray *selection = [nodeView selectedNodes];
-    
-    if (selection && ([selection count] == 1)) {  
-      return [[selection objectAtIndex: 0] isPackage];
-    }
-    
-    return NO;
+    } else if (sel_eq(action, @selector(openSelection:))) {
+      if ([[baseNode path] isEqual: [gworkspace trashPath]] == NO) {
+        return (lastSelection && [lastSelection count] 
+                && ([lastSelection isEqual: baseNodeArray] == NO));
+      } 
 
-  } else if ([itemTitle isEqual: NSLocalizedString(@"Open With...", @"")]) {
-    NSArray *selection = [nodeView selectedNodes];
-    BOOL canopen = YES;
-    int i;
-        
-    if (selection && [selection count]
-          && ([selection isEqual: [NSArray arrayWithObject: baseNode]] == NO)) {
-      for (i = 0; i < [selection count]; i++) {
-        FSNode *node = [selection objectAtIndex: i];
-    
-        if (([node isPlain] == NO) 
-              && (([node isPackage] == NO) || [node isApplication])) {
-          canopen = NO;
-          break;
-        }
+      return NO;
+
+    } else if (sel_eq(action, @selector(openSelectionAsFolder:))) {
+      if (lastSelection && ([lastSelection count] == 1)) {  
+        return [[lastSelection objectAtIndex: 0] isPackage];
       }
-    } else {
-      canopen = NO;
-    }
-    
-    return canopen;
-    
-  } else if ([itemTitle isEqual: NSLocalizedString(@"New Folder", @"")]
-       || [itemTitle isEqual: NSLocalizedString(@"New File", @"")]) {
-    if ([[baseNode path] isEqual: [gworkspace trashPath]] == NO) {
-      return [[nodeView shownNode] isWritable];
-    }
-    
-    return NO;
 
-  } else if ([itemTitle isEqual: NSLocalizedString(@"Open", @"")]) {
-    return ([[baseNode path] isEqual: [gworkspace trashPath]] == NO);
+      return NO;
+
+    } else if (sel_eq(action, @selector(openWith:))) {
+      BOOL canopen = YES;
+      int i;
+
+      if (lastSelection && [lastSelection count]
+            && ([lastSelection isEqual: baseNodeArray] == NO)) {
+        for (i = 0; i < [lastSelection count]; i++) {
+          FSNode *node = [lastSelection objectAtIndex: i];
+
+          if (([node isPlain] == NO) 
+                && (([node isPackage] == NO) || [node isApplication])) {
+            canopen = NO;
+            break;
+          }
+        }
+      } else {
+        canopen = NO;
+      }
+
+      return canopen;
+
+    } else if (sel_eq(action, @selector(newFolder:))
+                                || sel_eq(action, @selector(newFile:))) {
+      if ([[baseNode path] isEqual: [gworkspace trashPath]] == NO) {
+        return [[nodeView shownNode] isWritable];
+      }
+
+      return NO;
+    } 
+    
+    return YES;
   }
-
-  return YES;
+  
+  return NO;
 }
 
 @end
@@ -1196,8 +1236,3 @@
 
 
 
-
-
-
-
-
