@@ -51,6 +51,9 @@
 #define COLLAPSE_LIMIT 35
 #define MID_LIMIT 110
 
+#define OPEN_MAX 10
+
+
 @implementation GWViewer
 
 - (void)dealloc
@@ -58,11 +61,13 @@
   [nc removeObserver: self];
 
   RELEASE (baseNode);
+  RELEASE (baseNodeArray);
   TEST_RELEASE (lastSelection);
   TEST_RELEASE (rootViewerKey);
   RELEASE (watchedNodes);
   RELEASE (vwrwin);
   RELEASE (viewType);
+  RELEASE (viewerPrefs);
   RELEASE (history);
   
   [super dealloc];
@@ -77,12 +82,12 @@
   
   if (self) {
     NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];	
-    NSDictionary *viewerPrefs = nil;
     NSString *prefsname;
     id defEntry;
     NSRect r;
         
     ASSIGN (baseNode, [FSNode nodeWithPath: [node path]]);
+    ASSIGN (baseNodeArray, [NSArray arrayWithObject: baseNode]);
     fsnodeRep = [FSNodeRep sharedInstance];
     lastSelection = nil;
     history = [NSMutableArray new];
@@ -145,7 +150,11 @@
     viewType = [viewerPrefs objectForKey: @"viewtype"];
     
     if (viewType == nil) {
-      viewType = ((stype != nil) ? stype : @"Browser");
+      if (stype != nil) {
+        viewType = stype;
+      } else {
+        viewType = @"Browser";
+      }
     }
     if (([viewType isEqual: @"Icon"] == NO)
               && ([viewType isEqual: @"List"] == NO)
@@ -222,7 +231,7 @@
       [pathsScroll setDelegate: pathsView];
       
     } else if ([viewType isEqual: @"List"]) { 
-      NSRect r = [[nviewScroll contentView] frame];
+      NSRect r = [[nviewScroll contentView] bounds];
       
       nodeView = [[GWViewerListView alloc] initWithFrame: r forViewer: self];
        
@@ -282,18 +291,14 @@
         RELEASE (selection);
       }
     }
-
-    [self updeateInfoLabels];
-    [self tileViews];
-
-    [self scrollToBeginning];
-    
-    RELEASE (viewerPrefs);
-    
+        
     [nc addObserver: self 
            selector: @selector(columnsWidthChanged:) 
                name: @"GWBrowserColumnWidthChangedNotification"
              object: nil];
+
+    invalidated = NO;
+    closing = NO;    
   }
   
   return self;
@@ -301,7 +306,7 @@
 
 - (void)createSubviews
 {
-  NSRect r = [[vwrwin contentView] frame];
+  NSRect r = [[vwrwin contentView] bounds];
   float w = r.size.width;
 	float h = r.size.height;   
   float d = 0.0;
@@ -330,7 +335,7 @@
   [split addSubview: lowBox];
   RELEASE (lowBox);
 
-  r = [lowBox frame];
+  r = [lowBox bounds];
   w = r.size.width;
   h = r.size.height; 
   
@@ -347,7 +352,7 @@
 
   visibleCols = myrintf(r.size.width / [vwrwin resizeIncrements].width);  
   
-  r = [[pathsScroll contentView] frame];
+  r = [[pathsScroll contentView] bounds];
   pathsView = [[GWViewerIconsPath alloc] initWithFrame: r 
                    visibleIcons: visibleCols forViewer: self
                    ownsScroller: ([viewType isEqual: @"Browser"] == NO)];
@@ -458,6 +463,7 @@
 {
   [vwrwin makeKeyAndOrderFront: nil];
   [self tileViews];
+  [self scrollToBeginning];    
 }
 
 - (void)deactivate
@@ -467,7 +473,7 @@
 
 - (void)tileViews
 {
-  NSRect r = [split frame];
+  NSRect r = [split bounds];
   float w = r.size.width;
 	float h = r.size.height;   
   float d = [split dividerThickness];
@@ -493,6 +499,11 @@
   return invalidated;
 }
 
+- (BOOL)isClosing
+{
+  return closing;
+}
+
 - (void)setOpened:(BOOL)opened 
         repOfNode:(FSNode *)anode
 {
@@ -518,10 +529,14 @@
   FSNode *node;
   NSArray *components;
 
+  if (closing) {
+    return;
+  }
+
   [manager selectionChanged: newsel];
 
   if (lastSelection && [newsel isEqual: lastSelection]) {
-    if ([[newsel objectAtIndex: 0] isEqual: [[nodeView shownNode] path]] == NO) {
+    if ([[newsel objectAtIndex: 0] isEqual: [nodeView shownNode]] == NO) {
       return;
     }
   }
@@ -529,7 +544,7 @@
   ASSIGN (lastSelection, newsel);
   [self updeateInfoLabels]; 
     
-  node = [FSNode nodeWithPath: [newsel objectAtIndex: 0]];   
+  node = [newsel objectAtIndex: 0];   
      
   if (([node isDirectory] == NO) || [node isPackage] || ([newsel count] > 1)) {
     if ([node isEqual: baseNode] == NO) { // if baseNode is a package 
@@ -594,7 +609,7 @@
     if ([nodeView isSingleNode]) {
       [nodeView showContentsOfNode: node];
       [self scrollToBeginning];
-      [self selectionChanged: [NSArray arrayWithObject: [node path]]];
+      [self selectionChanged: [NSArray arrayWithObject: node]];
       
     } else {
       [nodeView setLastShownNode: node];
@@ -876,6 +891,8 @@
     
       [defaults setObject: updatedprefs forKey: prefsname];
     }
+    
+    ASSIGN (viewerPrefs, [updatedprefs makeImmutableCopyOnFail: NO]);
   }
 }
 
@@ -939,16 +956,22 @@
 //
 @implementation GWViewer (GWViewerWindowDelegateMethods)
 
+- (void)windowDidExpose:(NSNotification *)aNotification
+{
+  [self updeateInfoLabels];
+}
+
 - (void)windowDidBecomeKey:(NSNotification *)aNotification
 {
-  NSArray *selection = [nodeView selectedPaths];  
+  NSArray *selection = [nodeView selectedNodes];  
     
   [vwrwin makeFirstResponder: nodeView];  
 
   if ([selection count] == 0) {
-    selection = [NSArray arrayWithObject: [[nodeView shownNode] path]];
+    selection = [NSArray arrayWithObject: [nodeView shownNode]];
   }
-
+  
+  [manager updateDesktop];
   [self selectionChanged: selection];
   [manager changeHistoryOwner: self];
 }
@@ -960,7 +983,7 @@
     [pathsView stopRepNameEditing];  
 
     if ([nodeView isSingleNode]) {
-      NSRect r = [[vwrwin contentView] frame];
+      NSRect r = [[vwrwin contentView] bounds];
       int cols = myrintf(r.size.width / [vwrwin resizeIncrements].width);  
 
       if (cols != visibleCols) {
@@ -972,12 +995,14 @@
 
 - (BOOL)windowShouldClose:(id)sender
 {
+  [manager updateDesktop];
 	return YES;
 }
 
 - (void)windowWillClose:(NSNotification *)aNotification
 {
   if (invalidated == NO) {
+    closing = YES;
     [self updateDefaults];
     [manager viewerWillClose: self]; 
   }
@@ -995,12 +1020,26 @@
 {
   if ([[baseNode path] isEqual: [gworkspace trashPath]] == NO) {
     NSArray *selection = [nodeView selectedNodes]; 
-
-    if (selection && [selection count]) {
+    int count = (selection ? [selection count] : 0);
+    
+    if (count) {
       NSMutableArray *dirs = [NSMutableArray array];
       int i;
 
-      for (i = 0; i < [selection count]; i++) {
+      if (count > OPEN_MAX) {
+        NSString *msg1 = NSLocalizedString(@"Are you sure you want to open", @"");
+        NSString *msg2 = NSLocalizedString(@"items?", @"");
+
+        if (NSRunAlertPanel(nil,
+                    [NSString stringWithFormat: @"%@ %i %@", msg1, count, msg2],
+                    NSLocalizedString(@"Cancel", @""),
+                    NSLocalizedString(@"Yes", @""),
+                    nil)) {
+          return;
+        }
+      }
+
+      for (i = 0; i < count; i++) {
         FSNode *node = [selection objectAtIndex: i];
 
         NS_DURING
@@ -1117,7 +1156,7 @@
     if (selection && [selection count]) {
       if ([nodeView isSingleNode]) {
         [gworkspace duplicateFiles];
-      } else if ([selection isEqual: [NSArray arrayWithObject: baseNode]] == NO) {
+      } else if ([selection isEqual: baseNodeArray] == NO) {
         [gworkspace duplicateFiles];
       }
     }
@@ -1138,7 +1177,7 @@
     if (selection && [selection count]) {
       if ([nodeView isSingleNode]) {
         [gworkspace moveToTrash];
-      } else if ([selection isEqual: [NSArray arrayWithObject: baseNode]] == NO) {
+      } else if ([selection isEqual: baseNodeArray] == NO) {
         [gworkspace moveToTrash];
       }
     }
@@ -1157,7 +1196,7 @@
   if (selection && [selection count]) {
     if ([nodeView isSingleNode]) {
       [gworkspace deleteFiles];
-    } else if ([selection isEqual: [NSArray arrayWithObject: baseNode]] == NO) {
+    } else if ([selection isEqual: baseNodeArray] == NO) {
       [gworkspace deleteFiles];
     }
   }
@@ -1183,12 +1222,12 @@
   NSString *title = [sender title];
   
 	if ([title isEqual: NSLocalizedString(viewType, @"")] == NO) {
-    NSArray *selection = [nodeView selectedPaths];
+    NSArray *selection = [nodeView selectedNodes];
     int i;
     
     [nodeView updateNodeInfo: YES];
     if ([nodeView isSingleNode] && ([selection count] == 0)) {
-      selection = [NSArray arrayWithObject: [[nodeView shownNode] path]];
+      selection = [NSArray arrayWithObject: [nodeView shownNode]];
     }
     RETAIN (selection);
     
@@ -1230,7 +1269,7 @@
       ASSIGN (viewType, @"Icon");
       
     } else if ([title isEqual: NSLocalizedString(@"List", @"")]) {
-      NSRect r = [[nviewScroll contentView] frame];
+      NSRect r = [[nviewScroll contentView] bounds];
 
       NSScroller *scroller = RETAIN ([pathsScroll horizontalScroller]);
 
@@ -1256,25 +1295,20 @@
                     
     if ([selection count]) {
       if ([nodeView isSingleNode]) {
-        NSString *base;
-        FSNode *basend;
-
-        base = [selection objectAtIndex: 0];
-        basend = [FSNode nodeWithPath: base];
+        FSNode *basend = [selection objectAtIndex: 0];
         
-        if ([base isEqual: [baseNode path]] == NO) {
+        if ([basend isEqual: baseNode] == NO) {
           if (([selection count] > 1) 
-                || (([basend isDirectory] == NO) || ([basend isPackage]))) {
-            base = [base stringByDeletingLastPathComponent];
-            basend = [FSNode nodeWithPath: base];
+                || (([basend isDirectory] == NO) || ([basend isPackage]))) {            
+            basend = [FSNode nodeWithPath: [basend parentPath]];
           }
         }
 
         [nodeView showContentsOfNode: basend];
-        [nodeView selectRepsOfPaths: selection];
+        [nodeView selectRepsOfSubnodes: selection];
 
       } else {
-        [nodeView selectRepsOfPaths: selection];
+        [nodeView selectRepsOfSubnodes: selection];
       }
     }
     
@@ -1290,10 +1324,10 @@
     [watchedNodes removeAllObjects];
     
     DESTROY (lastSelection);
-    selection = [nodeView selectedPaths];
+    selection = [nodeView selectedNodes];
     
     if ([selection count] == 0) {
-      selection = [NSArray arrayWithObject: [[nodeView shownNode] path]];
+      selection = [NSArray arrayWithObject: [nodeView shownNode]];
     }
     
     [self selectionChanged: selection];
@@ -1420,109 +1454,97 @@
 
 - (BOOL)validateItem:(id)menuItem
 {
-  NSString *itemTitle = [menuItem title];
-  NSString *menuTitle = [[menuItem menu] title];
+  if ([NSApp keyWindow] == vwrwin) {
+    SEL action = [menuItem action];
+    NSString *itemTitle = [menuItem title];
+    NSString *menuTitle = [[menuItem menu] title];
 
-  if ([menuTitle isEqual: NSLocalizedString(@"Icon Size", @"")]) {
-    return [nodeView respondsToSelector: @selector(setIconSize:)];
-  } else if ([menuTitle isEqual: NSLocalizedString(@"Icon Position", @"")]) {
-    return [nodeView respondsToSelector: @selector(setIconPosition:)];
-  } else if ([menuTitle isEqual: NSLocalizedString(@"Label Size", @"")]) {
-    return [nodeView respondsToSelector: @selector(setLabelTextSize:)];
-  } else if ([itemTitle isEqual: NSLocalizedString(@"Label Color...", @"")]) {
-    return [nodeView respondsToSelector: @selector(setTextColor:)];
-  } else if ([itemTitle isEqual: NSLocalizedString(@"Background Color...", @"")]) {
-    return [nodeView respondsToSelector: @selector(setBackgroundColor:)];
+    if ([menuTitle isEqual: NSLocalizedString(@"Icon Size", @"")]) {
+      return [nodeView respondsToSelector: @selector(setIconSize:)];
+    } else if ([menuTitle isEqual: NSLocalizedString(@"Icon Position", @"")]) {
+      return [nodeView respondsToSelector: @selector(setIconPosition:)];
+    } else if ([menuTitle isEqual: NSLocalizedString(@"Label Size", @"")]) {
+      return [nodeView respondsToSelector: @selector(setLabelTextSize:)];
+    } else if ([itemTitle isEqual: NSLocalizedString(@"Label Color...", @"")]) {
+      return [nodeView respondsToSelector: @selector(setTextColor:)];
+    } else if ([itemTitle isEqual: NSLocalizedString(@"Background Color...", @"")]) {
+      return [nodeView respondsToSelector: @selector(setBackgroundColor:)];
 
-  } else if ([itemTitle isEqual: NSLocalizedString(@"Duplicate", @"")]
-       || [itemTitle isEqual: NSLocalizedString(@"Move to Recycler", @"")]
-       || [itemTitle isEqual: NSLocalizedString(@"Destroy", @"")]) {
-    NSArray *selection = [nodeView selectedNodes];
+    } else if (sel_eq(action, @selector(duplicateFiles:))
+                    || sel_eq(action, @selector(recycleFiles:))
+                        || sel_eq(action, @selector(deleteFiles:))) {
+      if (lastSelection && [lastSelection count]
+              && ([lastSelection isEqual: baseNodeArray] == NO)) {
+        return ([[baseNode path] isEqual: [gworkspace trashPath]] == NO);
+      }
 
-    if (selection && [selection count]) {
-      if ([nodeView isSingleNode]) {
-        if ([itemTitle isEqual: NSLocalizedString(@"Move to Recycler", @"")]) {
-          return ([[baseNode path] isEqual: [gworkspace trashPath]] == NO);
+      return NO;
+
+    } else if (sel_eq(action, @selector(openSelection:))) {
+      if ([[baseNode path] isEqual: [gworkspace trashPath]] == NO) {
+        BOOL canopen = YES;
+        int i;
+
+        if (lastSelection && [lastSelection count] 
+                && ([lastSelection isEqual: baseNodeArray] == NO)) {
+          for (i = 0; i < [lastSelection count]; i++) {
+            FSNode *node = [lastSelection objectAtIndex: i];
+
+            if ([node isDirectory] && ([node isPackage] == NO)) {
+              canopen = NO;
+              break;      
+            }
+          }
         } else {
-          return YES;
-        }
-        
-      } else if ([selection isEqual: [NSArray arrayWithObject: baseNode]] == NO) {
-        if ([itemTitle isEqual: NSLocalizedString(@"Move to Recycler", @"")]) {
-          return ([[baseNode path] isEqual: [gworkspace trashPath]] == NO);
-        } else {
-          return YES;
-        }
-      }
-    }
-    
-    return NO;
-
-  } else if ([itemTitle isEqual: NSLocalizedString(@"Open", @"")]) {
-    NSArray *selection = [nodeView selectedNodes];
-    BOOL canopen = YES;
-    int i;
-    
-    if (selection && [selection count] 
-            && ([selection isEqual: [NSArray arrayWithObject: baseNode]] == NO)) {
-      for (i = 0; i < [selection count]; i++) {
-        FSNode *node = [selection objectAtIndex: i];
-      
-        if ([node isDirectory] && ([node isPackage] == NO)) {
           canopen = NO;
-          break;      
         }
-      }
-    } else {
-      canopen = NO;
-    }
-    
-    return canopen;
-    
-  } else if ([itemTitle isEqual: NSLocalizedString(@"Open as Folder", @"")]) {
-    NSArray *selection = [nodeView selectedNodes];
-    
-    if (selection && ([selection count] == 1)) {  
-      return [[selection objectAtIndex: 0] isPackage];
-    }
-    
-    return NO;
 
-  } else if ([itemTitle isEqual: NSLocalizedString(@"Open With...", @"")]) {
-    NSArray *selection = [nodeView selectedNodes];
-    BOOL canopen = YES;
-    int i;
-    
-    if (selection && [selection count]
-          && ([selection isEqual: [NSArray arrayWithObject: baseNode]] == NO)) {
-      for (i = 0; i < [selection count]; i++) {
-        FSNode *node = [selection objectAtIndex: i];
-    
-        if (([node isPlain] == NO) 
-              && (([node isPackage] == NO) || [node isApplication])) {
-          canopen = NO;
-          break;
+        return canopen;
+      }
+
+      return NO;
+
+    } else if (sel_eq(action, @selector(openSelectionAsFolder:))) {
+      if (lastSelection && ([lastSelection count] == 1)) {  
+        return [[lastSelection objectAtIndex: 0] isPackage];
+      }
+
+      return NO;
+
+    } else if (sel_eq(action, @selector(openWith:))) {
+      BOOL canopen = YES;
+      int i;
+
+      if (lastSelection && [lastSelection count]
+            && ([lastSelection isEqual: baseNodeArray] == NO)) {
+        for (i = 0; i < [lastSelection count]; i++) {
+          FSNode *node = [lastSelection objectAtIndex: i];
+
+          if (([node isPlain] == NO) 
+                && (([node isPackage] == NO) || [node isApplication])) {
+            canopen = NO;
+            break;
+          }
         }
+      } else {
+        canopen = NO;
       }
-    } else {
-      canopen = NO;
+
+      return canopen;
+
+    } else if (sel_eq(action, @selector(newFolder:))
+                                  || sel_eq(action, @selector(newFile:))) {
+      if ([[baseNode path] isEqual: [gworkspace trashPath]] == NO) {
+        return [[nodeView shownNode] isWritable];
+      }
+
+      return NO;
     }
     
-    return canopen;
-
-  } else if ([itemTitle isEqual: NSLocalizedString(@"New Folder", @"")]
-       || [itemTitle isEqual: NSLocalizedString(@"New File", @"")]) {
-    if ([[baseNode path] isEqual: [gworkspace trashPath]] == NO) {
-      return [[nodeView shownNode] isWritable];
-    }
-    
-    return NO;
-
-  } else if ([itemTitle isEqual: NSLocalizedString(@"Open", @"")]) {
-    return ([[baseNode path] isEqual: [gworkspace trashPath]] == NO);
+    return YES;   
   }
-
-  return YES;
+  
+  return NO;
 }
 
 @end
