@@ -159,15 +159,14 @@
 - (void)setOwnsScroller:(BOOL)ownscr
 {
   ownScroller = ownscr;
-  [self setFrame: [[self superview] frame]];
+  [self setFrame: [[self superview] bounds]];
   [self tile];
 }
 
 - (void)showPathComponents:(NSArray *)components
                  selection:(NSArray *)selection
 {
-  NSString *firstsel = [selection objectAtIndex: 0];
-  FSNode *node = [FSNode nodeWithPath: firstsel];
+  FSNode *node = [selection objectAtIndex: 0];
   int count = [components count];
   FSNIcon *icon;
   int icncount;
@@ -206,8 +205,7 @@
       NSMutableArray *selnodes = [NSMutableArray array];
     
       for (i = 0; i < [selection count]; i++) {
-        NSString *selpath = [selection objectAtIndex: i];
-        FSNode *selnode = [FSNode nodeWithPath: selpath];
+        FSNode *selnode = [selection objectAtIndex: i];
         [selnodes addObject: selnode];
       }
       
@@ -229,7 +227,7 @@
   int cols = range.length;
 
   if (cols != visibleIcons) {
-    [self setFrame: [[self superview] frame]];
+    [self setFrame: [[self superview] bounds]];
     visibleIcons = cols;  
   }
 
@@ -728,7 +726,6 @@
     [nameEditor setNode: nil stringValue: @"" index: -1];
     [nameEditor removeFromSuperview];
     [self setNeedsDisplayInRect: edrect];
-    [[NSCursor arrowCursor] set];
   }
 
   for (i = 0; i < [icons count]; i++) {
@@ -833,9 +830,9 @@
       NSArray *dirContents = [ednode subNodeNamesOfParent];
       NSMutableDictionary *opinfo = [NSMutableDictionary dictionary];
 
-      if (range.length > 0) {
+      if (([newname length] == 0) || (range.length > 0)) {
         NSRunAlertPanel(NSLocalizedString(@"Error", @""), 
-                  NSLocalizedString(@"Invalid char in name", @""), 
+                  NSLocalizedString(@"Invalid name", @""), 
                             NSLocalizedString(@"Continue", @""), nil, nil);   
         CLEAREDITING;
       }	
