@@ -32,6 +32,9 @@
 #include "GWorkspace.h"
 #include "GWDesktopManager.h"
 
+#define OPEN_MAX 10
+
+
 static GWViewersManager *vwrsmanager = nil;
 
 @implementation GWViewersManager
@@ -446,12 +449,7 @@ static GWViewersManager *vwrsmanager = nil;
   NSArray *watchedNodes = [aviewer watchedNodes];
   NSNumber *key = [aviewer rootViewerKey];
   id parentViewer = [self parentOfSpatialViewer: aviewer];
-  id desktopManager = [gworkspace desktopManager];  
   int i;
-
-  if ([desktopManager isActive]) {
-    [desktopManager deselectAllIcons];
-  }
   
   if (parentViewer && ([parentViewer invalidated] == NO)) {
     [parentViewer setOpened: NO repOfNode: node];
@@ -569,7 +567,13 @@ static GWViewersManager *vwrsmanager = nil;
   id parentViewer = [self parentOfSpatialViewer: aviewer];
     
   if (parentViewer) {
+    id nodeView = [parentViewer nodeView];
+
     [parentViewer setOpened: YES repOfNode: [aviewer baseNode]];
+
+    if ([nodeView respondsToSelector: @selector(scrollSelectionToVisible)]) {
+      [nodeView scrollSelectionToVisible];
+    }  
   }
 }
 
@@ -605,9 +609,23 @@ static GWViewersManager *vwrsmanager = nil;
                   closeSender:(BOOL)close
 {
   NSArray *selreps = [[viewer nodeView] selectedReps];
+  int count = [selreps count];
   int i;
     
-  for (i = 0; i < [selreps count]; i++) {
+  if (count > OPEN_MAX) {
+    NSString *msg1 = NSLocalizedString(@"Are you sure you want to open", @"");
+    NSString *msg2 = NSLocalizedString(@"items?", @"");
+
+    if (NSRunAlertPanel(nil,
+                [NSString stringWithFormat: @"%@ %i %@", msg1, count, msg2],
+                NSLocalizedString(@"Cancel", @""),
+                NSLocalizedString(@"Yes", @""),
+                nil)) {
+      return;
+    }
+  }
+    
+  for (i = 0; i < count; i++) {
     FSNode *node = [[selreps objectAtIndex: i] node];
     
     if ([node hasValidPath]) {            
@@ -938,6 +956,15 @@ static GWViewersManager *vwrsmanager = nil;
   return orderingViewers;
 }
 
+- (void)updateDesktop
+{
+  id desktopManager = [gworkspace desktopManager];  
+
+  if ([desktopManager isActive]) {
+    [desktopManager deselectAllIcons];
+  }
+}
+
 - (void)updateDefaults
 {
   NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];  
