@@ -33,6 +33,7 @@
 #define NOXTERM 1
 
 @class NSWorkspace;
+@class FSNode;
 @class FSNodeRep;
 @class GWViewersManager;
 @class GWDesktopManager;
@@ -48,7 +49,6 @@
 @class RunExternalController;
 @class StartAppWin;
 @class GWLaunchedApp;
-@class NSCursor;
 
 @protocol	FSWClientProtocol
 
@@ -93,9 +93,15 @@
 - (oneway void)setAnnotations:(NSString *)annotations
                       forPath:(NSString *)path;
 
+- (oneway void)fileSystemDidChange:(NSData *)info;
+
 @end
 
 
+@protocol	MDExtractorProtocol
+
+@end
+
 /* The protocol of the remote dnd source */
 @protocol GWRemoteFilesDraggingInfo
 - (oneway void)remoteDraggingDestinationReply:(NSData *)reply;
@@ -107,7 +113,9 @@
   FSNodeRep *fsnodeRep;
   
 	NSArray *selectedPaths;
-
+  NSMutableArray *trashContents;
+  NSString *trashPath;
+  
   id fswatcher;
   BOOL fswnotifications;
 	NSCountedSet *watchedPaths;
@@ -116,6 +124,7 @@
   BOOL recyclerCanQuit;
   
   id ddbd;
+  id mdextractor;
   
   PrefController *prefController;
   Fiend *fiend;
@@ -147,8 +156,6 @@
   NSString *defXterm;
   NSString *defXtermArgs;
   BOOL teminalService;
-
-  NSCursor *waitCursor;
           
   NSFileManager *fm;
 
@@ -312,6 +319,10 @@
 - (void)ddbdSetAnnotations:(NSString *)annotations
                    forPath:(NSString *)path;
 
+- (void)connectMDExtractor;
+
+- (void)mdextractorConnectionDidDie:(NSNotification *)notif;
+
 - (void)slideImage:(NSImage *)image 
 							from:(NSPoint)fromPoint 
 								to:(NSPoint)toPoint;
@@ -424,6 +435,13 @@
 @end
 
 
+@interface GWorkspace (SharedInspector)
+
+- (oneway void)showExternalSelection:(NSArray *)selection;
+
+@end
+
+
 @interface GWorkspace (WorkspaceApplication)
 
 - (BOOL)performFileOperation:(NSString *)operation 
@@ -559,6 +577,8 @@
 
 - (BOOL)isHidden;
 
+- (void)hideApplication;
+
 - (void)unhideApplication;
 
 - (BOOL)isApplicationHidden;
