@@ -71,6 +71,9 @@ static GWorkspace *gworkspace = nil;
   #define TSHF_MAXF 999
 #endif
 
+#define OPEN_MAX 10
+
+
 + (void)initialize
 {
   NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
@@ -111,12 +114,15 @@ static GWorkspace *gworkspace = nil;
   }
   DESTROY (recyclerApp);
   DESTROY (ddbd);
+  DESTROY (mdextractor);
 	RELEASE (gwProcessName);
 	RELEASE (gwBundlePath);
 	RELEASE (defEditor);
 	RELEASE (defXterm);
 	RELEASE (defXtermArgs);
   RELEASE (selectedPaths);
+  RELEASE (trashContents);
+  RELEASE (trashPath);
   RELEASE (watchedPaths);
   TEST_RELEASE (fiend);
 	TEST_RELEASE (history);
@@ -130,7 +136,6 @@ static GWorkspace *gworkspace = nil;
   DESTROY (inspector);
   DESTROY (fileOpsManager);
   RELEASE (finder);
-  RELEASE (waitCursor);
   RELEASE (launchedApps);
   RELEASE (storedAppinfoPath);
   RELEASE (storedAppinfoLock);
@@ -405,7 +410,9 @@ static GWorkspace *gworkspace = nil;
   [fsnodeRep setUseThumbnails: boolentry];
   
 	selectedPaths = [[NSArray alloc] initWithObjects: NSHomeDirectory(), nil];
-
+  trashContents = [NSMutableArray new];
+  ASSIGN (trashPath, [self trashPath]);
+  
   startAppWin = [[StartAppWin alloc] init];
   
   watchedPaths = [[NSCountedSet alloc] initWithCapacity: 1];
@@ -466,15 +473,17 @@ static GWorkspace *gworkspace = nil;
   
   ddbd = nil;
   [self connectDDBd];
-  
+    
+  mdextractor = nil;
+  if ([defaults boolForKey: @"GSMetadataIndexingEnabled"]) {
+    [self connectMDExtractor];
+  }
+    
 	[defaults synchronize];
   terminating = NO;
   
   [self setContextHelp];
   
-  waitCursor = [[NSCursor alloc] initWithImage: [NSImage imageNamed: @"watch.tiff"]];
-  [waitCursor setHotSpot: NSMakePoint(8, 8)];
-
   storedAppinfoPath = [NSTemporaryDirectory() stringByAppendingPathComponent: @"GSLaunchedApplications"];
   RETAIN (storedAppinfoPath); 
   lockpath = [storedAppinfoPath stringByAppendingPathExtension: @"lock"];   
@@ -486,56 +495,66 @@ static GWorkspace *gworkspace = nil;
 
 - (void)applicationDidFinishLaunching:(NSNotification *)aNotification
 {
-  [[NSDistributedNotificationCenter defaultCenter] addObserver: self 
-                				selector: @selector(fileSystemWillChange:) 
-                					  name: @"GWFileSystemWillChangeNotification"
-                					object: nil];
+  NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
+  NSNotificationCenter *dnc = [NSDistributedNotificationCenter defaultCenter];
+  
+  [nc addObserver: self 
+         selector: @selector(fileSystemWillChange:) 
+             name: @"GWFileSystemWillChangeNotification"
+           object: nil];
 
-  [[NSDistributedNotificationCenter defaultCenter] addObserver: self 
-                				selector: @selector(fileSystemDidChange:) 
-                					  name: @"GWFileSystemDidChangeNotification"
-                					object: nil];
+  [nc addObserver: self 
+         selector: @selector(fileSystemDidChange:) 
+             name: @"GWFileSystemDidChangeNotification"
+           object: nil];
 
-  [[NSDistributedNotificationCenter defaultCenter] addObserver: self 
-                				selector: @selector(changeDefaultEditor:) 
-                					  name: @"GWDefaultEditorChangedNotification"
-                					object: nil];
+  [dnc addObserver: self 
+          selector: @selector(changeDefaultEditor:) 
+              name: @"GWDefaultEditorChangedNotification"
+            object: nil];
 
-  [[NSDistributedNotificationCenter defaultCenter] addObserver: self 
-                        selector: @selector(thumbnailsDidChange:) 
-                					  name: @"GWThumbnailsDidChangeNotification"
-                					object: nil];
+  [dnc addObserver: self 
+          selector: @selector(thumbnailsDidChange:) 
+              name: @"GWThumbnailsDidChangeNotification"
+            object: nil];
 
-  [[NSDistributedNotificationCenter defaultCenter] addObserver: self 
-                        selector: @selector(removableMediaPathsDidChange:) 
-                					  name: @"GSRemovableMediaPathsDidChangeNotification"
-                					object: nil];
+  [dnc addObserver: self 
+          selector: @selector(removableMediaPathsDidChange:) 
+              name: @"GSRemovableMediaPathsDidChangeNotification"
+            object: nil];
 
-  [[NSDistributedNotificationCenter defaultCenter] addObserver: self 
-                        selector: @selector(reservedMountNamesDidChange:) 
-                					  name: @"GSReservedMountNamesDidChangeNotification"
-                					object: nil];
+  [dnc addObserver: self 
+          selector: @selector(reservedMountNamesDidChange:) 
+              name: @"GSReservedMountNamesDidChangeNotification"
+            object: nil];
+ 
+  [dnc addObserver: self 
+          selector: @selector(hideDotsFileDidChange:) 
+              name: @"GSHideDotFilesDidChangeNotification"
+            object: nil];
 
-  [[NSDistributedNotificationCenter defaultCenter] addObserver: self 
-                        selector: @selector(hideDotsFileDidChange:) 
-                					  name: @"GSHideDotFilesDidChangeNotification"
-                					object: nil];
+  [dnc addObserver: self 
+          selector: @selector(customDirectoryIconDidChange:) 
+              name: @"GWCustomDirectoryIconDidChangeNotification"
+            object: nil];
 
-  [[NSDistributedNotificationCenter defaultCenter] addObserver: self 
-                        selector: @selector(customDirectoryIconDidChange:) 
-                					  name: @"GWCustomDirectoryIconDidChangeNotification"
-                					object: nil];
-
-  [[NSDistributedNotificationCenter defaultCenter] addObserver: self 
-                        selector: @selector(applicationForExtensionsDidChange:) 
-                					  name: @"GWAppForExtensionDidChangeNotification"
-                					object: nil];
+  [dnc addObserver: self 
+          selector: @selector(applicationForExtensionsDidChange:) 
+              name: @"GWAppForExtensionDidChangeNotification"
+            object: nil];
   
   [self initializeWorkspace]; 
 }
 
+- (void)applicationDidBecomeActive:(NSNotification *)aNotification
+{
+  [self resetSelectedPaths];
+}
+
 - (BOOL)applicationShouldTerminate:(NSApplication *)app 
 {
+  NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
+  
 #define TEST_CLOSE(o, w) if ((o) && ([w isVisible])) [w close]
   
   if ([fileOpsManager operationsPending]) {
@@ -576,12 +595,12 @@ static GWorkspace *gworkspace = nil;
   TEST_CLOSE (startAppWin, [startAppWin win]);
 
   if (fswatcher) {
-    NSConnection *fswconn = [(NSDistantObject *)fswatcher connectionForProxy];
+    NSConnection *conn = [(NSDistantObject *)fswatcher connectionForProxy];
   
-    if ([fswconn isValid]) {
-      [[NSNotificationCenter defaultCenter] removeObserver: self
-	                        name: NSConnectionDidDieNotification
-	                      object: fswconn];
+    if ([conn isValid]) {
+      [nc removeObserver: self
+	                  name: NSConnectionDidDieNotification
+	                object: conn];
       [fswatcher unregisterClient: (id <FSWClientProtocol>)self];  
       DESTROY (fswatcher);
     }
@@ -592,26 +611,37 @@ static GWorkspace *gworkspace = nil;
   [finder stopAllSearchs];
   
   if (recyclerApp) {
-    NSConnection *rcconn = [(NSDistantObject *)recyclerApp connectionForProxy];
+    NSConnection *conn = [(NSDistantObject *)recyclerApp connectionForProxy];
   
-    if (rcconn && [rcconn isValid]) {
-      [[NSNotificationCenter defaultCenter] removeObserver: self
-	                        name: NSConnectionDidDieNotification
-	                      object: rcconn];
+    if (conn && [conn isValid]) {
+      [nc removeObserver: self
+	                  name: NSConnectionDidDieNotification
+	                object: conn];
       DESTROY (recyclerApp);
     }
   }
   
   if (ddbd) {
-    NSConnection *ddbdconn = [(NSDistantObject *)ddbd connectionForProxy];
+    NSConnection *conn = [(NSDistantObject *)ddbd connectionForProxy];
   
-    if (ddbdconn && [ddbdconn isValid]) {
-      [[NSNotificationCenter defaultCenter] removeObserver: self
-	                        name: NSConnectionDidDieNotification
-	                      object: ddbdconn];
+    if (conn && [conn isValid]) {
+      [nc removeObserver: self
+	                  name: NSConnectionDidDieNotification
+	                object: conn];
       DESTROY (ddbd);
     }
   }
+
+  if (mdextractor) {
+    NSConnection *conn = [(NSDistantObject *)mdextractor connectionForProxy];
+  
+    if (conn && [conn isValid]) {
+      [nc removeObserver: self
+	                  name: NSConnectionDidDieNotification
+	                object: conn];
+      DESTROY (mdextractor);
+    }
+  }
   		
 	return YES;
 }
@@ -742,7 +772,10 @@ static GWorkspace *gworkspace = nil;
 
 - (NSImage *)tshelfBackground
 {
-  return ([dtopManager isActive]) ? [dtopManager tabbedShelfBackground] : nil;
+  if ([dtopManager isActive]) {
+    return [dtopManager tabbedShelfBackground];
+  }
+  return nil;
 }
 
 - (void)tshelfBackgroundDidChange
@@ -1013,74 +1046,51 @@ static GWorkspace *gworkspace = nil;
 
 - (BOOL)validateMenuItem:(id <NSMenuItem>)anItem
 {	
-	NSString *title = [anItem title];
+  SEL action = [anItem action];
 
-	if ([title isEqual: NSLocalizedString(@"Show Recycler", @"")]) {
-    return ([dtopManager isActive] == NO);
-
-  } else if ([title isEqual: NSLocalizedString(@"Empty Recycler", @"")]) {
-    if ([dtopManager isActive] || (recyclerApp != nil)) {
-      CREATE_AUTORELEASE_POOL(arp);
-      FSNode *node = [FSNode nodeWithPath: [self trashPath]];
-      NSArray *subNodes = [node subNodes];
-      int count = [subNodes count];
-      int i;  
-          
-      for (i = 0; i < [subNodes count]; i++) {
-        if ([[subNodes objectAtIndex: i] isReserved]) {
-          count --;
-        }
-      }  
-    
-      RELEASE (arp);
-      return (count != 0);
-    }
-    
-	} else if ([title isEqual: NSLocalizedString(@"Check for disks", @"")]) {
+  if (sel_eq(action, @selector(showRecycler:))) {
+    return (([dtopManager isActive] == NO) || ([dtopManager dockActive] == NO));
+  
+  } else if (sel_eq(action, @selector(emptyRecycler:))) {
+    return ([trashContents count] != 0);
+  
+  } else if (sel_eq(action, @selector(checkRemovableMedia:))) {  
     return [dtopManager isActive];
-        
-  } else if ([title isEqual: NSLocalizedString(@"Select Special Tab", @"")]
-              || [title isEqual: NSLocalizedString(@"Remove Current Tab", @"")]
-              || [title isEqual: NSLocalizedString(@"Rename Current Tab", @"")]
-              || [title isEqual: NSLocalizedString(@"Add Tab...", @"")]) {
+  
+  } else if (sel_eq(action, @selector(removeTShelfTab:))
+              || sel_eq(action, @selector(renameTShelfTab:))
+                      || sel_eq(action, @selector(addTShelfTab:))) {
     return [tshelfWin isVisible];
 
-  } else if ([title isEqual: NSLocalizedString(@"Activate context help", @"")]) {
+  } else if (sel_eq(action, @selector(activateContextHelp:))) {
     return ([NSHelpManager isContextHelpModeActive] == NO);
 
-  } else if ([title isEqual: NSLocalizedString(@"Logout", @"")]) {
+  } else if (sel_eq(action, @selector(logout:))) {
     return !loggingout;
-  }
-  
-	if ([title isEqual: NSLocalizedString(@"Cut", @"")]
-          || [title isEqual: NSLocalizedString(@"Copy", @"")]
-          || [title isEqual: NSLocalizedString(@"Paste", @"")]) {
+    
+  } else if (sel_eq(action, @selector(cut:))
+                || sel_eq(action, @selector(copy:))
+                  || sel_eq(action, @selector(paste:))) {
     NSWindow *kwin = [NSApp keyWindow];
 
-    if (kwin) {
-      if ([kwin isKindOfClass: [TShelfWin class]]) {
-        if ([tshelfWin isVisible] == NO) {
-          return NO;
-        } else {
-          TShelfViewItem *item = [[tshelfWin shelfView] selectedTabItem];
+    if (kwin && [kwin isKindOfClass: [TShelfWin class]]) {
+      TShelfViewItem *item = [[tshelfWin shelfView] selectedTabItem];
 
-          if (item) {
-            TShelfIconsView *iview = (TShelfIconsView *)[item view];
+      if (item) {
+        TShelfIconsView *iview = (TShelfIconsView *)[item view];
 
-            if ([iview iconsType] == DATA_TAB) {
-              if ([title isEqual: NSLocalizedString(@"Paste", @"")]) {
-                return YES;
-              } else {
-                return [iview hasSelectedIcon];
-              }
-            } else {
-              return NO;
-            }
+        if ([iview iconsType] == DATA_TAB) {
+          if (sel_eq(action, @selector(paste:))) {
+            return YES;
           } else {
-            return NO;
+            return [iview hasSelectedIcon];
           }
+        } else {
+          return NO;
         }
-      } 
+      } else {
+        return NO;
+      }               
     }
   }
   
@@ -1089,16 +1099,42 @@ static GWorkspace *gworkspace = nil;
            
 - (void)fileSystemWillChange:(NSNotification *)notif
 {
-//  [[NSNotificationCenter defaultCenter]
-// 				postNotificationName: @"GWFileSystemWillChangeNotification"
-//	 								    object: [notif userInfo]];
 }
 
 - (void)fileSystemDidChange:(NSNotification *)notif
 {
-//	[[NSNotificationCenter defaultCenter]
-// 				postNotificationName: @"GWFileSystemDidChangeNotification"
-//	 								    object: [notif userInfo]];
+  NSDictionary *info = (NSDictionary *)[notif object];
+  
+  if (info) { 
+    CREATE_AUTORELEASE_POOL(arp);   
+    NSString *source = [info objectForKey: @"source"];
+    NSString *destination = [info objectForKey: @"destination"];
+  
+    if ([source isEqual: trashPath] || [destination isEqual: trashPath]) {    
+      FSNode *node = [FSNode nodeWithPath: trashPath];
+
+      [trashContents removeAllObjects];
+
+      if (node && [node isValid]) {
+        NSArray *subNodes = [node subNodes];
+        int i; 
+
+        for (i = 0; i < [subNodes count]; i++) {
+          FSNode *subnode = [subNodes objectAtIndex: i];
+
+          if ([subnode isReserved] == NO) {
+            [trashContents addObject: subnode];
+          }
+        }
+      }      
+    }
+    
+    if (ddbd != nil) {
+      [ddbd fileSystemDidChange: [NSArchiver archivedDataWithRootObject: info]];
+    }
+    
+    RELEASE (arp);
+  } 
 }
 
 - (void)setSelectedPaths:(NSArray *)paths
@@ -1140,11 +1176,25 @@ static GWorkspace *gworkspace = nil;
 
 - (void)openSelectedPaths:(NSArray *)paths newViewer:(BOOL)newv
 {
+  int count = [paths count];
   int i;
   
   [self setSelectedPaths: paths];      
+
+  if (count > OPEN_MAX) {
+    NSString *msg1 = NSLocalizedString(@"Are you sure you want to open", @"");
+    NSString *msg2 = NSLocalizedString(@"items?", @"");
   
-  for (i = 0; i < [paths count]; i++) {
+    if (NSRunAlertPanel(nil,
+                [NSString stringWithFormat: @"%@ %i %@", msg1, count, msg2],
+                NSLocalizedString(@"Cancel", @""),
+                NSLocalizedString(@"Yes", @""),
+                nil)) {
+      return;
+    }
+  }
+  
+  for (i = 0; i < count; i++) {
     NSString *apath = [paths objectAtIndex: i];
     
     if ([fm fileExistsAtPath: apath]) {
@@ -1414,7 +1464,7 @@ static GWorkspace *gworkspace = nil;
 	  }
 
     [self performFileOperation: @"NSWorkspaceRecycleOperation"
-                    source: basePath destination: [self trashPath] 
+                    source: basePath destination: trashPath 
                                               files: files tag: &tag];
   }
 }
@@ -1504,21 +1554,24 @@ static GWorkspace *gworkspace = nil;
     if ([created count]) {
       NSString *dictName = @"thumbnails.plist";
       NSString *dictPath = [thumbnailDir stringByAppendingPathComponent: dictName];
-      NSDictionary *tdict = [NSDictionary dictionaryWithContentsOfFile: dictPath];
+      
+      if ([fm fileExistsAtPath: dictPath]) {
+        NSDictionary *tdict = [NSDictionary dictionaryWithContentsOfFile: dictPath];
 
-      for (i = 0; i < [created count]; i++) {
-        NSString *key = [created objectAtIndex: i];
-        NSString *dir = [key stringByDeletingLastPathComponent];
-        NSString *tumbname = [tdict objectForKey: key];
-        NSString *tumbpath = [thumbnailDir stringByAppendingPathComponent: tumbname]; 
+        for (i = 0; i < [created count]; i++) {
+          NSString *key = [created objectAtIndex: i];
+          NSString *dir = [key stringByDeletingLastPathComponent];
+          NSString *tumbname = [tdict objectForKey: key];
+          NSString *tumbpath = [thumbnailDir stringByAppendingPathComponent: tumbname]; 
 
-        if ([fm fileExistsAtPath: tumbpath]) {        
-          if ([tmbdirs containsObject: dir] == NO) {
-            [tmbdirs addObject: dir];
+          if ([fm fileExistsAtPath: tumbpath]) {        
+            if ([tmbdirs containsObject: dir] == NO) {
+              [tmbdirs addObject: dir];
+            }
           }
         }
       }
-
+      
       [vwrsManager thumbnailsDidChangeInPaths: tmbdirs];
       [dtopManager thumbnailsDidChangeInPaths: tmbdirs];
       
@@ -1617,77 +1670,59 @@ static GWorkspace *gworkspace = nil;
 - (void)connectFSWatcher
 {
   if (fswatcher == nil) {
-    id fsw = [NSConnection rootProxyForConnectionWithRegisteredName: @"fswatcher" 
-                                                               host: @""];
+    fswatcher = [NSConnection rootProxyForConnectionWithRegisteredName: @"fswatcher" 
+                                                                  host: @""];
 
-    if (fsw) {
-      NSConnection *c = [fsw connectionForProxy];
+    if (fswatcher == nil) {
+	    NSString *cmd;
+      int i;
+    
+      cmd = [[NSSearchPathForDirectoriesInDomains(
+                GSToolsDirectory, NSSystemDomainMask, YES) objectAtIndex: 0]
+                                      stringByAppendingPathComponent: @"fswatcher"];    
+                
+      [startAppWin showWindowWithTitle: @"GWorkspace"
+                               appName: @"fswatcher"
+                             operation: NSLocalizedString(@"starting:", @"")
+                          maxProgValue: 40.0];
+    
+      [NSTask launchedTaskWithLaunchPath: cmd arguments: nil];
+   
+      for (i = 1; i <= 40; i++) {
+        [startAppWin updateProgressBy: 1.0];
+	      [[NSRunLoop currentRunLoop] runUntilDate:
+		                     [NSDate dateWithTimeIntervalSinceNow: 0.1]];
 
+        fswatcher = [NSConnection rootProxyForConnectionWithRegisteredName: @"fswatcher" 
+                                                                      host: @""];                  
+        if (fswatcher) {
+          [startAppWin updateProgressBy: 40.0 - i];
+          break;
+        }
+      }
+
+      [[startAppWin win] close];
+    }
+    
+    if (fswatcher) {
+      RETAIN (fswatcher);
+      [fswatcher setProtocolForProxy: @protocol(FSWatcherProtocol)];
+    
 	    [[NSNotificationCenter defaultCenter] addObserver: self
 	                   selector: @selector(fswatcherConnectionDidDie:)
 		                     name: NSConnectionDidDieNotification
-		                   object: c];
-      
-      fswatcher = fsw;
-	    [fswatcher setProtocolForProxy: @protocol(FSWatcherProtocol)];
-      RETAIN (fswatcher);
-                                   
+		                   object: [fswatcher connectionForProxy]];
+                       
 	    [fswatcher registerClient: (id <FSWClientProtocol>)self 
                 isGlobalWatcher: NO];
-      
-	  } else {
-	    static BOOL recursion = NO;
-	    static NSString	*cmd = nil;
-
-	    if (recursion == NO) {
-        if (cmd == nil) {
-          cmd = RETAIN ([[NSSearchPathForDirectoriesInDomains(
-                      GSToolsDirectory, NSSystemDomainMask, YES) objectAtIndex: 0]
-                            stringByAppendingPathComponent: @"fswatcher"]);
-		    }
-      }
-	  
-      if (recursion == NO && cmd != nil) {
-        int i;
-        
-        [startAppWin showWindowWithTitle: @"GWorkspace"
-                                 appName: @"fswatcher"
-                               operation: NSLocalizedString(@"starting:", @"")
-                            maxProgValue: 40.0];
-
-	      [NSTask launchedTaskWithLaunchPath: cmd arguments: nil];
-        DESTROY (cmd);
-        
-        for (i = 1; i <= 40; i++) {
-          [startAppWin updateProgressBy: 1.0];
-	        [[NSRunLoop currentRunLoop] runUntilDate:
-		                       [NSDate dateWithTimeIntervalSinceNow: 0.1]];
-                           
-          fsw = [NSConnection rootProxyForConnectionWithRegisteredName: @"fswatcher" 
-                                                                  host: @""];                  
-          if (fsw) {
-            [startAppWin updateProgressBy: 40.0 - i];
-            break;
-          }
-        }
-        
-        [[startAppWin win] close];
-        
-	      recursion = YES;
-	      [self connectFSWatcher];
-	      recursion = NO;
-        
-	    } else { 
-        DESTROY (cmd);
-	      recursion = NO;
-        fswnotifications = NO;
-        NSRunAlertPanel(nil,
-                NSLocalizedString(@"unable to contact fswatcher\nfswatcher notifications disabled!", @""),
-                NSLocalizedString(@"Ok", @""),
-                nil, 
-                nil);  
-      }
-	  }
+    } else {
+      fswnotifications = NO;
+      NSRunAlertPanel(nil,
+              NSLocalizedString(@"unable to contact fswatcher\nfswatcher notifications disabled!", @""),
+              NSLocalizedString(@"Ok", @""),
+              nil, 
+              nil);  
+    }
   }
 }
 
@@ -1739,7 +1774,32 @@ static GWorkspace *gworkspace = nil;
 {
   CREATE_AUTORELEASE_POOL(arp);
   NSDictionary *info = [NSUnarchiver unarchiveObjectWithData: dirinfo];
+  NSString *event = [info objectForKey: @"event"];
 
+  if ([event isEqual: @"GWFileDeletedInWatchedDirectory"]
+            || [event isEqual: @"GWFileCreatedInWatchedDirectory"]) {
+    NSString *path = [info objectForKey: @"path"];
+
+    if ([path isEqual: trashPath]) {
+      FSNode *node = [FSNode nodeWithPath: trashPath];
+
+      [trashContents removeAllObjects];
+
+      if (node && [node isValid]) {
+        NSArray *subNodes = [node subNodes];
+        int i; 
+
+        for (i = 0; i < [subNodes count]; i++) {
+          FSNode *subnode = [subNodes objectAtIndex: i];
+
+          if ([subnode isReserved] == NO) {
+            [trashContents addObject: subnode];
+          }
+        }
+      }
+    }
+  }
+  
 	[[NSNotificationCenter defaultCenter]
  				 postNotificationName: @"GWFileWatcherFileDidChangeNotification"
 	 								     object: info];  
@@ -1753,68 +1813,56 @@ static GWorkspace *gworkspace = nil;
 - (void)connectRecycler
 {
   if (recyclerApp == nil) {
-    id rcl = [NSConnection rootProxyForConnectionWithRegisteredName: @"Recycler" 
-                                                               host: @""];
+    recyclerApp = [NSConnection rootProxyForConnectionWithRegisteredName: @"Recycler" 
+                                                                    host: @""];
 
-    if (rcl) {
+    if (recyclerApp == nil) {
+      int i;
+    
+      [startAppWin showWindowWithTitle: @"GWorkspace"
+                               appName: @"Recycler"
+                             operation: NSLocalizedString(@"starting:", @"")
+                          maxProgValue: 80.0];
+
+      [ws launchApplication: @"Recycler"];
+   
+      for (i = 1; i <= 80; i++) {
+        [startAppWin updateProgressBy: 1.0];
+	      [[NSRunLoop currentRunLoop] runUntilDate:
+		                     [NSDate dateWithTimeIntervalSinceNow: 0.1]];
+        recyclerApp = [NSConnection rootProxyForConnectionWithRegisteredName: @"Recycler" 
+                                                                        host: @""];                  
+        if (recyclerApp) {
+          [startAppWin updateProgressBy: 80.0 - i];
+          break;
+        }
+      }
+
+      [[startAppWin win] close];
+    }
+    
+    if (recyclerApp) {
       NSMenu *menu = [[[NSApp mainMenu] itemWithTitle: NSLocalizedString(@"Tools", @"")] submenu];
       id item = [menu itemWithTitle: NSLocalizedString(@"Show Recycler", @"")];
 
-      NSConnection *c = [rcl connectionForProxy];
-
-	    [[NSNotificationCenter defaultCenter] addObserver: self
-	                   selector: @selector(recyclerConnectionDidDie:)
-		                     name: NSConnectionDidDieNotification
-		                   object: c];
-      
-      recyclerApp = rcl;
-	    [recyclerApp setProtocolForProxy: @protocol(RecyclerAppProtocol)];
-      RETAIN (recyclerApp);
-      
       if (item != nil) {
         [item setTitle: NSLocalizedString(@"Hide Recycler", @"")];
       }
-      
-	  } else {
-	    static BOOL recursion = NO;
-	  
-      if (recursion == NO) {
-        int i;
-        
-        [startAppWin showWindowWithTitle: @"GWorkspace"
-                                 appName: @"Recycler"
-                               operation: NSLocalizedString(@"starting:", @"")
-                            maxProgValue: 80.0];
-
-        [ws launchApplication: @"Recycler"];
-
-        for (i = 1; i <= 80; i++) {
-          [startAppWin updateProgressBy: 1.0];
-	        [[NSRunLoop currentRunLoop] runUntilDate:
-		                       [NSDate dateWithTimeIntervalSinceNow: 0.1]];
-          rcl = [NSConnection rootProxyForConnectionWithRegisteredName: @"Recycler" 
-                                                                  host: @""];                  
-          if (rcl) {
-            [startAppWin updateProgressBy: 80.0 - i];
-            break;
-          }
-        }
-        
-        [[startAppWin win] close];
-        
-	      recursion = YES;
-	      [self connectRecycler];
-	      recursion = NO;
-        
-	    } else { 
-	      recursion = NO;
-        NSRunAlertPanel(nil,
-                NSLocalizedString(@"unable to contact Recycler!", @""),
-                NSLocalizedString(@"Ok", @""),
-                nil, 
-                nil);  
-      }
-	  }
+    
+      RETAIN (recyclerApp);
+      [recyclerApp setProtocolForProxy: @protocol(RecyclerAppProtocol)];
+    
+	    [[NSNotificationCenter defaultCenter] addObserver: self
+	                   selector: @selector(recyclerConnectionDidDie:)
+		                     name: NSConnectionDidDieNotification
+		                   object: [recyclerApp connectionForProxy]];
+    } else {
+      NSRunAlertPanel(nil,
+              NSLocalizedString(@"unable to contact Recycler!", @""),
+              NSLocalizedString(@"Ok", @""),
+              nil, 
+              nil);  
+    }
   }
 }
 
@@ -1851,77 +1899,58 @@ static GWorkspace *gworkspace = nil;
 - (void)connectDDBd
 {
   if (ddbd == nil) {
-    id db = [NSConnection rootProxyForConnectionWithRegisteredName: @"ddbd" 
-                                                              host: @""];
+    ddbd = [NSConnection rootProxyForConnectionWithRegisteredName: @"ddbd" 
+                                                             host: @""];
 
-    if (db) {
-      NSConnection *c = [db connectionForProxy];
+    if (ddbd == nil) {
+	    NSString *cmd;
+      int i;
+    
+      cmd = [[NSSearchPathForDirectoriesInDomains(
+                GSToolsDirectory, NSSystemDomainMask, YES) objectAtIndex: 0]
+                                      stringByAppendingPathComponent: @"ddbd"];    
+                
+      [startAppWin showWindowWithTitle: @"GWorkspace"
+                               appName: @"ddbd"
+                             operation: NSLocalizedString(@"starting:", @"")
+                          maxProgValue: 40.0];
+    
+      [NSTask launchedTaskWithLaunchPath: cmd arguments: nil];
+   
+      for (i = 1; i <= 40; i++) {
+        [startAppWin updateProgressBy: 1.0];
+	      [[NSRunLoop currentRunLoop] runUntilDate:
+		                     [NSDate dateWithTimeIntervalSinceNow: 0.1]];
 
+        ddbd = [NSConnection rootProxyForConnectionWithRegisteredName: @"ddbd" 
+                                                                 host: @""];                  
+        if (ddbd) {
+          [startAppWin updateProgressBy: 40.0 - i];
+          break;
+        }
+      }
+
+      [[startAppWin win] close];
+    }
+    
+    if (ddbd) {
+      RETAIN (ddbd);
+      [ddbd setProtocolForProxy: @protocol(DDBdProtocol)];
+    
 	    [[NSNotificationCenter defaultCenter] addObserver: self
 	                   selector: @selector(ddbdConnectionDidDie:)
 		                     name: NSConnectionDidDieNotification
-		                   object: c];
-      
-      ddbd = db;
-	    [ddbd setProtocolForProxy: @protocol(DDBdProtocol)];
-      RETAIN (ddbd);
-                                         
-	  } else {
-	    static BOOL recursion = NO;
-	    static NSString	*cmd = nil;
-
-	    if (recursion == NO) {
-        if (cmd == nil) {
-            cmd = RETAIN ([[NSSearchPathForDirectoriesInDomains(
-                      GSToolsDirectory, NSSystemDomainMask, YES) objectAtIndex: 0]
-                            stringByAppendingPathComponent: @"ddbd"]);
-		    }
-      }
-	  
-      if (recursion == NO && cmd != nil) {
-        int i;
-        
-        [startAppWin showWindowWithTitle: @"GWorkspace"
-                                 appName: @"ddbd"
-                               operation: NSLocalizedString(@"starting:", @"")
-                            maxProgValue: 40.0];
-
-	      [NSTask launchedTaskWithLaunchPath: cmd arguments: nil];
-        DESTROY (cmd);
-        
-        for (i = 1; i <= 40; i++) {
-          [startAppWin updateProgressBy: 1.0];
-	        [[NSRunLoop currentRunLoop] runUntilDate:
-		                       [NSDate dateWithTimeIntervalSinceNow: 0.1]];
-                           
-          db = [NSConnection rootProxyForConnectionWithRegisteredName: @"ddbd" 
-                                                                 host: @""];                  
-          if (db) {
-            [startAppWin updateProgressBy: 40.0 - i];
-            break;
-          }
-        }
-        
-        [[startAppWin win] close];
-        
-	      recursion = YES;
-	      [self connectDDBd];
-	      recursion = NO;
-        
-	    } else { 
-        DESTROY (cmd);
-	      recursion = NO;
-        ddbd = nil;
-        NSRunAlertPanel(nil,
-                NSLocalizedString(@"unable to contact ddbd.", @""),
-                NSLocalizedString(@"Ok", @""),
-                nil, 
-                nil);  
-      }
-	  }
+		                   object: [ddbd connectionForProxy]];
+    } else {
+      NSRunAlertPanel(nil,
+              NSLocalizedString(@"unable to contact ddbd.", @""),
+              NSLocalizedString(@"Ok", @""),
+              nil, 
+              nil);  
+    }
   }
-}
-
+}  
+  
 - (void)ddbdConnectionDidDie:(NSNotification *)notif
 {
   id connection = [notif object];
@@ -1978,6 +2007,83 @@ static GWorkspace *gworkspace = nil;
   }
 }
 
+- (void)connectMDExtractor
+{
+  if (mdextractor == nil) {
+    mdextractor = [NSConnection rootProxyForConnectionWithRegisteredName: @"mdextractor" 
+                                                                    host: @""];
+
+    if (mdextractor == nil) {
+	    NSString *cmd;
+      int i;
+    
+      cmd = [[NSSearchPathForDirectoriesInDomains(
+                GSToolsDirectory, NSSystemDomainMask, YES) objectAtIndex: 0]
+                                      stringByAppendingPathComponent: @"mdextractor"];    
+                
+      [startAppWin showWindowWithTitle: @"MDIndexing"
+                               appName: @"mdextractor"
+                             operation: NSLocalizedString(@"starting:", @"")
+                          maxProgValue: 80.0];
+    
+      [NSTask launchedTaskWithLaunchPath: cmd arguments: nil];
+   
+      for (i = 1; i <= 80; i++) {
+        [startAppWin updateProgressBy: 1.0];
+	      [[NSRunLoop currentRunLoop] runUntilDate:
+		                     [NSDate dateWithTimeIntervalSinceNow: 0.1]];
+
+        mdextractor = [NSConnection rootProxyForConnectionWithRegisteredName: @"mdextractor" 
+                                                                        host: @""];                  
+        if (mdextractor) {
+          [startAppWin updateProgressBy: 80.0 - i];
+          break;
+        }
+      }
+
+      [[startAppWin win] close];
+    }
+    
+    if (mdextractor) {
+      [mdextractor setProtocolForProxy: @protocol(MDExtractorProtocol)];
+      RETAIN (mdextractor);
+    
+	    [[NSNotificationCenter defaultCenter] addObserver: self
+	                   selector: @selector(mdextractorConnectionDidDie:)
+		                     name: NSConnectionDidDieNotification
+		                   object: [mdextractor connectionForProxy]];
+    } else {
+      NSRunAlertPanel(nil,
+              NSLocalizedString(@"unable to contact mdextractor!", @""),
+              NSLocalizedString(@"Ok", @""),
+              nil, 
+              nil);  
+    }
+  }
+}
+
+- (void)mdextractorConnectionDidDie:(NSNotification *)notif
+{
+  id connection = [notif object];
+
+  [[NSNotificationCenter defaultCenter] removeObserver: self
+	              name: NSConnectionDidDieNotification
+	            object: connection];
+
+  NSAssert(connection == [mdextractor connectionForProxy],
+		                                  NSInternalInconsistencyException);
+  RELEASE (mdextractor);
+  mdextractor = nil;
+
+  if (NSRunAlertPanel(nil,
+                    NSLocalizedString(@"The mdextractor connection died.\nDo you want to restart it?", @""),
+                    NSLocalizedString(@"Yes", @""),
+                    NSLocalizedString(@"No", @""),
+                    nil)) {
+    [self connectMDExtractor];                
+  }  
+}
+
 - (void)slideImage:(NSImage *)image 
 							from:(NSPoint)fromPoint 
 								to:(NSPoint)toPoint
@@ -2044,16 +2150,16 @@ static GWorkspace *gworkspace = nil;
   [d setObject: @"GWorkspace" forKey: @"ApplicationName"];
   [d setObject: NSLocalizedString(@"GNUstep Workspace Manager", @"")
       	forKey: @"ApplicationDescription"];
-  [d setObject: @"GWorkspace 0.8.3" forKey: @"ApplicationRelease"];
-  [d setObject: @"08 2006" forKey: @"FullVersionID"];
+  [d setObject: @"GWorkspace 0.8.6" forKey: @"ApplicationRelease"];
+  [d setObject: @"06 2007" forKey: @"FullVersionID"];
   [d setObject: [NSArray arrayWithObjects: 
-                    @"Enrico Sersale <enrico@fibernet.ro>",
+                    @"Enrico Sersale <enrico.sersale@gmail.com>",
                     @"Documentation and Help contents by: \n"
                     @"Dennis Leeuw <dleeuw@made-it.com>", 
                     nil]
         forKey: @"Authors"];
   [d setObject: NSLocalizedString(@"See http://www.gnustep.it/enrico/gworkspace", @"") forKey: @"URL"];
-  [d setObject: @"Copyright (C) 2003, 2004, 2005, 2006 Free Software Foundation, Inc."
+  [d setObject: @"Copyright (C) 2003, 2004, 2005, 2006, 2007 Free Software Foundation, Inc."
         forKey: @"Copyright"];
   [d setObject: NSLocalizedString(@"Released under the GNU General Public License 2.0", @"")
         forKey: @"CopyrightDescription"];
@@ -2387,7 +2493,7 @@ static GWorkspace *gworkspace = nil;
             }  
 
             if (cutted) {
-              if ([source isEqual: [self trashPath]]) {
+              if ([source isEqual: trashPath]) {
                 operation = @"GWorkspaceRecycleOutOperation";
               } else {
 		            operation = NSWorkspaceMoveOperation;
@@ -2424,7 +2530,6 @@ static GWorkspace *gworkspace = nil;
 - (void)emptyRecycler:(id)sender
 {
   CREATE_AUTORELEASE_POOL(arp);
-  NSString *trashPath = [self trashPath];
   FSNode *node = [FSNode nodeWithPath: trashPath];
   NSMutableArray *subNodes = [[node subNodes] mutableCopy];
   int count = [subNodes count];
@@ -2467,7 +2572,7 @@ static GWorkspace *gworkspace = nil;
 - (void)selectionChanged:(NSArray *)newsel
 {
   if (newsel && [newsel count] && ([vwrsManager orderingViewers] == NO)) {
-    [self setSelectedPaths: newsel];
+    [self setSelectedPaths: [FSNode pathsOfNodes: newsel]];
   }
 }
 
@@ -2481,11 +2586,25 @@ static GWorkspace *gworkspace = nil;
 - (void)openSelectionWithApp:(id)sender
 {
   NSString *appName = (NSString *)[(NSMenuItem *)sender representedObject];
-    
-  if (selectedPaths && [selectedPaths count]) {
+  int count = (selectedPaths ? [selectedPaths count] : 0);
+  
+  if (count) {
     int i;
-    
-    for (i = 0; i < [selectedPaths count]; i++) {
+
+    if (count > OPEN_MAX) {
+      NSString *msg1 = NSLocalizedString(@"Are you sure you want to open", @"");
+      NSString *msg2 = NSLocalizedString(@"items?", @"");
+
+      if (NSRunAlertPanel(nil,
+                  [NSString stringWithFormat: @"%@ %i %@", msg1, count, msg2],
+                  NSLocalizedString(@"Cancel", @""),
+                  NSLocalizedString(@"Yes", @""),
+                  nil)) {
+        return;
+      }
+    }
+
+    for (i = 0; i < count; i++) {
       NSString *path = [selectedPaths objectAtIndex: i];
     
       NS_DURING
@@ -2631,3 +2750,23 @@ static GWorkspace *gworkspace = nil;
 
 @end
 
+
+@implementation GWorkspace (SharedInspector)
+
+- (oneway void)showExternalSelection:(NSArray *)selection
+{
+  if ([[inspector win] isVisible] == NO) {
+    [self showContentsInspector: nil];    
+  }  
+  
+  if (selection) {
+    [inspector setCurrentSelection: selection];
+  } else {
+    [self resetSelectedPaths];
+  }
+}
+
+@end
+
+
+
