@@ -19,26 +19,22 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
 #include <Foundation/Foundation.h>
 #include <AppKit/AppKit.h>
-  #ifdef GNUSTEP 
-#include "GWLib.h"
-#include "GWFunctions.h"
-#include "GWNotifications.h"
-  #else
-#include <GWorkspace/GWLib.h>
-#include <GWorkspace/GWFunctions.h>
-#include <GWorkspace/GWNotifications.h>
-  #endif
+#include "FSNodeRep.h"
+#include "FSNFunctions.h"
 #include "HiddenFilesPref.h"
 #include "GWorkspace.h"
 #include "GNUstep.h"
 
 static NSString *nibName = @"HiddenFilesPref";
 
+#define ICON_SIZE 48
+#define LINEH 20
+
 #define CHECKSIZE(sz) \
 if (sz.width < 0) sz.width = 0; \
 if (sz.height < 0) sz.height = 0
@@ -49,7 +45,7 @@ if (sz.height < 0) sz.height = 0
 {
 	[[NSNotificationCenter defaultCenter] removeObserver: self];
   TEST_RELEASE (prefbox);
-  RELEASE (currentPath);
+  RELEASE (currentNode);
   RELEASE (hiddenPaths);
   TEST_RELEASE (leftMatrix);
   TEST_RELEASE (rightMatrix); 
@@ -68,11 +64,9 @@ if (sz.height < 0) sz.height = 0
     } else {
       NSArray *hpaths;
       NSArray *selection;
-      NSString *path;  
-      NSString *defApp;
-      NSString *type;
-      BOOL isdir;
-
+      FSNode *node;
+      NSImage *icon;
+      
       RETAIN (prefbox);
       RELEASE (win); 
 
@@ -81,26 +75,26 @@ if (sz.height < 0) sz.height = 0
       ws = [NSWorkspace sharedWorkspace];
 
       selection = [gw selectedPaths];
-      path = [selection objectAtIndex: 0];
+      node = [FSNode nodeWithPath: [selection objectAtIndex: 0]];
 
       if ([selection count] > 1) {
-        path = [path stringByDeletingLastPathComponent];     
+        node = [FSNode nodeWithPath: [node parentPath]];
       } else {
-        [fm fileExistsAtPath: path isDirectory: &isdir];    
-        if (isdir == NO) {
-          path = [path stringByDeletingLastPathComponent];     
-        } else if ([GWLib isPakageAtPath: path]) {
-          path = [path stringByDeletingLastPathComponent];         
+        if ([node isDirectory] == NO) {
+          node = [FSNode nodeWithPath: [node parentPath]];
+        } else if ([node isPackage]) {
+          node = [FSNode nodeWithPath: [node parentPath]];
         }
       }
 
-      ASSIGN (currentPath, path);
+      ASSIGN (currentNode, node);
 
-      [ws getInfoForFile: currentPath application: &defApp type: &type];      
-      [iconView setImage: [GWLib iconForFile: currentPath ofType: type]]; 		
+      icon = [[FSNodeRep sharedInstance] iconOfSize: ICON_SIZE 
+                                            forNode: currentNode];
+      [iconView setImage: icon]; 		
       
       cellPrototipe = [NSBrowserCell new];
-
+      
       [leftScroll setBorderType: NSBezelBorder];
 		  [leftScroll setHasHorizontalScroller: NO];
   	  [leftScroll setHasVerticalScroller: YES]; 
@@ -119,7 +113,7 @@ if (sz.height < 0) sz.height = 0
 
       [[NSNotificationCenter defaultCenter] addObserver: self 
                 					  selector: @selector(selectionChanged:) 
-                						    name: GWCurrentSelectionChangedNotification
+                						    name: @"GWCurrentSelectionChangedNotification"
                 					    object: nil];
                               
       [hiddenDirsScroll setBorderType: NSBezelBorder];
@@ -127,7 +121,7 @@ if (sz.height < 0) sz.height = 0
   	  [hiddenDirsScroll setHasVerticalScroller: YES]; 
                     
       hiddenPaths = [NSMutableArray new];
-      hpaths = [GWLib hiddenPaths];
+      hpaths = [[FSNodeRep sharedInstance] hiddenPaths];
       
       if ([hpaths count]) { 
         NSSize cs, ms;
@@ -139,7 +133,7 @@ if (sz.height < 0) sz.height = 0
 				            			        mode: NSListModeMatrix prototype: cellPrototipe
 			       											              numberOfRows: 0 numberOfColumns: 0];
         [dirsMatrix setIntercellSpacing: NSZeroSize];
-        [dirsMatrix setCellSize: NSMakeSize(130, 14)];
+        [dirsMatrix setCellSize: NSMakeSize(130, LINEH)];
         [dirsMatrix setAutoscroll: YES];
 	      [dirsMatrix setAllowsEmptySelection: YES];
         cs = [hiddenDirsScroll contentSize];
@@ -168,15 +162,28 @@ if (sz.height < 0) sz.height = 0
       [setDirButt setEnabled: NO];
                                                             
       /* Internationalization */
-      [setButt setTitle: NSLocalizedString(@"Activate changes", @"")];
-      [loadButt setTitle: NSLocalizedString(@"Load", @"")];
-      [hiddenlabel setStringValue: NSLocalizedString(@"Hidden files", @"")];
-      [shownlabel setStringValue: NSLocalizedString(@"Shown files", @"")];
-      [labelinfo setStringValue: NSLocalizedString(@"Select and move the files to hide or to show", @"")];
-      [hiddenDirslabel setStringValue: NSLocalizedString(@"Hidden directories", @"")];
-      [addDirButt setTitle: NSLocalizedString(@"add", @"")];
-      [removeDirButt setTitle: NSLocalizedString(@"remove", @"")];
-      [setDirButt setTitle: NSLocalizedString(@"Activate changes", @"")];
+      {
+        int tabindex;
+        NSTabViewItem *item;
+        
+        tabindex = [tabView indexOfTabViewItemWithIdentifier: @"Files"];
+        item = [tabView tabViewItemAtIndex: tabindex];
+        [item setLabel: NSLocalizedString(@"Files", @"")];
+
+        tabindex = [tabView indexOfTabViewItemWithIdentifier: @"Folders"];
+        item = [tabView tabViewItemAtIndex: tabindex];
+        [item setLabel: NSLocalizedString(@"Folders", @"")];
+
+        [setButt setTitle: NSLocalizedString(@"Activate changes", @"")];
+        [loadButt setTitle: NSLocalizedString(@"Load", @"")];
+        [hiddenlabel setStringValue: NSLocalizedString(@"Hidden files", @"")];
+        [shownlabel setStringValue: NSLocalizedString(@"Shown files", @"")];
+        [labelinfo setStringValue: NSLocalizedString(@"Select and move the files to hide or to show", @"")];
+        [hiddenDirslabel setStringValue: NSLocalizedString(@"Hidden directories", @"")];
+        [addDirButt setTitle: NSLocalizedString(@"add", @"")];
+        [removeDirButt setTitle: NSLocalizedString(@"remove", @"")];
+        [setDirButt setTitle: NSLocalizedString(@"Activate changes", @"")];
+      }
     }
   }
   
@@ -196,31 +203,29 @@ if (sz.height < 0) sz.height = 0
 - (void)selectionChanged:(NSNotification *)n
 {
   NSArray *selection;
-  NSString *path;  
-  NSString *defApp;
-  NSString *type;
-  BOOL isdir;
+  FSNode *node;
+  NSImage *icon;
 
   selection = [gw selectedPaths];
-  path = [selection objectAtIndex: 0];
+  node = [FSNode nodeWithPath: [selection objectAtIndex: 0]];
 
   if ([selection count] > 1) {
-    path = [path stringByDeletingLastPathComponent];     
+    node = [FSNode nodeWithPath: [node parentPath]];
   } else {
-    [fm fileExistsAtPath: path isDirectory: &isdir];    
-    if (isdir == NO) {
-      path = [path stringByDeletingLastPathComponent];     
-    } else if ([GWLib isPakageAtPath: path]) {
-      path = [path stringByDeletingLastPathComponent];         
+    if ([node isDirectory] == NO) {
+      node = [FSNode nodeWithPath: [node parentPath]];
+    } else if ([node isPackage]) {
+      node = [FSNode nodeWithPath: [node parentPath]];
     }
   }
 
-  ASSIGN (currentPath, path);
+  ASSIGN (currentNode, node);
 
-  [ws getInfoForFile: currentPath application: &defApp type: &type];      
-  [iconView setImage: [GWLib iconForFile: currentPath ofType: type]]; 		
+  icon = [[FSNodeRep sharedInstance] iconOfSize: ICON_SIZE 
+                                            forNode: currentNode];
+  [iconView setImage: icon]; 		
 
-  [pathField setStringValue: [currentPath lastPathComponent]]; 
+  [pathField setStringValue: [currentNode name]]; 
 
   [self clearAll];
 
@@ -241,7 +246,7 @@ if (sz.height < 0) sz.height = 0
 				            			    mode: NSListModeMatrix prototype: cellPrototipe
 			       											          numberOfRows: 0 numberOfColumns: 0];
   [leftMatrix setIntercellSpacing: NSZeroSize];
-  [leftMatrix setCellSize: NSMakeSize(130, 14)];
+  [leftMatrix setCellSize: NSMakeSize(130, LINEH)];
   [leftMatrix setAutoscroll: YES];
 	[leftMatrix setAllowsEmptySelection: YES];
   cs = [leftScroll contentSize];
@@ -261,7 +266,7 @@ if (sz.height < 0) sz.height = 0
 				            			    mode: NSListModeMatrix prototype: cellPrototipe
 			       											          numberOfRows: 0 numberOfColumns: 0];
   [rightMatrix setIntercellSpacing: NSZeroSize];
-  [rightMatrix setCellSize: NSMakeSize(130, 14)];
+  [rightMatrix setCellSize: NSMakeSize(130, LINEH)];
   [rightMatrix setAutoscroll: YES];
 	[rightMatrix setAllowsEmptySelection: YES];
   cs = [rightScroll contentSize];
@@ -276,7 +281,7 @@ if (sz.height < 0) sz.height = 0
 
 - (IBAction)loadContents:(id)sender
 {
-  NSArray *files;
+  NSArray *subNodes;
   NSMutableArray *hiddenFiles;
 	BOOL hideSysFiles;
   NSString *h; 
@@ -284,9 +289,10 @@ if (sz.height < 0) sz.height = 0
 
   [self clearAll];
 	
-  files = [fm directoryContentsAtPath: currentPath];
+  subNodes = [currentNode subNodes];
 
-	h = [currentPath stringByAppendingPathComponent: @".hidden"];
+	h = [[currentNode path] stringByAppendingPathComponent: @".hidden"];
+  
   if ([fm fileExistsAtPath: h]) {
 	  h = [NSString stringWithContentsOfFile: h];
 	  hiddenFiles = [[h componentsSeparatedByString: @"\n"] mutableCopy];
@@ -305,32 +311,42 @@ if (sz.height < 0) sz.height = 0
   } else {
     hiddenFiles = nil;
   }
-	hideSysFiles = [GWLib hideSysFiles];
+	hideSysFiles = [[FSNodeRep sharedInstance] hideSysFiles];
 	
 	if (hiddenFiles != nil  ||  hideSysFiles) {	
-		NSMutableArray *mutableFiles = AUTORELEASE ([files mutableCopy]);
-	
-		if (hiddenFiles != nil) {
-	    [mutableFiles removeObjectsInArray: hiddenFiles];
-	  }
-	
+		NSMutableArray *mutableNodes = AUTORELEASE ([subNodes mutableCopy]);
+    
+    if (hiddenFiles) {
+      int count = [mutableNodes count];
+      
+      for (i = 0; i < count; i++) {
+        FSNode *node = [mutableNodes objectAtIndex: i];
+        
+        if ([hiddenFiles containsObject: [node name]]) {
+          [mutableNodes removeObject: node];
+          count--;
+          i--;
+        }
+      }
+    }
+    
 		if (hideSysFiles) {
-	    int j = [mutableFiles count] - 1;
+	    int j = [mutableNodes count] - 1;
 	    
 	    while (j >= 0) {
-				NSString *file = (NSString *)[mutableFiles objectAtIndex: j];
+				NSString *file = [(FSNode *)[mutableNodes objectAtIndex: j] name];
 
 				if ([file hasPrefix: @"."]) {
-		    	[mutableFiles removeObjectAtIndex: j];
+		    	[mutableNodes removeObjectAtIndex: j];
 		  	}
 				j--;
 			}
 	  }
 		
-		files = mutableFiles;
+		subNodes = mutableNodes;
 	}
 
-  count = [files count];
+  count = [subNodes count];
   if (count == 0) {
     TEST_RELEASE (hiddenFiles);
 		return;
@@ -344,7 +360,7 @@ if (sz.height < 0) sz.height = 0
 		  [rightMatrix insertRow: i];
     }     
     cell = [rightMatrix cellAtRow: i column: 0];   
-    [cell setStringValue: [files objectAtIndex: i]];
+    [cell setStringValue: [(FSNode *)[subNodes objectAtIndex: i] name]];
     [cell setLeaf: YES];
   }  
   [rightMatrix sizeToCells]; 
@@ -415,9 +431,9 @@ if (sz.height < 0) sz.height = 0
 
 - (IBAction)activateChanges:(id)sender
 {
-  if ([fm isWritableFileAtPath: currentPath] == NO) {
+  if ([currentNode isWritable] == NO) {
     NSString *message = @"You have not write access to ";
-    message = [message stringByAppendingString: [currentPath lastPathComponent]]; 
+    message = [message stringByAppendingString: [currentNode name]]; 
     
     NSRunAlertPanel(NSLocalizedString(@"error", @""), 
                         NSLocalizedString(message, @""), 
@@ -425,6 +441,8 @@ if (sz.height < 0) sz.height = 0
                         nil, nil);
     return;
   } else {
+    NSString *base = [currentNode path];
+    NSMutableArray *paths = [NSMutableArray array];
     NSArray *cells = [leftMatrix cells];
     
     if (cells) {	
@@ -435,14 +453,17 @@ if (sz.height < 0) sz.height = 0
      
       names = [NSMutableArray arrayWithCapacity: 1];    
       for (i = 0; i < [cells count]; i++) {
-        id cell = [cells objectAtIndex: i];      
-        [names addObject: [cell stringValue]];
+        id cell = [cells objectAtIndex: i]; 
+        NSString *name = [cell stringValue];
+        
+        [names addObject: name];
+        [paths addObject: [base stringByAppendingPathComponent: name]];
       }
       
       hconts = [names componentsJoinedByString: @"\n"];      
-      h = [currentPath stringByAppendingPathComponent: @".hidden"];
+      h = [[currentNode path] stringByAppendingPathComponent: @".hidden"];
       [hconts writeToFile: h atomically: YES];
-      
+      [gw hiddenFilesDidChange: paths];
       [setButt setEnabled: NO];
     }
   }
@@ -460,7 +481,9 @@ if (sz.height < 0) sz.height = 0
   [openPanel setCanChooseFiles: NO];
   [openPanel setCanChooseDirectories: YES];
 
-  result = [openPanel runModalForDirectory: fixPath(@"/", 0) file: nil types: nil];
+  result = [openPanel runModalForDirectory: path_separator() 
+                                      file: nil 
+                                     types: nil];
 	if(result != NSOKButton) {
 		return;
   }
@@ -483,7 +506,7 @@ if (sz.height < 0) sz.height = 0
 				            			    mode: NSListModeMatrix prototype: cellPrototipe
 			       											          numberOfRows: 0 numberOfColumns: 0];
     [dirsMatrix setIntercellSpacing: NSZeroSize];
-    [dirsMatrix setCellSize: NSMakeSize(130, 14)];
+    [dirsMatrix setCellSize: NSMakeSize(130, LINEH)];
     [dirsMatrix setAutoscroll: YES];
 	  [dirsMatrix setAllowsEmptySelection: YES];
     cs = [hiddenDirsScroll contentSize];
@@ -531,8 +554,8 @@ if (sz.height < 0) sz.height = 0
 
 - (IBAction)activateDirChanges:(id)sender
 {
-  [GWLib setHiddenPaths: hiddenPaths];
-  [gw checkViewersAfterHidingOfPaths: hiddenPaths];
+  [[FSNodeRep sharedInstance] setHiddenPaths: hiddenPaths];
+  [gw hiddenFilesDidChange: hiddenPaths];
   [setDirButt setEnabled: NO];
 }
 
