@@ -60,8 +60,9 @@
     int i;
     
 		[self setReleasedWhenClosed: NO];
+    [self setExcludedFromWindowsMenu: YES];
         
-    tView = [[TShelfView alloc] initWithFrame: [[self contentView] frame]];
+    tView = [[TShelfView alloc] initWithFrame: [[self contentView] bounds]];
     [self setContentView: tView];		
     
     defaults = [NSUserDefaults standardUserDefaults];	
