@@ -132,7 +132,10 @@
 
 - (NSDictionary *)activeApplication
 {
-  return ((activeApplication != nil) ? [activeApplication appInfo] : nil);
+  if (activeApplication != nil) {
+    return [activeApplication appInfo];
+  }
+  return nil;
 }
 
 - (BOOL)openFile:(NSString *)fullPath
@@ -157,9 +160,8 @@
     
     appname = [ws getBestAppInRole: nil forExtension: ext];
     
-    if (appName == nil) {
-      NSWarnLog(@"No known applications for file extension '%@'", ext);
-      return NO;
+    if (appname == nil) {
+      appname = defEditor;      
     }
   }
 
@@ -583,7 +585,8 @@
 {
   GWLaunchedApp *app = [self launchedAppWithPath: path andName: name];
 
-  if (app && ([app isActive] == NO)) {
+//  if (app && ([app isActive] == NO)) {
+  if (app) {
     [app activateApplication];
   }
 }
@@ -715,7 +718,11 @@
   
   apps = [runningInfo objectForKey: @"GSLaunched"];
   
-  return ((apps != nil) ? [apps allValues] : nil);
+  if (apps != nil) {
+    return [apps allValues];
+  }
+  
+  return nil;
 }
 
 - (void)updateStoredAppInfoWithLaunchedApps:(NSArray *)apps
@@ -1009,10 +1016,9 @@
 
 - (void)dealloc
 {
+  [nc removeObserver: self];
+
   if (conn && [conn isValid]) {
-    [nc removeObserver: self
-	                name: NSConnectionDidDieNotification
-	              object: conn];
     DESTROY (application);  
     RELEASE (conn);  
   }
@@ -1160,6 +1166,21 @@
   return hidden;
 }
 
+- (void)hideApplication
+{
+  NS_DURING
+    {
+  [application hide: nil];
+    }
+  NS_HANDLER
+    {
+  NSLog(@"Unable to hide %@", name);
+  NSLog(@"GWorkspace caught exception %@: %@", 
+	        [localException name], [localException reason]);
+    }
+  NS_ENDHANDLER
+}    
+
 - (void)unhideApplication
 {
   NS_DURING
@@ -1210,7 +1231,7 @@
   if (application) {
     NS_DURING
       {
-    [application terminate: nil];      
+    [application terminate: nil];
       }
     NS_HANDLER
       {
