@@ -41,6 +41,7 @@
   }
 
   TEST_RELEASE (imagePath);	
+  TEST_RELEASE (image);	
   TEST_RELEASE (nextPath);	
   TEST_RELEASE (editPath);	
   RELEASE (extsarr);
@@ -57,7 +58,7 @@
   self = [super initWithFrame: frameRect];
   
   if(self) {
-    NSRect r = [self frame];
+    NSRect r = [self bounds];
     
     r.origin.y += 60;
     r.size.height -= 60;
@@ -95,7 +96,7 @@
 
     r.origin.x = 2;
     r.origin.y = 170;
-    r.size.width = [self frame].size.width - 4;
+    r.size.width = [self bounds].size.width - 4;
     r.size.height = 25;
     errLabel = [[NSTextField alloc] initWithFrame: r];	
     [errLabel setFont: [NSFont systemFontOfSize: 18]];
@@ -143,6 +144,7 @@
     imagePath = nil;
     nextPath = nil;
     editPath = nil;
+    image = nil;
     
     [self setContextHelp];
   }
@@ -193,7 +195,7 @@
     [NSTask launchedTaskWithLaunchPath: cmd 
                              arguments: [NSArray arrayWithObject: cname]];
   } else {
-    NSSize imsize = [imview frame].size;
+    NSSize imsize = [imview bounds].size;
     
     imsize.width -= 4;
     imsize.height -= 4;
@@ -209,6 +211,7 @@
     if (forced) {
       [self displayPath: editPath];
     } else {
+      [imview setImage: image];
       [inspector contentsReadyAt: editPath];
     }
   }
@@ -228,7 +231,7 @@
 - (void)setResizer:(id)anObject
 {
   if (resizer == nil) {
-    NSSize imsize = [imview frame].size;
+    NSSize imsize = [imview bounds].size;
     
     imsize.width -= 4;
     imsize.height -= 4;
@@ -302,7 +305,8 @@
   }
         
   if (imgdata) {
-    NSImage *image = [[NSImage alloc] initWithData: imgdata];
+    DESTROY (image);
+    image = [[NSImage alloc] initWithData: imgdata];
     
     if (image) {
       float width = [[imginfo objectForKey: @"width"] floatValue];
@@ -316,7 +320,6 @@
       }
 
       [imview setImage: image];
-      RELEASE (image);
 
       str = NSLocalizedString(@"Width:", @"");
       str = [NSString stringWithFormat: @"%@ %.0f", str, width];
@@ -374,6 +377,7 @@
 
 - (void)stopTasks
 {
+  [imview setImage: nil];
 }
 
 - (BOOL)canDisplayPath:(NSString *)path
