@@ -5,7 +5,7 @@
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: January 2004
  *
- * This file is part of the GNUstep Inspector application
+ * This file is part of the GNUstep GWorkspace application
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
 #ifndef CONTENTVIEWERSPROTOCOL_H
@@ -30,22 +30,6 @@
 - (id)initWithFrame:(NSRect)frameRect
           inspector:(id)insp;
 
-- (void)setBundlePath:(NSString *)path;
-
-- (NSString *)bundlePath;
-
-- (void)setDataRepresentation:(NSData *)rep;
-
-- (NSData *)dataRepresentation;
-
-- (void)setIsRemovable:(BOOL)value;
-
-- (BOOL)isRemovable;
-
-- (void)setIsExternal:(BOOL)value;
-
-- (BOOL)isExternal;
-
 - (void)displayPath:(NSString *)path;
 
 - (void)displayLastPath:(BOOL)forced;
