@@ -85,7 +85,7 @@ static NSString *nibName = @"Contents";
     imagepath = [bundle pathForResource: @"Pboard" ofType: @"tiff"];
     pboardImage = [[NSImage alloc] initWithContentsOfFile: imagepath]; 
         
-    r = [[viewersBox contentView] frame];
+    r = [[viewersBox contentView] bounds];
 
     bundlesDir = [NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSSystemDomainMask, YES) lastObject];
     bundlesDir = [bundlesDir stringByAppendingPathComponent: @"Bundles"];
@@ -167,14 +167,10 @@ static NSString *nibName = @"Contents";
 {
 	int i;
   
-  if (path == nil) {
+  if ((path == nil) || ([fm fileExistsAtPath: path] == NO)) {
     return nil;
   }
-  
-  if ([fm fileExistsAtPath: path] == NO) {
-    return nil;
-  }
-  
+    
 	for (i = 0; i < [viewers count]; i++) {
 		id vwr = [viewers objectAtIndex: i];		
 		if ([vwr canDisplayPath: path]) {
@@ -207,7 +203,7 @@ static NSString *nibName = @"Contents";
 	NSString *winName;
 
   if (currentViewer) {
-    if ([currentViewer conformsToProtocol: @protocol(ContentViewersProtocol)]) {
+    if ([currentViewer respondsToSelector: @selector(stopTasks)]) {
       [currentViewer stopTasks];  
     }
   }   
@@ -240,8 +236,12 @@ static NSString *nibName = @"Contents";
       if ([textViewer tryToDisplayPath: path]) {
         [viewersBox setContentView: textViewer];
         currentViewer = textViewer;
-			  winName = NSLocalizedString(@"Text Inspector", @"");
-      
+			  winName = NSLocalizedString(@"Text Inspector", @"");      
+        if (currentPath == nil) {
+          ASSIGN (currentPath, path); 
+          [inspector addWatcherForPath: currentPath];
+        }
+        
       } else {
         [viewersBox setContentView: genericView];
         currentViewer = genericView;
@@ -292,7 +292,7 @@ static NSString *nibName = @"Contents";
 	id viewer;
 
   if (currentViewer) {
-    if ([currentViewer conformsToProtocol: @protocol(ContentViewersProtocol)]) {
+    if ([currentViewer respondsToSelector: @selector(stopTasks)]) {
       [currentViewer stopTasks]; 
     }
   }   
@@ -344,12 +344,11 @@ static NSString *nibName = @"Contents";
       [self showContentsAt: nil];
 
     } else if ([event isEqual: @"GWWatchedFileModified"]) {
-      if (currentViewer 
-              && [currentViewer conformsToProtocol: @protocol(ContentViewersProtocol)]) {
-        if ([currentPath isEqual: [currentViewer currentPath]]) {
-          [inspector removeWatcherForPath: currentPath];
-          [currentViewer displayLastPath: YES];
-          [inspector addWatcherForPath: currentPath];
+      if (currentViewer) {
+        if ([currentViewer respondsToSelector: @selector(displayPath:)]) {
+          [currentViewer displayPath: currentPath];
+        } else if (currentViewer == textViewer) {
+          [currentViewer tryToDisplayPath: currentPath];
         }
       }
     }
@@ -378,7 +377,7 @@ static NSString *nibName = @"Contents";
   self = [super initWithFrame: frameRect];
   
   if (self) {
-    NSRect r = [self frame];
+    NSRect r = [self bounds];
         
     r.origin.y += 45;
     r.size.height -= 45;
@@ -392,7 +391,7 @@ static NSString *nibName = @"Contents";
     [self addSubview: scrollView]; 
     RELEASE (scrollView);
     
-    r = [[scrollView contentView] frame];
+    r = [[scrollView contentView] bounds];
     textView = [[NSTextView alloc] initWithFrame: r];
     [textView setBackgroundColor: [NSColor whiteColor]];
     [textView setRichText: YES];
@@ -559,7 +558,7 @@ static NSString *nibName = @"Contents";
     
     r = NSMakeRect(0, 60, frameRect.size.width, 140);
     textview = [[NSTextView alloc] initWithFrame: r];
-    [[textview textContainer] setContainerSize: [textview frame].size];
+    [[textview textContainer] setContainerSize: [textview bounds].size];
 		[textview setDrawsBackground: NO];
     [textview setRichText: NO];
     [textview setSelectable: NO];
