@@ -5,7 +5,7 @@
  * Author: Enrico Sersale <enrico@imago.ro>
  * Date: January 2004
  *
- * This file is part of the GNUstep Inspector application
+ * This file is part of the GNUstep GWorkspace application
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
 #ifndef TIMEDATEVIEW_H
@@ -30,6 +30,7 @@
 
 @class NSCalendarDate;
 @class NSImage;
+@class NSTextFieldCell;
 
 @interface TimeDateView : NSView
 {
@@ -39,6 +40,7 @@
 	NSImage *dayweekImage;
 	NSImage *daymont1Image, *daymont2Image;
 	NSImage *monthImage;
+  NSTextFieldCell *yearlabel;
 }
 
 - (void)setDate:(NSCalendarDate *)adate;
